enum BlockType/////////////////////////////////////////
{
	UNKNOWN_BLOCK=0, FUNC_BLOCK, CLASS_BLOCK, ENUM_BLOCK, NORMAL_BLOCK, LOOP_BLOCK, SWITCH_BLOCK, BRANCH_BLOCK, LOCK_BLOCK, DELEGATE_BLOCK
};

const intptr_t TNT_OFFSET = 1 + (sizeof(intptr_t) + (sizeof(wchar_t)-1)) / sizeof(wchar_t);


template<typename TREE> class SIASTreeNodeItem : public SIAItem
{
	TREE *pt;
public:
	SIASTreeNodeItem(const _TCHAR *const la) : SIAItem(la)
	{
		pt = NULL;
	}
	TREE* get()
	{
		return pt;
	}
	TREE* set(TREE *const &x)
	{
		return pt = x;
	}
};

class SIASDLLItem : public SIAItem
{
	HINSTANCE hin;
public:
	SIASDLLItem(const _TCHAR *const la) : SIAItem(la)
	{
		hin = LoadLibrary(la);
	}
	~SIASDLLItem()
	{
		FreeLibrary(hin);
	}
	FARPROC getfunc(const _TCHAR *const fn)
	{
		int i, l;
		char *s;
		FARPROC fpro;
		l = lstrlen(fn);
		s = new char[l+1];
		for(i = 0; i < l; i++) s[i] = (char)fn[i];
		s[i] = '\0';
		fpro = GetProcAddress(hin, s);
		delete [] s;
		return fpro;
	}
};

struct SwitchExtStruct
{
	int32_t swtype;
	bool teeburutukutta;
	intptr_t def;
	PWCArray<intptr_t> defbp;
	PWCArray<int64_t> atai;
	PWCArray<intptr_t> ataibp;
	PWCArray<intptr_t> tgnsuruusakibp;
	SwitchExtStruct()
	{
		teeburutukutta = false;
		def = 0;
	}
};

struct BranchExtStruct
{
	int mode;
	bool commnai;
	int8_t bunkisita;
	intptr_t nextcond;
	int32_t dino;
	PWCArrayQuad< PWCArray<wchar_t> > label;
	PWCArray<intptr_t> labelbp;
	PWCArray<intptr_t> tgnelsesakibp;
	PWCArray<intptr_t> tgncommsakibp;
	BranchExtStruct()
	{
		mode = 0;
		nextcond = 0;
		dino = 0;
		commnai = false;
		bunkisita = 0;
	}
	bool addlabel(const wchar_t *const str, const intptr_t len)
	{
		intptr_t i;
		label[label.GetLength()].Clear();
		label.Tail().AddTail(str, len);
		for(i = 0; i < label.GetLength()-1; i++){
			if(label.GetTail() == label.GetConst()[i]){
				label.Pop();
				return false;
			}
		}
		return true;
	}
	int32_t findlabel(const wchar_t *const str, const intptr_t len)
	{
		intptr_t i;
		PWCArray<wchar_t> tmp;
		tmp.AddTail(str, len);
		for(i = 0; i < label.GetLength(); i++){
			if(tmp == label.GetConst()[i]) return (int32_t)i;
		}
		return -1;
	}
};

struct LoopExtStruct
{
	int mode;
	intptr_t doad;
	PWCArray<intptr_t> doadbp;
	intptr_t contad;
	PWCArray<intptr_t> contadbp;
	intptr_t nextad;
	PWCArray<intptr_t> nextadbp;
	LoopExtStruct()
	{
		mode = 0;
		doad = contad = nextad = 0;
	}
};

template<typename FTree> class Functions
{
	int32_t totalid;
	PWCArray<FTree*> functree;
public:
	Functions()
	{
		totalid = 0;
	}
	void Clear()
	{
		totalid = 0;
	}
	int32_t Add(FTree *const tree)
	{
		int32_t id;
		functree.SetLength(totalid);
		functree += tree;
		id = totalid++;
		return id;
	}
	FTree* Get(const int32_t id)
	{
		return functree.GetConst()[id];
	}
	int32_t GetTotal()
	{
		return totalid;
	}
};

template<typename FTree> struct Hensuu
{
	PWCArray<intptr_t> type;
	intptr_t size;
	FTree *tree;
	Hensuu & operator = (const Hensuu &x)
	{
		type = x.type;
		size = x.size;
		tree = x.tree;
		return *this;
	}
};

template<typename STree, typename GC> struct STStatic
{
	SSZStatic<GC> *ss;
	Functions<STree> funclist;
	StrIdxArray< SIASTreeNodeItem<STree> > srclist;
	StrIdxArray<SIASDLLItem> dlllist;
	PWCArrayQuad< PWCArray<int8_t> > literallist;
	intptr_t literaltotal;
	bool tokubetuerr;
	STStatic(SSZStatic<GC> *sstat)
	{
		ss = sstat;
		literaltotal = 0;
	}
};

template<typename GC> struct SourceTree
{
	typedef Source<GC> Src;
	STStatic<SourceTree, GC> *stat;
	PWCArray<wchar_t> myname;
	Src *srce;
	SourceTree *subsrc;
	SourceTree *root;
	int32_t funcid;
	bool kakutei;
	BlockType selftype;
	bool glbconstant;
	bool touka;
	SourceTree *parent;
	SourceTree *frp;
	SourceTree *crp;
	PWCArray<intptr_t> rettype;
	int hikisuucount;
	int tpcount;
	intptr_t topsrcidx;
	intptr_t topsrcgyo;
	intptr_t gblhani;
	intptr_t gblsaikouhani;
	intptr_t mbrhani;
	intptr_t mbrsaikouhani;
	PWCArray< Hensuu<SourceTree> > funchensuu;
	PWCArrayQuad< PWCArray<wchar_t> > hensuname;
	PWCArray<intptr_t> hensuidx;
	intptr_t breaknaihani;
	PWCArray<SourceTree*> children;
	PWCArray<SourceTree*> subfunc;
	PWCArrayQuad< PWCArray<intptr_t> > token;
	PWCArray<intptr_t> gyo;
	PWCArray<intptr_t> defdel;
	PWCArray<intptr_t> junkandata;
	union{
		intptr_t golbalgeta;
		intptr_t thatoshori;
		intptr_t junkanid;
	} sonota;
	intptr_t mbsize;
	intptr_t byteadress;
	intptr_t deladress;
	PWCArray<intptr_t> backpatch;
	PWCArray<intptr_t> breakpatch;
	bool returnshita;
	intptr_t teigikinshi;
	bool breakin;
	bool breakinnest;
	SwitchExtStruct *extswitch;
	BranchExtStruct *extbranch;
	LoopExtStruct *extloop;
	PWCArray<intptr_t> *ptk;//̏ꂾp
	SourceTree(STStatic<SourceTree, GC> *st, const PWCArray<wchar_t> *const name = NULL, SourceTree *const p = NULL, const bool fukakutei = false)
	{
		FlgReset();
		stat = st;
		parent = p;
		subsrc = NULL;
		extswitch = NULL;
		extbranch = NULL;
		extloop = NULL;
		if(parent != NULL){
			selftype = UNKNOWN_BLOCK;
			root = parent->root;
			glbconstant = parent->glbconstant;
			if(name != NULL){
				breakinnest = false;
				kakutei = !fukakutei;
				funcid = stat->funclist.Add(this);
				frp = crp = this;
				myname = *name;
				parent->frp->subfunc += this;
			}else{
				breakinnest = parent->breakinnest;
				kakutei = false;
				frp = parent->frp;
				crp = parent->crp;
			}
		}else{
			selftype = FUNC_BLOCK;
			root = this;
			glbconstant = false;
			breakinnest = false;
			kakutei = false;
			funcid = stat->funclist.Add(this);
			frp = crp = this;
			rettype = VOID_TOKEN;
		}
		if(root == this){
			srce = new Src(stat->ss);
		}else{
			srce = root->srce;
		}
	}
	~SourceTree()
	{
		if(root == this) delete srce;
		Del();
	}
	void Del()
	{
		intptr_t i;
		delete subsrc;
		subsrc = NULL;
		for(i = 0; i < children.GetLength(); i++) delete children.GetConst()[i];
		children.Clear();
		delete extswitch;
		extswitch = NULL;
		delete extbranch;
		extbranch = NULL;
		delete extloop;
		extloop = NULL;
	}
	bool AruNashi(const PWCArray<wchar_t> &name)
	{
		intptr_t i;
		for(i = 0; i < hensuname.GetLength(); i++){
			if(name == hensuname.GetConst()[i]) return true;
		}
		for(i = 0; i < frp->subfunc.GetLength(); i++){
			if(name == frp->subfunc.GetConst()[i]->myname) return true;
		}
		return false;
	}
	intptr_t AddHiddenHensuu(PWCArray<intptr_t> *ptoke, const PWCArray<intptr_t> &type, const intptr_t size)
	{
		PWCArray<intptr_t> *ptok;
		frp->funchensuu[frp->funchensuu.GetLength()].tree = this;
		frp->funchensuu.Tail().type = type;
		frp->funchensuu.Tail().size = size;
		hensuname[hensuname.GetLength()][0] = L'0';
		hensuidx += frp->funchensuu.GetLength()-1;
		if(ptoke != NULL){
			if(teigikinshi >= 0){
				ptok = &token.GetData()[teigikinshi];
			}else{
				ptok = ptoke;
			}
			if(frp == root) *ptok += TILDE_TOKEN;
			*ptok += ~(frp->funchensuu.GetLength()-1);
			*ptok += VOID_TOKEN;
			*ptok += SEMICOLON_TOKEN;
		}
		return frp->funchensuu.GetLength()-1;
	}
	bool AddHensuu(const PWCArray<intptr_t> &type, const PWCArray<wchar_t> &name, const intptr_t size)
	{
		if(L'0' <= name.GetConst()[0] && name.GetConst()[0] <= L'9') return false;
		if(name.GetLength() >= 2 && name.GetConst()[name.GetLength()-2] == L'_' && name.GetConst()[name.GetLength()-1] == L't'){
			if(type.GetConst()[0] != TYPE_TOKEN) return false;
		}else{
			if(type.GetConst()[0] == TYPE_TOKEN) return false;
		}
		if(AruNashi(name)) return false;
		frp->funchensuu[frp->funchensuu.GetLength()].tree = this;
		frp->funchensuu.Tail().type = type;
		frp->funchensuu.Tail().size = size;
		hensuname += name;
		hensuidx += frp->funchensuu.GetLength()-1;
		return true;
	}
	SourceTree* GetChild(const PWCArray<wchar_t> *const name, const bool fukakutei, const bool any, const bool create)
	{
		intptr_t i;
		if(name != NULL){
			if(!any && L'0' <= name->GetConst()[0] && name->GetConst()[0] <= L'9') return NULL;
			if(!any && name->GetLength() >= 2 && name->GetConst()[name->GetLength()-2] == L'_' && name->GetConst()[name->GetLength()-1] == L't') return NULL;
			if(fukakutei){
				for(i = 0; i < hensuname.GetLength(); i++){
					if(*name == hensuname.GetConst()[i]) return NULL;
				}
				for(i=0; i < frp->subfunc.GetLength(); i++){
					if(*name == frp->subfunc.GetConst()[i]->myname) return frp->subfunc.GetConst()[i];
				}
			}else{
				if(AruNashi(*name)) return NULL;
			}
		}
		if(!create) return NULL;
		children += new SourceTree(stat, name, this, fukakutei);
		return children.GetTail();
	}
	bool ZeroHajimari(uint64_t &ui64, const wchar_t *const tstr, const intptr_t tlen)
	{
		intptr_t i;
		if(tlen <= 1) return false;
		ui64 = 0;
		switch(tstr[0]){
		case L'b':
			for(i = 1; i < tlen; i++){
				if(tstr[i] == L'0' || tstr[i] == L'1'){
					ui64 *= 2;
					ui64 += tstr[i] - L'0';
				}else{
					return false;
				}
			}
			break;
		case L'd':
			for(i = 1; i < tlen; i++){
				if(L'0' <= tstr[i] && tstr[i] <= L'9'){
					ui64 *= 10;
					ui64 += tstr[i] - L'0';
				}else{
					return false;
				}
			}
			break;
		case L'o':
			for(i = 1; i < tlen; i++){
				if(L'0' <= tstr[i] && tstr[i] <= L'7'){
					ui64 *= 8;
					ui64 += tstr[i] - L'0';
				}else{
					return false;
				}
			}
			break;
		case L'x':
			for(i = 1; i < tlen; i++){
				if(L'0' <= tstr[i] && tstr[i] <= L'9'){
					ui64 *= 16;
					ui64 += tstr[i] - L'0';
				}else if(L'A' <= tstr[i] && tstr[i] <= L'F'){
					ui64 *= 16;
					ui64 += tstr[i] - L'A' + 10;
				}else if(L'a' <= tstr[i] && tstr[i] <= L'f'){
					ui64 *= 16;
					ui64 += tstr[i] - L'a' + 10;
				}else{
					return false;
				}
			}
			break;
		default:
			return false;
		}
		return true;
	}
	bool Suuji(PWCArray<intptr_t> &tk, Src &src, const intptr_t t)
	{
		intptr_t j, k;
		uint64_t ui64;
		double baisei;
		ui64 = 0;
		if(src.source.GetConst()[t] == L'0'){
			if(src.idx - t == 1){
			}else if(ZeroHajimari(ui64, src.source.GetConst()+t+1, src.idx-t-1)){
				tk += CONST_TOKEN;
				tk += ULONG_TOKEN;
				tk.SetLength(tk.GetLength()+(sizeof(uint64_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
				*(uint64_t*)(tk.GetConst()+tk.GetLength()-(sizeof(uint64_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = ui64;
				return true;
			}else{
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"Ŏn܂鎯ʎq͎g܂B");
				return false;
			}
		}else{
			for(j = t; j < src.idx; j++){
				if(L'0' <= src.source.GetConst()[j] && src.source.GetConst()[j] <= L'9'){
					ui64 *= 10;
					ui64 += src.source.GetConst()[j] - L'0';
				}else{
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"Ŏn܂鎯ʎq͎g܂B");
					return false;
				}
			}
		}
		if(
			src.idx >= src.source.GetLength() || src.source.GetConst()[src.idx] != L'.'
			|| (src.idx+1 < src.source.GetLength() && src.source.GetConst()[src.idx+1] == L'.'))
		{
			tk += CONST_TOKEN;
			tk += LONG_TOKEN;
			tk.SetLength(tk.GetLength()+(sizeof(int64_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(int64_t*)(tk.GetConst()+tk.GetLength()-(sizeof(int64_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = (int64_t)ui64;
			return true;
		}
		src.idx++;
		j = src.idx;
		if(!src.GetNumString()){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"_̌ '0' KvłB");
			return false;
		}
		baisei = 0.0;
		for(k = 1; k <= src.idx-j; k++){
			baisei += (double)(src.source.GetConst()[src.idx-k] - L'0');
			baisei /= 10.0;
		}
		baisei += (double)ui64;
		if(src.idx >= src.source.GetLength() || (src.source.GetConst()[src.idx] != L'E' && src.source.GetConst()[src.idx] != L'e')){
			if(src.IsTSChar(src.source.GetConst()[src.idx])){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			tk += CONST_TOKEN;
			tk += DOUBLE_TOKEN;
			tk.SetLength(tk.GetLength()+(sizeof(double)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(double*)(tk.GetConst()+tk.GetLength()-(sizeof(double)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = baisei;
			return true;
		}
		src.idx++;
		if(src.idx >= src.source.GetLength()){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"sȈʒuŃt@CI[ɒB܂B");
			return false;
		}
		if(src.source.GetConst()[src.idx] == L'-'){
			src.idx++;
			j = src.idx;
			if(!src.GetTString()){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			k = 0;
			for(; j < src.idx; j++){
				if(L'0' <= src.source.GetConst()[j] && src.source.GetConst()[j] <= L'9'){
					k *= 10;
					k += src.source.GetConst()[j] - L'0';
				}else{
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
			}
			baisei *= pow(10.0, -(double)k);
		}else{
			if(src.source.GetConst()[src.idx] == L'+') src.idx++;
			j = src.idx;
			if(!src.GetTString()){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			k = 0;
			for(; j < src.idx; j++){
				if(L'0' <= src.source.GetConst()[j] && src.source.GetConst()[j] <= L'9'){
					k *= 10;
					k += src.source.GetConst()[j] - L'0';
				}else{
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
			}
			baisei *= pow(10.0, (double)k);
		}
		tk += CONST_TOKEN;
		tk += DOUBLE_TOKEN;
		tk.SetLength(tk.GetLength()+(sizeof(double)+sizeof(intptr_t)-1)/sizeof(intptr_t));
		*(double*)(tk.GetConst()+tk.GetLength()-(sizeof(double)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = baisei;
		return true;
	}
	SourceTree* BlockOpen(const PWCArray<wchar_t> *const name, const BlockType type, const bool any, const bool create)
	{
		SourceTree *pst;
		if(type != FUNC_BLOCK && type != CLASS_BLOCK){
			if((pst = GetChild(name, false, any, create)) == NULL) return NULL;
			pst->selftype = type;
			if(type == DELEGATE_BLOCK) pst->crp = pst;
			return pst;
		}
		if((pst = GetChild(name, true, any, create)) == NULL) return NULL;
		pst->selftype = type;
		return pst;
	}
	bool IsOpen()
	{
		intptr_t ii;
		for(ii = 0; ii < funchensuu.GetLength(); ii++){
			if(funchensuu.GetConst()[ii].size > 0){
				if(funchensuu.GetConst()[ii].type.GetConst()[0] != PUBLIC_TOKEN) return false;
				if(funchensuu.GetConst()[ii].type.GetConst()[1] == REF_TOKEN) return false;
				if(funchensuu.GetConst()[ii].type.GetConst()[1] == LIST_TOKEN) return false;
				if(funchensuu.GetConst()[ii].type.GetConst()[1] == THREAD_TOKEN) return false;
				if(funchensuu.GetConst()[ii].type.GetConst()[1] == CLASS_TOKEN){
					if(!stat->funclist.Get(funchensuu.GetConst()[ii].type.GetConst()[2])->IsOpen()) return false;
				}
			}
		}
		return true;
	}
	int32_t GetFuncId(const PWCArray<wchar_t> &name, const bool create)
	{
		SourceTree *pst;
		if((pst = frp->GetChild(&name, true, false, create)) != NULL) return pst->funcid;
		return -1;
	}
	intptr_t GetMemberSize()
	{
		intptr_t ii;
		intptr_t size;
		if(mbsize >= 0) return mbsize;
		size = 0;
		for(ii = 0; ii < funchensuu.GetLength(); ii++){
			size += funchensuu.GetConst()[ii].size;
		}
		if(this->selftype == FUNC_BLOCK) size = Aligner(size);
		return mbsize = size;
	}
	intptr_t GetMemberOffset(const intptr_t mbi)
	{
		intptr_t o = 0, i;
		if(frp != this &&  tpcount <= mbi && mbi < tpcount+hikisuucount){
			for(i = frp->tpcount+frp->hikisuucount; i < tpcount; i++) o += frp->funchensuu.GetConst()[i].size;
			for(i = tpcount+hikisuucount-1; i > mbi; i--) o += frp->funchensuu.GetConst()[i].size;
		}else{
			if(mbi < frp->tpcount+frp->hikisuucount){
				for(i = frp->tpcount+frp->hikisuucount; i < frp->funchensuu.GetLength(); i++) o += frp->funchensuu.GetConst()[i].size;
				o = Aligner(o);
				for(i = frp->tpcount+frp->hikisuucount-1; i > mbi; i--) o += frp->funchensuu.GetConst()[i].size;
			}else{
				for(i = frp->tpcount+frp->hikisuucount; i < mbi; i++) o += frp->funchensuu.GetConst()[i].size;
			}
		}
		return o;
	}
	bool CheckJunkanBetaClsSub(PWCArray<int32_t> &idary, intptr_t tsize, PWCArray<intptr_t> &jkd, intptr_t &ll)
	{
		intptr_t ii, jj, kk, oll, mm;
		oll = ll;
		for(ii = 0; ii < funchensuu.GetLength(); ii++){
			if(funchensuu.GetConst()[ii].size <= 0) continue;
			jj = 0;
			if(IsSoushokuToken((TokenKind)funchensuu.GetConst()[ii].type.GetConst()[0])) jj++;
			kk = jj;
			while(funchensuu.GetConst()[ii].type.GetConst()[jj] == REF_TOKEN
				|| funchensuu.GetConst()[ii].type.GetConst()[jj] == LIST_TOKEN) jj++;
			if(funchensuu.GetConst()[ii].type.GetConst()[jj] == CLASS_TOKEN){
				kk = jj - kk;
				if(kk > 0){
					mm = stat->funclist.Get(funchensuu.GetConst()[ii].type.GetConst()[jj+1])->CheckJunkan(idary);
					if(mm >= 0){
						if(mm < ll) ll = mm;
						if(mm < oll){
							jkd += tsize;
							jkd += (intptr_t)stat->funclist.Get(funchensuu.GetConst()[ii].type.GetConst()[jj+1])->funcid;
							jkd += (intptr_t)(kk-1);
						}
					}
				}else{
					if(!stat->funclist.Get(funchensuu.GetConst()[ii].type.GetConst()[jj+1])->CheckJunkanBetaClsSub(idary, tsize, jkd, ll)){
						return false;
					}
				}
			}else if(funchensuu.GetConst()[ii].type.GetConst()[jj] == NULL_TOKEN){
				ll = 0;
				jkd += tsize;
				jkd += -1;
				jkd += (intptr_t)(jj - kk - 1);
			}
			tsize += funchensuu.GetConst()[ii].size;
		}
		return true;
	}
	intptr_t CheckJunkan(PWCArray<int32_t> &idary)
	{
		PWCArray<wchar_t> name;
		int32_t fi;
		intptr_t ll;
		if(junkandata.GetLength() > 0){
			if(idary.GetLength() <= 0) return 0;
			for(ll = 0; ll < idary.GetLength(); ll++){
				if(idary.GetConst()[ll] == funcid) return ll;
			}
			return -1;
		}
		name = L"break";
		if((fi = GetFuncId(name, false)) < 0 || stat->funclist.Get(fi)->token.GetLength() <= 0) return -1;
		junkandata = GetMemberSize();
		junkandata += stat->funclist.Get(fi)->byteadress;
		idary += funcid;
		ll = idary.GetLength();
		if(CheckJunkanBetaClsSub(idary, 0, junkandata, ll) && ll < idary.GetLength()){
			sonota.junkanid = stat->ss->CircularGC.AddPL(junkandata);
			idary.Pop();
			return ll;
		}
		idary.Pop();
		junkandata.Clear();
		return -1;
	}
	bool IsJunkanAble()
	{
		if(sonota.junkanid < 0) return false;
		if(junkandata.GetLength() > 0) return true;
		PWCArray<int32_t> idary;
		return CheckJunkan(idary) >= 0 ? true : (sonota.junkanid = -1, false);
	}
	SourceTree* IjouTansaku(const PWCArray<wchar_t> &name, intptr_t &memberichi, SourceTree &stree, const intptr_t *const iap)
	{
		SourceTree *pst;
		intptr_t sa;
		int32_t fi;
		intptr_t i;
		pst = NULL;
		sa = 0;
		fi = GetFuncId(name, false);
		if(fi >= 0){
			pst = stat->funclist.Get(fi);
			if(pst->selftype != FUNC_BLOCK) return NULL;
			if(pst->kakutei){
				if(iap != NULL){
					Hensuu<SourceTree> *ph;
					PWCArray<intptr_t> type;
					ph = &stree.frp->funchensuu.GetData()[~iap[stree.frp == stree.root ? 1 : 0]];
					if(pst->hikisuucount > 0 && pst->hensuname.GetConst()[pst->tpcount].GetConst()[0] == L'0'){
						type = pst->funchensuu.GetConst()[pst->hensuidx.GetConst()[pst->tpcount]].type;
						type.Pop();
					}else{
						type = REF_TOKEN;
						type += OTHERKIND_TOKEN;
					}
					ph->type = type;//K^Cvς
				}
			}else{
				pst = NULL;
			}
		}
		if(pst == NULL) for(i = 0; i < funchensuu.GetLength(); i++){
			if(funchensuu.GetConst()[i].type.GetConst()[0] == CORE_TOKEN){
				if(pst != NULL) return NULL;
				pst = stat->funclist.Get((int32_t)funchensuu.GetConst()[i].type.GetConst()[2])->IjouTansaku(name, memberichi, stree, iap);
				if(pst != NULL) memberichi += sa;
			}
			sa += funchensuu.GetConst()[i].size;
		}
		return pst;
	}
	SourceTree* GetIjousaki(intptr_t &memberichi, SourceTree &stree, const intptr_t *const iap)
	{
		if(parent == NULL || parent->selftype != CLASS_BLOCK || frp != this) return NULL;
		return parent->IjouTansaku(myname, memberichi, stree, iap);
	}
	intptr_t TypeIdtoSize(const int32_t typ) const
	{
		switch(typ){
		case BYTE_TYPEID:
			return sizeof(int8_t);
		case UBYTE_TYPEID:
			return sizeof(uint8_t);
		case SHORT_TYPEID:
			return sizeof(int16_t);
		case USHORT_TYPEID:
			return sizeof(uint16_t);
		case INT_TYPEID:
			return sizeof(int32_t);
		case UINT_TYPEID:
			return sizeof(uint32_t);
		case LONG_TYPEID:
			return sizeof(int64_t);
		case ULONG_TYPEID:
			return sizeof(uint64_t);
		case FLOAT_TYPEID:
			return sizeof(float);
		case DOUBLE_TYPEID:
			return sizeof(double);
		case CHAR_TYPEID:
			return sizeof(wchar_t);
		case BOOL_TYPEID:
			return sizeof(bool);
		case ADDRESS_TYPEID:
			return sizeof(intptr_t);
		case FUNC_TYPEID:
		case METHOD_TYPEID:
			return sizeof(intptr_t);
		case DELEGATE_TYPEID:
			return Aliszof<intptr_t>()*3;
		case REF_TYPEID:
		case LIST_TYPEID:
			return sizeof(Reference);
		case DYNREF_TYPEID:
		case DYNLIST_TYPEID:
			return sizeof(DynamicRef);
		case VOID_TYPEID:
			return 0;
		default:
			if(typ < 0){
				if(ENDTHREAD_TYPEID <= typ && typ < 0) return sizeof(Thread);
				return -1;
			}
			switch((stat->funclist.Get(typ))->selftype){
			case CLASS_BLOCK:
				return stat->funclist.Get(typ)->GetMemberSize();
			case ENUM_BLOCK:
				return sizeof(int32_t);
			}
			break;
		}
		return -1;
	}
	void GetTemplateNameSub(PWCArray<wchar_t> &name) const
	{
		if(myname.GetConst()[0] == L'\0'){
			PWCArray<intptr_t> type;
			Reference r;
			(*(SourceTree**)(myname.GetConst()+1))->GetTemplateNameSub(name);
			if(name.GetTail() != L':') name += L',';
			type.AddTail((intptr_t*)(myname.GetConst()+TNT_OFFSET), (myname.GetLength()-TNT_OFFSET)*sizeof(wchar_t)/sizeof(intptr_t));
			r.init();
			TypeIdtoName(r, GetTypeId(type));
			name.AddTail((wchar_t*)r.atpos(), r.len()/sizeof(wchar_t));
		}else{
			name = myname;
			name += L"<:";
		}
	}
	void TypeIdtoName(Reference &r, const int32_t typ) const
	{
		switch(typ){
		case BYTE_TYPEID:
			Benry::WstrtoRef(r, L"byte");
			break;
		case UBYTE_TYPEID:
			Benry::WstrtoRef(r, L"ubyte");
			break;
		case SHORT_TYPEID:
			Benry::WstrtoRef(r, L"short");
			break;
		case USHORT_TYPEID:
			Benry::WstrtoRef(r, L"ushort");
			break;
		case INT_TYPEID:
			Benry::WstrtoRef(r, L"int");
			break;
		case UINT_TYPEID:
			Benry::WstrtoRef(r, L"uint");
			break;
		case LONG_TYPEID:
			Benry::WstrtoRef(r, L"long");
			break;
		case ULONG_TYPEID:
			Benry::WstrtoRef(r, L"ulong");
			break;
		case FLOAT_TYPEID:
			Benry::WstrtoRef(r, L"float");
			break;
		case DOUBLE_TYPEID:
			Benry::WstrtoRef(r, L"double");
			break;
		case CHAR_TYPEID:
			Benry::WstrtoRef(r, L"char");
			break;
		case BOOL_TYPEID:
			Benry::WstrtoRef(r, L"bool");
			break;
		case ADDRESS_TYPEID:
			Benry::WstrtoRef(r, L"index");
			break;
		case FUNC_TYPEID:
			Benry::WstrtoRef(r, L"func");
			break;
		case METHOD_TYPEID:
			Benry::WstrtoRef(r, L"method");
			break;
		case DELEGATE_TYPEID:
			Benry::WstrtoRef(r, L"delegate");
			break;
		case REF_TYPEID:
			Benry::WstrtoRef(r, L"ref:");
			break;
		case LIST_TYPEID:
			Benry::WstrtoRef(r, L"list:");
			break;
		case DYNREF_TYPEID:
			Benry::WstrtoRef(r, L"ref");
			break;
		case DYNLIST_TYPEID:
			Benry::WstrtoRef(r, L"list");
			break;
		case VOID_TYPEID:
			Benry::WstrtoRef(r, L"void");
			break;
		default:
			{
				PWCArray<wchar_t> tmp;
				if(typ < 0){
					if(~typ < stat->funclist.GetTotal()){
						tmp = L"thread<:";
						if(stat->funclist.Get(~typ)->tpcount > 0){
							stat->funclist.Get(~typ)->GetTemplateNameSub(tmp);
							tmp += L":>";
						}else{
							tmp += stat->funclist.Get(~typ)->myname;
						}
						tmp += L":>";
					}else{
						tmp = L"<UNKNOWN>";
					}
				}else if(typ < stat->funclist.GetTotal()){
					switch(stat->funclist.Get(typ)->selftype){
					case CLASS_BLOCK:
						tmp = L"class ";
						if(stat->funclist.Get(typ)->tpcount > 0){
							stat->funclist.Get(typ)->GetTemplateNameSub(tmp);
							tmp += L":>";
						}else{
							tmp += stat->funclist.Get(typ)->myname;
						}
						break;
					case ENUM_BLOCK:
						tmp = L"enum ";
						tmp += stat->funclist.Get(typ)->myname;
						break;
					default:
						if(stat->funclist.Get(typ) != stat->funclist.Get(typ)->root){
							if(stat->funclist.Get(typ)->tpcount > 0){
								stat->funclist.Get(typ)->GetTemplateNameSub(tmp);
								tmp += L":>";
							}else{
								tmp = stat->funclist.Get(typ)->myname;
							}
						}else{
							tmp = stat->funclist.Get(typ)->srce->filename;
						}
						break;
					}
				}else{
					tmp = L"<UNKNOWN>";
				}
				tmp += L'\0';
				Benry::WstrtoRef(r, tmp.GetConst());
			}
			break;
		}
	}
	intptr_t GetTypeSize(const PWCArray<intptr_t> &type) const
	{
		return TypeIdtoSize(GetTypeId(type));
	}
	int32_t GetTypeId(const PWCArray<intptr_t> &type) const
	{
		intptr_t i;
		if(type.GetConst()[0] == NULL_TOKEN){
			return (int32_t)type.GetConst()[1];
		}
		return TokenTtoTypeId(type.GetConst(), i=0);
	}
	bool YuruiKataCheck(const PWCArray<intptr_t> &watasu, const PWCArray<intptr_t> &toru, const PWCArray<wchar_t> &filename, const intptr_t gyousuu) const
	{
		intptr_t wnbn, tnbn;
		wnbn = tnbn = 0;
		if(IsSoushokuToken((TokenKind)watasu.GetConst()[wnbn])) wnbn++;
		if(IsSoushokuToken((TokenKind)toru.GetConst()[tnbn])) tnbn++;
		while(wnbn < watasu.GetLength() && tnbn < toru.GetLength()){
			if(toru.GetConst()[tnbn] == REF_TOKEN){
				if(watasu.GetConst()[wnbn] == LIST_TOKEN){
					tnbn++;
					wnbn++;
				}
			}else if(toru.GetConst()[tnbn] == FUNC_TOKEN || toru.GetConst()[tnbn] == METHOD_TOKEN){
				if(toru.GetConst()[tnbn] == FUNC_TOKEN){
					if(watasu.GetConst()[wnbn] != FUNC_TOKEN){
						stat->ss->ErrorMessage.Add(filename, gyousuu, L"^G[łB");
						return false;
					}
				}else{
					tnbn++;
					if(watasu.GetConst()[wnbn] == METHOD_TOKEN){
						wnbn++;
						if(watasu.GetConst()[wnbn] != toru.GetConst()[tnbn]){
							stat->ss->ErrorMessage.Add(filename, gyousuu, L"^G[łB");
							return false;
						}
					}else if(watasu.GetConst()[wnbn] != FUNC_TOKEN){
						stat->ss->ErrorMessage.Add(filename, gyousuu, L"^G[łB");
						return false;
					}
				}
				tnbn++;
				wnbn++;
				if(watasu.GetConst()[wnbn] == CONST_TOKEN){
					wnbn++;
					if(IsReadOnlyToken((TokenKind)toru.GetConst()[tnbn])) tnbn++;
				}else if(watasu.GetConst()[wnbn] == PROPERTY_TOKEN){
					wnbn++;
					if(toru.GetConst()[tnbn] == PROPERTY_TOKEN) tnbn++;
				}
			}
			if(toru.GetConst()[tnbn] == CONST_TOKEN){
				tnbn++;
				if(watasu.GetConst()[wnbn] == CONST_TOKEN) wnbn++;
			}else if(toru.GetConst()[tnbn] == PROPERTY_TOKEN){
				tnbn++;
				if(IsReadOnlyToken((TokenKind)watasu.GetConst()[wnbn])) wnbn++;
			}
			if(toru.GetConst()[tnbn] == CLASS_TOKEN || toru.GetConst()[tnbn] == ENUM_TOKEN){
				if(watasu.GetConst()[wnbn] != toru.GetConst()[tnbn]){
					stat->ss->ErrorMessage.Add(filename, gyousuu, L"^G[łB");
					return false;
				}
				tnbn++;
				wnbn++;
			}
			if(watasu.GetConst()[wnbn] != toru.GetConst()[tnbn]){
				stat->ss->ErrorMessage.Add(filename, gyousuu, L"^G[łB");
				return false;
			}
			tnbn++;
			wnbn++;
		}
		if(tnbn != toru.GetLength()){
			stat->ss->ErrorMessage.Add(filename, gyousuu, L"^G[łB");
			return false;
		}
		return true;
	}
	static bool KansuuKata(const SourceTree *const pstree, const PWCArray<intptr_t> &katmari, PWCArrayQuad< PWCArray<intptr_t> > &waketa)
	{
		intptr_t i, j;
		i = 0;
		if(IsSoushokuToken((TokenKind)katmari.GetConst()[i])) i++;
		if(katmari.GetConst()[i] != FUNC_TOKEN && katmari.GetConst()[i] != DELEGATE_TOKEN && katmari.GetConst()[i] != PLUGIN_TOKEN){
			if(katmari.GetConst()[i] != METHOD_TOKEN) return false;
			i++;
			if(pstree == NULL || (int32_t)katmari.GetConst()[i] != (pstree->touka ? pstree->parent->funcid : pstree->funcid)) return false;
		}
		i++;
		waketa.Clear();
		waketa[0].Clear();
		if(IsReadOnlyToken((TokenKind)katmari.GetConst()[i])){
			waketa.GetData()[0] += katmari.GetConst()[i];
			i++;
		}
		switch(katmari.GetConst()[i]){
		case BOOL_TOKEN:
		case CHAR_TOKEN:
		case DOUBLE_TOKEN:
		case FLOAT_TOKEN:
		case BYTE_TOKEN:
		case INT_TOKEN:
		case LONG_TOKEN:
		case SHORT_TOKEN:
		case UBYTE_TOKEN:
		case UINT_TOKEN:
		case ULONG_TOKEN:
		case USHORT_TOKEN:
		case INDEX_TOKEN:
		case VOID_TOKEN:
			waketa.GetData()[0] += katmari.GetConst()[i++];
			break;
		case ENUM_TOKEN:
			waketa.GetData()[0] += katmari.GetConst()[i++];
			waketa.GetData()[0] += katmari.GetConst()[i++];
			break;
		case FUNC_TOKEN:
			if(!FuncToken(waketa.GetData()[0], katmari.GetConst(), i)) return false;
			break;
		default:
			return false;
		}
		if(katmari.GetConst()[i] != KAKKOCOLOOPEN_TOKEN) return false;
		if(katmari.GetConst()[i+1] == KAKKOCOLOCLOSE_TOKEN) return true;
		j = 0;
		while(true){
			i++;
			j++;
			waketa[j].Clear();
			do{
				waketa.GetData()[j] += katmari.GetConst()[i++];
				if(katmari.GetConst()[i-1] == FUNC_TOKEN){
					FuncToken(waketa.GetData()[j], katmari.GetConst(), i);
				}else if(katmari.GetConst()[i-1] == CLASS_TOKEN || katmari.GetConst()[i-1] == ENUM_TOKEN || katmari.GetConst()[i-1] == DELEGATE_TOKEN){
					waketa.GetData()[j] += katmari.GetConst()[i++];
				}
				if(katmari.GetConst()[i] == KAKKOCOLOCLOSE_TOKEN) return true;
			}while(katmari.GetConst()[i] != COMMA_TOKEN);
		}
		return true;
	}
	bool GblhaniSet(Src &src, const SourceTree &hkishiki)
	{
		if(gblhani == INTPTR_MAX){
			if(root == hkishiki.root && hkishiki.frp->gblhani != INTPTR_MAX){
				gblhani = hkishiki.frp->gblhani;
				if(gblhani <= gblsaikouhani){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ĂȂ\̂O[oϐg֐łB");
					return false;
				}
			}else{
				gblhani = root->hensuidx.GetLength();
			}
		}
		return true;
	}
	bool MbrhaniSet(Src &src, const SourceTree &hkishiki)
	{
		if(parent->selftype != CLASS_BLOCK && mbrhani == INTPTR_MAX){
			if(parent == hkishiki.frp->parent && hkishiki.frp->mbrhani != INTPTR_MAX){
				mbrhani = hkishiki.frp->mbrhani;
				if(mbrhani <= mbrsaikouhani){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ĂȂ\̂郁o[ϐg֐łB");
					return false;
				}
			}else{
				mbrhani = parent->hensuidx.GetLength();
			}
		}
		return true;
	}
	bool FuncTypeGet(Src &src, PWCArray<intptr_t> &type, bool cont, const SourceTree &hkishiki)
	{
		intptr_t ii;
		if(!GblhaniSet(src, hkishiki)) return false;
		if(!MbrhaniSet(src, hkishiki)) return false;
		ii = 0;
		if(rettype[ii] == PUBLIC_TOKEN) ii++;//public͊֐|C^̌^ɉe^ȂB
		type.AddTail(rettype.GetConst()+ii, rettype.GetLength()-ii);
		type += KAKKOCOLOOPEN_TOKEN;
		if(hikisuucount > 0){
			for(ii = tpcount; ii < tpcount+hikisuucount; ii++){
				type += funchensuu.GetConst()[ii].type;
				type += COMMA_TOKEN;
			}
			type.Tail() = KAKKOCOLOCLOSE_TOKEN;
		}else{
			type += KAKKOCOLOCLOSE_TOKEN;
		}
		if(cont) type += funcid;
		return true;
	}
	template<typename TYPE> void TankouConst(const TokenKind to, TYPE &x)
	{
		switch(to){
		case PULUS_TOKEN:
			break;
		case MINUS_TOKEN:
			x *= -1;
			break;
		case EXCLAMATION_TOKEN:
			x = ~x;
			break;
		}
	}
	template<typename TYPE> void TankouConstReal(const TokenKind to, TYPE &x)
	{
		switch(to){
		case PULUS_TOKEN:
			break;
		case MINUS_TOKEN:
			x *= -1;
			break;
		}
	}
	template<typename TYPE> static bool ConstCast(PWCArray<intptr_t> &last, TYPE &x)
	{
		intptr_t tmpi;
		tmpi = 0;
		if(last.GetConst()[tmpi] != CONST_TOKEN) return false;
		tmpi++;
		switch(last.GetConst()[tmpi++]){
		case BYTE_TOKEN:
			x = (TYPE)*(int8_t *)(last.GetConst()+tmpi);
			break;
		case SHORT_TOKEN:
			x = (TYPE)*(int16_t *)(last.GetConst()+tmpi);
			break;
		case INT_TOKEN:
			x = (TYPE)*(int32_t *)(last.GetConst()+tmpi);
			break;
		case LONG_TOKEN:
			x = (TYPE)*(int64_t *)(last.GetConst()+tmpi);
			break;
		case UBYTE_TOKEN:
			x = (TYPE)*(uint8_t *)(last.GetConst()+tmpi);
			break;
		case USHORT_TOKEN:
			x = (TYPE)*(uint16_t *)(last.GetConst()+tmpi);
			break;
		case UINT_TOKEN:
			x = (TYPE)*(uint32_t *)(last.GetConst()+tmpi);
			break;
		case ULONG_TOKEN:
			x = (TYPE)*(uint64_t *)(last.GetConst()+tmpi);
			break;
		case INDEX_TOKEN:
			x = (TYPE)*(intptr_t *)(last.GetConst()+tmpi);
			break;
		case FLOAT_TOKEN:
			x = (TYPE)*(float *)(last.GetConst()+tmpi);
			break;
		case DOUBLE_TOKEN:
			x = (TYPE)*(double *)(last.GetConst()+tmpi);
			break;
		case CHAR_TOKEN:
			x = (TYPE)*(wchar_t *)(last.GetConst()+tmpi);
			break;
		case BOOL_TOKEN:
			x = (TYPE)*(bool *)(last.GetConst()+tmpi);
			break;
		case ENUM_TOKEN:
			x = (TYPE)*(int32_t *)(last.GetConst()+tmpi+1);
			break;
		default:
			return false;
		}
		return true;
	}
	template<typename TYPE> intptr_t ConstCastint(const int32_t fromt, const void *const fromd, TYPE *const dstd)
	{
		switch(fromt){
		case BYTE_TYPEID:
			*dstd = (TYPE)*(int8_t*)fromd;
			return sizeof(int8_t);
		case SHORT_TYPEID:
			*dstd = (TYPE)*(int16_t*)fromd;
			return sizeof(int16_t);
		case INT_TYPEID:
			*dstd = (TYPE)*(int32_t*)fromd;
			return sizeof(int32_t);
		case LONG_TYPEID:
			*dstd = (TYPE)*(int64_t*)fromd;
			return sizeof(int64_t);
		case ADDRESS_TYPEID:
			*dstd = (TYPE)*(intptr_t*)fromd;
			return sizeof(intptr_t);
		}
		return -1;
	}
	template<typename TYPE> intptr_t ConstCastuint(const int32_t fromt, const void *const fromd, TYPE *const dstd)
	{
		switch(fromt){
		case UBYTE_TYPEID:
			*dstd = (TYPE)*(uint8_t*)fromd;
			return sizeof(uint8_t);
		case USHORT_TYPEID:
			*dstd = (TYPE)*(uint16_t*)fromd;
			return sizeof(uint16_t);
		case UINT_TYPEID:
			*dstd = (TYPE)*(uint32_t*)fromd;
			return sizeof(uint32_t);
		case ULONG_TYPEID:
			*dstd = (TYPE)*(uint64_t*)fromd;
			return sizeof(uint64_t);
		}
		return -1;
	}
	template<typename TYPE> intptr_t ConstCastReal(const int32_t fromt, const void *const fromd, TYPE *const dstd)
	{
		switch(fromt){
		case FLOAT_TYPEID:
			*dstd = (TYPE)*(float*)fromd;
			return sizeof(float);
		case DOUBLE_TYPEID:
			*dstd = (TYPE)*(double*)fromd;
			return sizeof(double);
		}
		return -1;
	}
	intptr_t CastConst(const int32_t srct, const void *const srcd, const int32_t dstt, void *const dstd)
	{
		switch(dstt){
		case BYTE_TYPEID:
			return ConstCastint(srct, srcd, (int8_t *)dstd);
		case SHORT_TYPEID:
			return ConstCastint(srct, srcd, (int16_t *)dstd);
		case INT_TYPEID:
			return ConstCastint(srct, srcd, (int32_t *)dstd);
		case LONG_TYPEID:
			return ConstCastint(srct, srcd, (int64_t *)dstd);
		case ADDRESS_TYPEID:
			return ConstCastint(srct, srcd, (intptr_t *)dstd);
		case UBYTE_TYPEID:
			return ConstCastuint(srct, srcd, (uint8_t *)dstd);
		case USHORT_TYPEID:
			return ConstCastuint(srct, srcd, (uint16_t *)dstd);
		case UINT_TYPEID:
			return ConstCastuint(srct, srcd, (uint32_t *)dstd);
		case ULONG_TYPEID:
			return ConstCastuint(srct, srcd, (uint64_t *)dstd);
		case FLOAT_TYPEID:
			return ConstCastReal(srct, srcd, (float *)dstd);
		case DOUBLE_TYPEID:
			return ConstCastReal(srct, srcd, (double *)dstd);
		case BOOL_TYPEID:
			if(srct != BOOL_TYPEID) return -1;
			*(bool *)dstd = *(bool *)srcd;
			return sizeof(bool);
		case CHAR_TYPEID:
			if(srct != CHAR_TYPEID) return -1;
			*(wchar_t *)dstd = *(wchar_t *)srcd;
			return sizeof(wchar_t);
		default:
			if(srct < 0 || srct > ENDCLASS_TYPEID || srct != dstt || stat->funclist.Get(srct)->selftype != ENUM_BLOCK) return -1;
			*(int32_t *)dstd = *(int32_t *)srcd;
			return sizeof(int32_t);
		}
		return -1;
	}
	static bool ConstCaster(PWCArray<intptr_t> &last, const PWCArray<intptr_t> &type)
	{
		intptr_t tmpi, tmpii, pmpip;
		union{
			int8_t _byte; int16_t _short; int32_t _int; int64_t _long;
			uint8_t _ubyte; uint16_t _ushort; uint32_t _uint; uint64_t _ulong;
			float _float; double _double;
			intptr_t _index;
			wchar_t _char;
			bool _bool;
		} x;
		tmpi = 0;
		if(type.GetConst()[tmpi] != CONST_TOKEN) return false;
		tmpi++;
		switch(type.GetConst()[tmpi++]){
		case BYTE_TOKEN:
			if(!ConstCast(last, x._byte)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(int8_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(int8_t*)(last.GetConst()+last.GetLength()-(sizeof(int8_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._byte;
			break;
		case SHORT_TOKEN:
			if(!ConstCast(last, x._short)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(int16_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(int16_t*)(last.GetConst()+last.GetLength()-(sizeof(int16_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._short;
			break;
		case INT_TOKEN:
			if(!ConstCast(last, x._int)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(int32_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(int32_t*)(last.GetConst()+last.GetLength()-(sizeof(int32_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._int;
			break;
		case LONG_TOKEN:
			if(!ConstCast(last, x._long)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(int64_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(int64_t*)(last.GetConst()+last.GetLength()-(sizeof(int64_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._long;
			break;
		case UBYTE_TOKEN:
			if(!ConstCast(last, x._ubyte)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(uint8_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(uint8_t*)(last.GetConst()+last.GetLength()-(sizeof(uint8_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._ubyte;
			break;
		case USHORT_TOKEN:
			if(!ConstCast(last, x._ushort)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(uint16_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(uint16_t*)(last.GetConst()+last.GetLength()-(sizeof(uint16_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._ushort;
			break;
		case UINT_TOKEN:
			if(!ConstCast(last, x._uint)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(uint32_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(uint32_t*)(last.GetConst()+last.GetLength()-(sizeof(uint32_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._uint;
			break;
		case ULONG_TOKEN:
			if(!ConstCast(last, x._ulong)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(uint64_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(uint64_t*)(last.GetConst()+last.GetLength()-(sizeof(uint64_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._ulong;
			break;
		case INDEX_TOKEN:
			if(!ConstCast(last, x._index)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(intptr_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(intptr_t*)(last.GetConst()+last.GetLength()-(sizeof(intptr_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._index;
			break;
		case FLOAT_TOKEN:
			if(!ConstCast(last, x._float)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(float)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(float*)(last.GetConst()+last.GetLength()-(sizeof(float)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._float;
			break;
		case DOUBLE_TOKEN:
			if(!ConstCast(last, x._double)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(double)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(double*)(last.GetConst()+last.GetLength()-(sizeof(double)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._double;
			break;
		case CHAR_TOKEN:
			if(!ConstCast(last, x._char)) return false;
			last = type;
			last.SetLength(tmpi+(sizeof(wchar_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(wchar_t*)(last.GetConst()+last.GetLength()-(sizeof(wchar_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = x._char;
			break;
		case BOOL_TOKEN:
		case ENUM_TOKEN:
		case REF_TOKEN:
			if(last.GetLength()-1 > type.GetLength()) return false;
			for(tmpi = 0; tmpi < last.GetLength()-1; tmpi++){
				if(last.GetConst()[tmpi] != type.GetConst()[tmpi]) return false;
			}
			break;
		case METHOD_TOKEN:
			if(last.GetConst()[tmpi] != type.GetConst()[tmpi]) return false;
			tmpi++;
		case FUNC_TOKEN:
			if(type.GetConst()[tmpi] == CONST_TOKEN){
				if(last.GetConst()[tmpi] != CONST_TOKEN) return false;
				tmpi++;
			}else if(type.GetConst()[tmpi] == PROPERTY_TOKEN){
				if(!IsReadOnlyToken((TokenKind)last.GetConst()[tmpi])) return false;
				tmpi++;
			}
			if(last.GetConst()[tmpi] == CONST_TOKEN){
				tmpii = 1;
			}else{
				tmpii = 0;
			}
			if(last.GetLength()-tmpii-1 > type.GetLength()) return false;
			for(tmpi += tmpii; tmpi < last.GetLength()-1; tmpi++){
				if(last.GetConst()[tmpi] != type.GetConst()[tmpi-tmpii]) return false;
			}
			if(tmpii > 0){
				pmpip = last.GetTail();
				last.Clear();
				last.AddTail(type.GetConst(), tmpi-tmpii);
				last += pmpip;
			}
			break;
		default:
			return false;
		}
		return true;
	}
	bool ConstNanikaPath(Src &src, PWCArray<intptr_t> &last, SourceTree &hkishiki)
	{
		int32_t tyki;
		PWCArray<wchar_t> name,tmp;
		Hensuu<SourceTree> *phs;
		intptr_t t, ii;
		if((tyki = PathtoClassID(src, FUNC_BLOCK, true, false, hkishiki, false)) < 0) return false;
		switch(stat->funclist.Get(tyki)->selftype){
		case FUNC_BLOCK:
			if(stat->funclist.Get(tyki) != stat->funclist.Get(tyki)->root){
				if(!stat->funclist.Get(tyki)->kakutei){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"m肵ĂȂ̂Ŋ֐|C^ł܂B");
					return false;
				}
				if(stat->funclist.Get(tyki)->parent == stat->funclist.Get(tyki)->root){
					last = CONST_TOKEN;
					last += FUNC_TOKEN;
					if(!stat->funclist.Get(tyki)->FuncTypeGet(src, last, true, hkishiki)) return false;
				}else{
					last = CONST_TOKEN;
					last += METHOD_TOKEN;
					last += stat->funclist.Get(tyki)->parent->funcid;
					if(!stat->funclist.Get(tyki)->FuncTypeGet(src, last, true, hkishiki)) return false;
				}
				break;
			}
		case CLASS_BLOCK:
			src.GetToken(t);
			src.GetToken(t);
			name.Clear();
			name.AddTail(src.source.GetConst()+t, src.idx-t);
			if((phs = stat->funclist.Get(tyki)->GetHensuu(name, ii, true)) == NULL){
				tmp = L'\'';
				tmp += name;
				tmp += L"' o萔Ɍ܂B";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
				return false;
			}
			last = phs->type;
			break;
		case ENUM_BLOCK:
			src.GetToken(t);
			if(src.GetToken(t) != OTHERKIND_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			name.Clear();
			name.AddTail(src.source.GetConst()+t, src.idx-t);
			if((phs = stat->funclist.Get(tyki)->GetHensuu(name, ii, true)) == NULL){
				tmp = L'\'';
				tmp += name;
				tmp += L"' ܂B";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
				return false;
			}
			last = phs->type;
			break;
		default:
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
			return false;
		}
		return true;
	}
	bool Delegate(Src &src, PWCArray<intptr_t> &tk)
	{
		SourceTree *pst;
		TokenKind kin;
		intptr_t t;
		PWCArray<wchar_t> name, tmp;
		intptr_t size;
		PWCArray<intptr_t> type;
		if(frp == root){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"O[odelegate^͒`ł܂B");
			return false;
		}
		tk += DELEGATE_TOKEN;
		if((pst = BlockOpen(NULL, DELEGATE_BLOCK, false, true)) == NULL){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"svcȃG[łB");
			return false;
		}
		pst->srce = &src;
		pst->tpcount = (int)frp->funchensuu.GetLength();
		pst->rettype.Clear();
		if(!ReturnType(pst->rettype, src, true)) return false;
		if(IsReadOnlyToken((TokenKind)pst->rettype.GetConst()[0])){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"q͂܂B");
			return false;
		}
		tk += pst->rettype;
		if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
			return false;
		}
		tk += KAKKOCOLOOPEN_TOKEN;
		pst->hikisuucount = 0;
		kin = src.GetToken(t);
		while(kin != SHOUKAKKOCLOSE_TOKEN){
			src.idx = t;
			type.Clear();
			if(!TypeNanika(type, *srce, false, false, false, *this, true)) return false;
			switch(type.GetConst()[0]){
			case REF_TOKEN:
				if(!pst->RefTouroku(type, t, NULL)) return false;
				break;
			case LIST_TOKEN:
				if(!pst->ListTouroku(type, t, NULL)) return false;
				break;
			case THREAD_TOKEN:
				if(!pst->ThreadTouroku(type, NULL)) return false;
				break;
			case CLASS_TOKEN:
				if(!pst->ClassObjToroku(type, (int32_t)type.GetConst()[1], NULL)) return false;
				break;
			default:
				if(src.GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				name.Clear();
				name.AddTail(src.source.GetConst()+t, src.idx-t);
				if((size = GetTypeSize(type)) < 0){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
					return false;
				}
				if(!pst->AddHensuu(type, name, size)){
					tmp = L'\'';
					tmp += name;
					tmp += L"' ͖OdĂ邩AsłB";
					tmp += L'\0';
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
					return false;
				}
				break;
			}
			tk += type;
			pst->hikisuucount++;
			kin = src.GetToken(t);
			if(kin == DAINYUU_TOKEN){
				if(type.GetConst()[0] == DELEGATE_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"delegate^͎QƓnł܂B");
					return false;
				}
				frp->funchensuu.Tail().type += ~DAINYUU_TOKEN;
				frp->funchensuu.Tail().size = sizeof(intptr_t);
				tk += ~DAINYUU_TOKEN;
				kin = src.GetToken(t);
			}else{
				if(type.GetConst()[0] == THREAD_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"thread^͒lnł܂B");
					return false;
				}
				if(type.GetConst()[0] == CLASS_TOKEN && !stat->funclist.Get((int32_t)type.GetConst()[1])->kakutei){
					stat->funclist.Get((int32_t)type.GetConst()[1])->backpatch += (intptr_t)frp;
					stat->funclist.Get((int32_t)type.GetConst()[1])->breakpatch += frp->funchensuu.GetLength()-1;
				}
			}
			frp->funchensuu.Tail().size = Aligner(frp->funchensuu.GetTail().size);
			switch(kin){
			case COMMA_TOKEN:
				tk += COMMA_TOKEN;
				src.GetToken(t);
				continue;
			case SHOUKAKKOCLOSE_TOKEN:
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			break;
		}
		tk += KAKKOCOLOCLOSE_TOKEN;
		tk += (intptr_t)pst;
		if(src.GetToken(t) != CHUUKAKKOOPEN_TOKEN){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'{' ܂B");
			return false;
		}
		tk += CHUUKAKKOOPEN_TOKEN;
		tk += children.GetLength()-1;
		if(!pst->MakeTree()) return false;
		tk += CHUUKAKKOCLOSE_TOKEN;
		if(src.GetToken(t) != DAIKAKKOCLOSE_TOKEN){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"']' ܂B");
			return false;
		}
		return true;
	}
	bool Literal(Src &src, const PWCArray<intptr_t> &type, PWCArray<intptr_t> &tk)
	{
		intptr_t t;
		PWCArray<int8_t> *plit;
		PWCArray<intptr_t> last;
		TokenKind kin;
		if(kakutei){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ǉŔz̃ëmۂł܂B");
			return false;
		}
		plit = &stat->literallist[stat->literallist.GetLength()];
		plit->SetLength(sizeof(Reference)+sizeof(HeapObjHead));
		kin = src.GetToken(t);
		if(kin != DAIKAKKOCLOSE_TOKEN) while(true){
			ConstShiki(src, t, NULL_TOKEN, kin, last, DAIKAKKOCLOSE_TOKEN, false);
			if(!ConstCaster(last, type)){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^G[łB");
				return false;
			}
			plit->AddTail((int8_t*)(last.GetData()+2), GetTypeSize(last));
			if(src.source.GetConst()[t] == L','){
				kin = src.GetToken(t);
			}else{
				break;
			}
		}
		((Reference*)plit->GetData())->position = 0;
		((Reference*)plit->GetData())->length = plit->GetLength() - (sizeof(Reference)+sizeof(HeapObjHead));
		((HeapObjHead*)(plit->GetData()+sizeof(Reference)))->refcount = 1;
		((HeapObjHead*)(plit->GetData()+sizeof(Reference)))->mutex = NULL;
		((HeapObjHead*)(plit->GetData()+sizeof(Reference)))->datasize = plit->GetLength() - (sizeof(Reference)+sizeof(HeapObjHead));
		tk += CONST_TOKEN;
		tk += REF_TOKEN;
		tk += type;
		tk += stat->literaltotal;
		stat->literaltotal += plit->GetLength();
		return true;
	}
	bool ConstMojiretsu(Src &src, const PWCArray<wchar_t> &str, PWCArray<intptr_t> &tk)
	{
		PWCArray<int8_t> *plit;
		if(kakutei){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ǉŕ̃ëmۂł܂B");
			return false;
		}
		plit = &stat->literallist[stat->literallist.GetLength()];
		plit->SetLength(sizeof(Reference)+sizeof(HeapObjHead));
		plit->AddTail((int8_t*)str.GetConst(), str.GetLength()*sizeof(wchar_t));
		((Reference*)plit->GetData())->position = 0;
		((Reference*)plit->GetData())->length = plit->GetLength() - (sizeof(Reference)+sizeof(HeapObjHead));
		((HeapObjHead*)(plit->GetData()+sizeof(Reference)))->refcount = 1;
		((HeapObjHead*)(plit->GetData()+sizeof(Reference)))->mutex = NULL;
		((HeapObjHead*)(plit->GetData()+sizeof(Reference)))->datasize = plit->GetLength() - (sizeof(Reference)+sizeof(HeapObjHead));
		tk += CONST_TOKEN;
		tk += REF_TOKEN;
		tk += CONST_TOKEN;
		tk += CHAR_TOKEN;
		tk += stat->literaltotal;
		stat->literaltotal += plit->GetLength();
		return true;
	}
	bool ConstCast(Src &src, intptr_t &t, TokenKind kin, PWCArray<intptr_t> &type, SourceTree &hkishiki)
	{
		PWCArray<intptr_t> typ;
		if(src.GetToken(t) != SHOUKAKKOCLOSE_TOKEN){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"')' ܂B");
			return false;
		}
		typ = CONST_TOKEN;
		typ += kin;
		kin = src.GetToken(t);
		if(kin == DAIKAKKOOPEN_TOKEN){
			type.Clear();
			if(!hkishiki.Literal(src, typ, type)) return false;
		}else{
			if(!ConstNanika(src, t, kin, type, hkishiki)) return false;
			if(!ConstCaster(type, typ)){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"sȃLXgłB");
				return false;
			}
		}
		return true;
	}
	bool ConstNanika(Src &src, intptr_t &t, const TokenKind kin, PWCArray<intptr_t> &last, SourceTree &hkishiki)
	{
		Hensuu<SourceTree> *phs;
		PWCArray<wchar_t> name, tmp;
		PWCArray<intptr_t> type;
		TokenKind ki;
		wchar_t wc;
		intptr_t oi, ot, ogy, ii, tmpi, size;
		int32_t fi;
		switch(kin){
		case MBR_TOKEN:
			if(frp == root || frp->parent == root || frp->selftype == CLASS_BLOCK){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"mbr͎g܂B");
				return false;
			}
			switch(src.GetToken(t)){
			case POINT_TOKEN:
				if(src.GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(!frp->parent->ConstNanika(src, t, OTHERKIND_TOKEN, last, hkishiki)) return false;
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			return true;
		case TILDE_TOKEN:
			switch(src.GetToken(t)){
			case POINT_TOKEN:
				if(src.GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(!root->ConstNanika(src, t, OTHERKIND_TOKEN, last, hkishiki)) return false;
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			return true;
		case COLOCOLO_TOKEN:
		case TENTEN_TOKEN:
			src.idx = t;
			return ConstNanikaPath(src, last, hkishiki);
		case OTHERKIND_TOKEN:
			if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
				last.Clear();
				if(!Suuji(last, src, t)) return false;
			}else{
				name.Clear();
				name.AddTail(src.source.GetConst()+t, src.idx-t);
				oi = src.idx;
				ot = t;
				ogy = src.gyousuu;
				switch(src.GetToken(t)){
				case SHOUKAKKOOPEN_TOKEN:
				case BRACOLOOPEN_TOKEN:
				case COLOYAJIRUSHI_TOKEN:
				case COLOCOLO_TOKEN:
					src.idx = ot;
					src.gyousuu = ogy;
					if(!ConstNanikaPath(src, last, hkishiki)) return false;
					break;
				default:
					src.idx = oi;
					t = ot;
					src.gyousuu = ogy;
					if((phs = GetHensuu(name, ii, false)) == NULL){
						if((fi = GetFuncId(name, false)) < 0 ){
							tmp = L'\'';
							tmp += name;
							tmp += L"' ͒`Ă܂B";
							tmp += L'\0';
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
							return false;
						}
						if(stat->funclist.Get(fi)->selftype != FUNC_BLOCK){
							tmp = L'\'';
							tmp += name;
							tmp += L"' class܂enumłB";
							tmp += L'\0';
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
							return false;
						}
						if(!stat->funclist.Get(fi)->kakutei){
							tmp = L'\'';
							tmp += name;
							tmp += L"' ͊m肵ĂȂߊ֐|C^ł܂B";
							tmp += L'\0';
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
							return false;
						}
						if(stat->funclist.Get(fi)->parent == stat->funclist.Get(fi)->root){
							last = CONST_TOKEN;
							last += FUNC_TOKEN;
							if(!stat->funclist.Get(fi)->FuncTypeGet(src, last, true, hkishiki)) return false;
						}else{
							last = CONST_TOKEN;
							last += METHOD_TOKEN;
							last += stat->funclist.Get(fi)->parent->funcid;
							if(!stat->funclist.Get(fi)->FuncTypeGet(src, last, true, hkishiki)) return false;
						}
						break;
					}
					switch(phs->type.GetConst()[0]){
					case LIB_TOKEN:
						if(src.GetToken(t) != POINT_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"lib^͒l܂B");
							return false;
						}
						if(src.GetToken(t) != OTHERKIND_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						if(!((SourceTree *)phs->type.GetConst()[1])->ConstNanika(src, t, OTHERKIND_TOKEN, last, hkishiki)) return false;
						break;
					case TYPE_TOKEN:
						src.idx = ot;
						src.gyousuu = ogy;
						if(!ConstNanikaPath(src, last, hkishiki)) return false;
						break;
					case CONST_TOKEN:
						last = phs->type;
						break;
					default:
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"萔ɒ萔ȊOgƂĂ܂B");
						return false;
					}
					break;
				}
			}
			return true;
		case DQUOT_TOKEN:
		case BACKSLA_TOKEN:
			name.Clear();
			src.idx = t;
			if(!MjiretuLiteral(name, src)) return false;
			last.Clear();
			if(!hkishiki.ConstMojiretsu(src, name, last)) return false;
			return true;
		case SQUOT_TOKEN:
			last = CONST_TOKEN;
			last += CHAR_TOKEN;
			if(!Moji(wc, src)) return false;
			last += wc;
			return true;
		case FALSE_TOKEN:
			last = CONST_TOKEN;
			last += BOOL_TOKEN;
			last += (intptr_t)false;
			return true;
		case TRUE_TOKEN:
			last = CONST_TOKEN;
			last += BOOL_TOKEN;
			last += (intptr_t)true;
			return true;
		case SIZEOF_TOKEN:
			if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
				return false;
			}
			type.Clear();
			if(!TypeNanika(type, src, false, false, false, hkishiki, false)) return false;
			if(src.GetToken(t) != SHOUKAKKOCLOSE_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"')' ܂B");
				return false;
			}
			if((size = GetTypeSize(type)) < 0){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"TCY߂܂B");
				return false;
			}
			last = CONST_TOKEN;
			last += INDEX_TOKEN;
			last.SetLength(last.GetLength()+(sizeof(intptr_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(intptr_t*)(last.GetConst()+last.GetLength()-(sizeof(intptr_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = size;
			return true;
		case TYPEIDOF_TOKEN:
			if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
				return false;
			}
			type.Clear();
			if(!TypeNanika(type, src, false, false, false, hkishiki, false)) return false;
			if(src.GetToken(t) != SHOUKAKKOCLOSE_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"')' ܂B");
				return false;
			}
			if((fi = GetTypeId(type)) == UNKNOWN_TYPEID){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^CvID߂܂B");
				return false;
			}
			last = CONST_TOKEN;
			last += INT_TOKEN;
			last.SetLength(last.GetLength()+(sizeof(int32_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
			*(int32_t*)(last.GetConst()+last.GetLength()-(sizeof(int32_t)+sizeof(intptr_t)-1)/sizeof(intptr_t)) = fi;
			return true;
		case SHOUKAKKOOPEN_TOKEN:
			ki = src.GetToken(t);
			switch(ki){
			case BOOL_TOKEN:
			case BYTE_TOKEN:
			case CHAR_TOKEN:
			case DOUBLE_TOKEN:
			case FLOAT_TOKEN:
			case INT_TOKEN:
			case LONG_TOKEN:
			case SHORT_TOKEN:
			case UBYTE_TOKEN:
			case UINT_TOKEN:
			case ULONG_TOKEN:
			case USHORT_TOKEN:
			case INDEX_TOKEN:
				if(!ConstCast(src, t, ki, last, hkishiki)) return false;
				break;
			case SHOUKAKKOCLOSE_TOKEN:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"̏ʂłB");
				return false;
			case OTHERKIND_TOKEN:
			case MBR_TOKEN:
			case TILDE_TOKEN:
				oi = src.idx;
				ot = t;
				ogy = src.gyousuu;
				src.idx = t;
				type.Clear();
				if(TypeNanika(type, src, false, false, false, *this, false) && type.GetConst()[0] != NULL_TOKEN){
					switch(type.GetConst()[0]){
					case BOOL_TOKEN:
					case BYTE_TOKEN:
					case CHAR_TOKEN:
					case DOUBLE_TOKEN:
					case FLOAT_TOKEN:
					case INT_TOKEN:
					case LONG_TOKEN:
					case SHORT_TOKEN:
					case UBYTE_TOKEN:
					case UINT_TOKEN:
					case ULONG_TOKEN:
					case USHORT_TOKEN:
					case INDEX_TOKEN:
						if(!ConstCast(src, t, (TokenKind)type.GetConst()[0], last, hkishiki)) return false;
						break;
					default:
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"̌^ɂ̓LXgł܂B");
						return false;
					}
					break;
				}
				if(stat->tokubetuerr) return false;
				stat->ss->ErrorMessage.Clear();
				src.idx = oi;
				t = ot;
				src.gyousuu = ogy;
			default:
				if(!hkishiki.ConstShiki(src, t, NULL_TOKEN, ki, last, SHOUKAKKOCLOSE_TOKEN, false)) return false;
				break;
			}
			return true;
		case PULUS_TOKEN:
		case MINUS_TOKEN:
		case EXCLAMATION_TOKEN:
			if(!ConstNanika(src, t, src.GetToken(t), last, hkishiki)) return false;
			tmpi = 0;
			if(last.GetConst()[tmpi] != CONST_TOKEN) return false;
			tmpi++;
			switch(last.GetConst()[tmpi++]){
			case BYTE_TOKEN:
				TankouConst(kin, *(int8_t*)(last.GetConst()+tmpi));
				break;
			case SHORT_TOKEN:
				TankouConst(kin, *(int16_t*)(last.GetConst()+tmpi));
				break;
			case INT_TOKEN:
				TankouConst(kin, *(int32_t*)(last.GetConst()+tmpi));
				break;
			case LONG_TOKEN:
				TankouConst(kin, *(int64_t*)(last.GetConst()+tmpi));
				break;
			case UBYTE_TOKEN:
				TankouConst(kin, *(uint8_t*)(last.GetConst()+tmpi));
				break;
			case USHORT_TOKEN:
				TankouConst(kin, *(uint16_t*)(last.GetConst()+tmpi));
				break;
			case UINT_TOKEN:
				TankouConst(kin, *(uint32_t*)(last.GetConst()+tmpi));
				break;
			case ULONG_TOKEN:
				TankouConst(kin, *(uint64_t*)(last.GetConst()+tmpi));
				break;
			case INDEX_TOKEN:
				TankouConst(kin, *(intptr_t*)(last.GetConst()+tmpi));
				break;
			case FLOAT_TOKEN:
				if(kin == EXCLAMATION_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'!' ͕_^ɂ͎g܂B");
					return false;
				}
				TankouConstReal(kin, *(float*)(last.GetConst()+tmpi));
				break;
			case DOUBLE_TOKEN:
				if(kin == EXCLAMATION_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'!' ͕_^ɂ͎g܂B");
					return false;
				}
				TankouConstReal(kin, *(double*)(last.GetConst()+tmpi));
				break;
			case CHAR_TOKEN:
				TankouConst(kin, *(wchar_t*)(last.GetConst()+tmpi));
				break;
			case BOOL_TOKEN:
				if(kin != EXCLAMATION_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"bool^ɕ͂܂B");
					return false;
				}
				*(bool*)(last.GetConst()+tmpi) = !*(bool*)(last.GetConst()+tmpi);
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^G[łB");
				return false;
			}
			return true;
		}
		stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
		return false;
	}
	template<typename TYPE> bool NikouConst(const TokenKind to, TYPE &x, PWCArray<intptr_t> &test, const TYPE y)
	{
		switch(to){
		case KAKERU_TOKEN:
			x *= y;
			break;
		case WARU_TOKEN:
			if(y == 0) return false;
			x /= y;
			break;
		case AMARI_TOKEN:
			if(y == 0) return false;
			x %= y;
			break;
		case PULUS_TOKEN:
			x += y;
			break;
		case MINUS_TOKEN:
			x -= y;
			break;
		case LSHIFT_TOKEN:
			x <<= y;
			break;
		case RSHIFT_TOKEN:
			x >>= y;
			break;
		case SHOU_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x < y);
			break;
		case SHOUOREQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x <= y);
			break;
		case DAI_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x > y);
			break;
		case DAIOREQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x >= y);
			break;
		case EQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x == y);
			break;
		case NOTEQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x != y);
			break;
		case AND_TOKEN:
			x &= y;
			break;
		case XOR_TOKEN:
			x ^= y;
			break;
		case OR_TOKEN:
			x |= y;
			break;
		default:
			return false;
		}
		return true;
	}
	template<typename TYPE> bool NikouConstReal(const TokenKind to, TYPE &x, PWCArray<intptr_t> &test, const TYPE y)
	{
		switch(to){
		case KAKERU_TOKEN:
			x *= y;
			break;
		case WARU_TOKEN:
			x /= y;
			break;
		case PULUS_TOKEN:
			x += y;
			break;
		case MINUS_TOKEN:
			x -= y;
			break;
		case SHOU_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x < y);
			break;
		case SHOUOREQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x <= y);
			break;
		case DAI_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x > y);
			break;
		case DAIOREQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x >= y);
			break;
		case EQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x == y);
			break;
		case NOTEQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x != y);
			break;
		default:
			return false;
		}
		return true;
	}
	bool NikouConstBool(const TokenKind to, bool &x, const bool y)
	{
		switch(to){
		case EQUAL_TOKEN:
			x = x == y;
			break;
		case NOTEQUAL_TOKEN:
			x = x != y;
			break;
		case AND_TOKEN:
			x = x && y;
			break;
		case XOR_TOKEN:
			x = x != y;
			break;
		case OR_TOKEN:
			x = x || y;
			break;
		case ANDAND_TOKEN:
			x = x && y;
			break;
		case OROR_TOKEN:
			x = x || y;
			break;
		default:
			return false;
		}
		return true;
	}
	bool NikouConstEnum(const TokenKind to, const int32_t x, PWCArray<intptr_t> &test, const int32_t y)
	{
		switch(to){
		case SHOU_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x < y);
			break;
		case SHOUOREQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x <= y);
			break;
		case DAI_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x > y);
			break;
		case DAIOREQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x >= y);
			break;
		case EQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x == y);
			break;
		case NOTEQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x != y);
			break;
		default:
			return false;
		}
		return true;
	}
	bool NikouConstFunc(const TokenKind to, const intptr_t x, PWCArray<intptr_t> &test, const intptr_t y)
	{
		switch(to){
		case EQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x == y);
			break;
		case NOTEQUAL_TOKEN:
			test = CONST_TOKEN;
			test += BOOL_TOKEN;
			test += (intptr_t)(x != y);
			break;
		default:
			return false;
		}
		return true;
	}
	bool EnzanConst(const TokenKind kin, PWCArray<intptr_t> &test, PWCArray<intptr_t> &last)
	{
		intptr_t tmpi;
		if(!ConstCaster(last, test)) return false;
		tmpi = 0;
		if(test.GetConst()[tmpi] != CONST_TOKEN) return false;
		tmpi++;
		switch(test.GetConst()[tmpi++]){
		case BYTE_TOKEN:
			if(!NikouConst(kin, *(int8_t*)(test.GetConst()+tmpi), test, *(int8_t*)(last.GetConst()+tmpi))) return false;
			break;
		case SHORT_TOKEN:
			if(!NikouConst(kin, *(int16_t*)(test.GetConst()+tmpi), test, *(int16_t*)(last.GetConst()+tmpi))) return false;
			break;
		case INT_TOKEN:
			if(!NikouConst(kin, *(int32_t*)(test.GetConst()+tmpi), test, *(int32_t*)(last.GetConst()+tmpi))) return false;
			break;
		case LONG_TOKEN:
			if(!NikouConst(kin, *(int64_t*)(test.GetConst()+tmpi), test, *(int64_t*)(last.GetConst()+tmpi))) return false;
			break;
		case UBYTE_TOKEN:
			if(!NikouConst(kin, *(uint8_t*)(test.GetConst()+tmpi), test, *(uint8_t*)(last.GetConst()+tmpi))) return false;
			break;
		case USHORT_TOKEN:
			if(!NikouConst(kin, *(uint16_t*)(test.GetConst()+tmpi), test, *(uint16_t*)(last.GetConst()+tmpi))) return false;
			break;
		case UINT_TOKEN:
			if(!NikouConst(kin, *(uint32_t*)(test.GetConst()+tmpi), test, *(uint32_t*)(last.GetConst()+tmpi))) return false;
			break;
		case ULONG_TOKEN:
			if(!NikouConst(kin, *(uint64_t*)(test.GetConst()+tmpi), test, *(uint64_t*)(last.GetConst()+tmpi))) return false;
			break;
		case INDEX_TOKEN:
			if(!NikouConst(kin, *(intptr_t*)(test.GetConst()+tmpi), test, *(intptr_t*)(last.GetConst()+tmpi))) return false;
			break;
		case FLOAT_TOKEN:
			if(!NikouConstReal(kin, *(float*)(test.GetConst()+tmpi), test, *(float*)(last.GetConst()+tmpi))) return false;
			break;
		case DOUBLE_TOKEN:
			if(!NikouConstReal(kin, *(double*)(test.GetConst()+tmpi), test, *(double*)(last.GetConst()+tmpi))) return false;
			break;
		case CHAR_TOKEN:
			if(!NikouConst(kin, *(wchar_t*)(test.GetConst()+tmpi), test, *(wchar_t*)(last.GetConst()+tmpi))) return false;
			break;
		case BOOL_TOKEN:
			if(!NikouConstBool(kin, *(bool*)(test.GetConst()+tmpi), *(bool*)(last.GetConst()+tmpi))) return false;
			break;
		case ENUM_TOKEN:
			if(!NikouConstEnum(kin, *(int32_t*)(test.GetConst()+test.GetLength()-1), test, *(int32_t*)(last.GetConst()+last.GetLength()-1))) return false;
			break;
		case FUNC_TOKEN:
			if(!NikouConstFunc(kin, *(intptr_t*)(test.GetConst()+test.GetLength()-1), test, *(intptr_t*)(last.GetConst()+last.GetLength()-1))) return false;
			break;
		case METHOD_TOKEN:
			if(!NikouConstFunc(kin, *(intptr_t*)(test.GetConst()+test.GetLength()-1), test, *(intptr_t*)(last.GetConst()+last.GetLength()-1))) return false;
			break;
		default:
			return false;
		}
		return true;
	}
	bool ConstShiki(Src &src, intptr_t &t, const TokenKind enzansi, const TokenKind oskin, PWCArray<intptr_t> &last, const TokenKind end, const bool bubun)
	{
		PWCArray<intptr_t> type;
		TokenKind kin;
		kin = oskin;
		PWCArray<intptr_t> test;
		intptr_t oi, ot, ogy;
		if(kin != end){
			if(!ConstNanika(src, t, kin, last, *this)) return false;
			test = last;
			ot = t;
			oi = src.idx;
			ogy = src.gyousuu;
			while(KAKERU_TOKEN <= (kin = src.GetToken(t)) && kin <= QUESTION_TOKEN){
				switch(enzansi){
				case NULL_TOKEN:
					break;
				case KAKERU_TOKEN:case WARU_TOKEN:case AMARI_TOKEN:
					src.idx = t;
					return true;
				case PULUS_TOKEN:case MINUS_TOKEN:
					if(kin >= PULUS_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case LSHIFT_TOKEN:case RSHIFT_TOKEN:
					if(kin >= LSHIFT_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case SHOU_TOKEN:case SHOUOREQUAL_TOKEN:case DAI_TOKEN:case DAIOREQUAL_TOKEN:
					if(kin >= SHOU_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case EQUAL_TOKEN:case NOTEQUAL_TOKEN:
					if(kin >= EQUAL_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case AND_TOKEN:
					if(kin >= AND_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case XOR_TOKEN:
					if(kin >= XOR_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case OR_TOKEN:
					if(kin >= OR_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case ANDAND_TOKEN://E
					if(kin >= OROR_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case OROR_TOKEN://E
					if(kin >= QUESTION_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case QUESTION_TOKEN://E
					break;
				default:
					if(bubun) break;
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(src.GetToken(t) == end){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				src.idx = t;
				if(kin == QUESTION_TOKEN){
					type = CONST_TOKEN;
					type += BOOL_TOKEN;
					if(!ConstCaster(test, type)){
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^G[łB");
						return false;
					}
					if(*(bool*)(test.GetConst()+type.GetLength())){
						if(!ConstShiki(src, t, QUESTION_TOKEN, src.GetToken(t), last, COLON_TOKEN, false)) return false;
						if(src.source.GetConst()[t] != L':'){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"':' ܂B");
							return false;
						}
						if(!ConstShiki(src, t, QUESTION_TOKEN, src.GetToken(t), test, end, false)) return false;
					}else{
						if(!ConstShiki(src, t, QUESTION_TOKEN, src.GetToken(t), last, COLON_TOKEN, false)) return false;
						if(src.source.GetConst()[t] != L':'){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"':' ܂B");
							return false;
						}
						if(!ConstShiki(src, t, QUESTION_TOKEN, src.GetToken(t), last, end, false)) return false;
					}
				}else{
					if(!ConstShiki(src, t, kin, src.GetToken(t), last, end, false)){
						if(!bubun) return false;
						if(stat->tokubetuerr) return false;
						stat->ss->ErrorMessage.Clear();
						last = test;
						t = ot;
						src.idx = oi;
						src.gyousuu = ogy;
						return true;
					}
					if(!EnzanConst(kin, test, last)){
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"萔̃G[łB");
						return false;
					}
					last = test;
				}
				ot = t;
				oi = src.idx;
				ogy = src.gyousuu;
			}
		}else{
			if(bubun) return false;
		}
		if(bubun){
			src.idx = t;
			return true;
		}
		if(enzansi != NULL_TOKEN && !(enzansi == QUESTION_TOKEN && end == COLON_TOKEN)) src.idx = t;
		if(kin != end && (end != COLON_TOKEN || kin != COLOYAJIRUSHI_TOKEN)){
			if(end == SHOUKAKKOCLOSE_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"')' ܂B");
				return false;
			}
			if(kin != COMMA_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
		}
		return true;
	}
	bool NanikaPath(PWCArray<intptr_t> &tk, Src &src, const bool pub, SourceTree &hkishiki)
	{
		int32_t tyki;
		SourceTree *pst;
		PWCArray<wchar_t> name, tmp;
		PWCArray<intptr_t> type;
		const Hensuu<SourceTree> *phs;
		intptr_t t, ii, jj;
		bool temp, temp2;
		if((tyki = PathtoClassID(src, FUNC_BLOCK, true, pub, hkishiki, true)) < 0) return false;
		pst = stat->funclist.Get(tyki);
		switch(pst->selftype){
		case FUNC_BLOCK:
			if(pst != pst->root){
				if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
					if(frp == root){
						tk += TILDE_TOKEN;
					}else if(frp == hkishiki.frp->parent){
						tk += MBR_TOKEN;
					}
					if(pst->parent == pst->root){
						src.idx = t;
						tk += CONST_TOKEN;
						tk += FUNC_TOKEN;
						if(!pst->FuncTypeGet(src, tk, true, hkishiki)) return false;
					}else{
						src.idx = t;
						tk += CONST_TOKEN;
						tk += METHOD_TOKEN;
						tk += pst->parent->funcid;
						if(!pst->FuncTypeGet(src, tk, true, hkishiki)) return false;
					}
				}else{
					if(pst->parent == pst->root){
						tk += TILDE_TOKEN;
					}else if(pst->parent == hkishiki.frp->parent){
						tk += MBR_TOKEN;
					}else if(pst->parent != frp){
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"̊֐͌Ăׂ܂B");
						return false;
					}
					if(!pst->GblhaniSet(src, hkishiki)) return false;
					if(!pst->MbrhaniSet(src, hkishiki)) return false;
					tk += SHOUKAKKOOPEN_TOKEN;
					tk += ~tyki;
					temp = pst->rettype.GetLength() == 0 || (pst->rettype.GetLength() == 1 && IsPublicToken((TokenKind)pst->rettype.GetConst()[0]));
					temp2 = pst->hikisuucount > 0 && pst->hensuname.GetConst()[pst->tpcount].GetConst()[0] == L'0';
					if(temp){
						type = REF_TOKEN;
						type += NULL_TOKEN;
						ii = hkishiki.AddHiddenHensuu(&tk, type, sizeof(DynamicRef));
						pst->backpatch += (intptr_t)hkishiki.frp;
						pst->breakpatch += ii;
					}else if(temp2){
						phs = &pst->funchensuu.GetConst()[pst->hensuidx.GetConst()[pst->tpcount]];
						type = phs->type;
						type.Pop();
						ii = hkishiki.AddHiddenHensuu(&tk, type, (phs->type.GetConst()[1] == NULL_TOKEN ? sizeof(DynamicRef) : sizeof(Reference)));
					}
					if(temp || temp2){
						if(hkishiki.frp == hkishiki.root) tk += TILDE_TOKEN;
						tk += ~ii;
						tk += NULL_TOKEN;
						tk += DAINYUU_TOKEN;
						tk += COMMA_TOKEN;
					}
					jj = tk.GetLength();
					if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
					if(src.GetToken(t) == SOTOCOMMA_TOKEN){
						if(tk.GetLength() > jj) tk += COMMA_TOKEN;
						if(!hkishiki.CSorNanika(tk, src, t, SHARP_TOKEN, src.GetToken(t))) return false;
					}else{
						src.idx = t;
					}
					tk += SHOUKAKKOCLOSE_TOKEN;
					if(temp){
						tk += NULL_TOKEN;
					}
					if(temp || temp2){
						tk += SHARP_TOKEN;
						if(hkishiki.frp == hkishiki.root) tk += TILDE_TOKEN;
						tk += ~ii;
					}
				}
				break;
			}
		case CLASS_BLOCK:
			src.GetToken(t);
			src.GetToken(t);
			name.Clear();
			name.AddTail(src.source.GetConst()+t, src.idx-t);
			if((phs = pst->GetHensuu(name, ii, true)) == NULL){
				tmp = L'\'';
				tmp += name;
				tmp += L"' o萔Ɍ܂B";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
				return false;
			}
			if(frp == root){
				tk += TILDE_TOKEN;
			}else if(frp == hkishiki.frp->parent){
				tk += MBR_TOKEN;
			}
			tk += phs->type;
			break;
		case ENUM_BLOCK:
			src.GetToken(t);
			if(src.GetToken(t) != OTHERKIND_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			name.Clear();
			name.AddTail(src.source.GetConst()+t, src.idx-t);
			if((phs = pst->GetHensuu(name, ii, true)) == NULL){
				tmp = L'\'';
				tmp += name;
				tmp += L"' ܂B";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
				return false;
			}
			if(frp == root){
				tk += TILDE_TOKEN;
			}else if(frp == hkishiki.frp->parent){
				tk += MBR_TOKEN;
			}
			tk += phs->type;
			break;
		default:
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
			return false;
		}
		return true;
	}
	bool RefDotNanika(PWCArray<intptr_t> &tk, Src &src, const intptr_t *const iap)
	{
		intptr_t t;
		int32_t ci;
		PWCArray<wchar_t> name;
		switch(src.GetToken(t)){
		case NEW_TOKEN:
			if(iap[0] == NULL_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
				return false;
			}
			if(iap[0] == CLASS_TOKEN && !stat->funclist.Get((int32_t)iap[1])->kakutei){
				name = L"default";
				if((ci = stat->funclist.Get((int32_t)iap[1])->GetFuncId(name, true)) < 0){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
					return false;
				}
				if(!stat->funclist.Get(ci)->GblhaniSet(src, *this)) return false;
				name = L"new";
				if((ci = stat->funclist.Get((int32_t)iap[1])->GetFuncId(name, true)) < 0){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
					return false;
				}
				if(!stat->funclist.Get(ci)->GblhaniSet(src, *this)) return false;
				name = L"break";
				if((ci = stat->funclist.Get((int32_t)iap[1])->GetFuncId(name, true)) < 0){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
					return false;
				}
				if(!stat->funclist.Get(ci)->GblhaniSet(src, *this)) return false;
				name = L"delete";
				if((ci = stat->funclist.Get((int32_t)iap[1])->GetFuncId(name, true)) < 0){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
					return false;
				}
				if(!stat->funclist.Get(ci)->GblhaniSet(src, *this)) return false;
			}
			tk += NEW_TOKEN;
			switch(src.GetToken(t)){
			case SHOUKAKKOOPEN_TOKEN:
				tk += SHOUKAKKOOPEN_TOKEN;
				if(!Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
				tk += SHOUKAKKOCLOSE_TOKEN;
				break;
			case DAIKAKKOOPEN_TOKEN:
				tk += DAIKAKKOOPEN_TOKEN;
				if(!Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), DAIKAKKOCLOSE_TOKEN)) return false;
				tk += DAIKAKKOCLOSE_TOKEN;
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			break;
		case TYPESIZE_TOKEN:
			if(iap[0] != NULL_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
				return false;
			}
			tk += TYPESIZE_TOKEN;
			break;
		case TYPEID_TOKEN:
			if(iap[0] != NULL_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
				return false;
			}
			tk += TYPEID_TOKEN;
			break;
		case CAST_TOKEN:
			if(iap[0] != NULL_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
				return false;
			}
			if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
				return false;
			}
			tk += CAST_TOKEN;
			tk += SHOUKAKKOOPEN_TOKEN;
			if(!Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
			tk += SHOUKAKKOCLOSE_TOKEN;
			break;
		default:
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
			return false;
		}
		return true;
	}
	bool ClassNanika(PWCArray<intptr_t> &tk, Src &src, SourceTree &hkishiki, const PWCArray<intptr_t> &type, intptr_t &jj, const bool coref)
	{
		intptr_t t;
		int32_t tyki;
		tk += POINT_TOKEN;
		jj++;
		tyki = (int32_t)type.GetConst()[jj];
		if(src.GetToken(t) != OTHERKIND_TOKEN){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
			return false;
		}
		if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
			return false;
		}
		if(!stat->funclist.Get(tyki)->Nanika(tk, src, t, OTHERKIND_TOKEN, !coref&&(parent==NULL||tyki!=parent->funcid), hkishiki)) return false;
		return true;
	}
	bool Moji(wchar_t &wc, Src &src)
	{
		wc = src.GetChar();
		switch(wc){
		case L'\'':
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
			return false;
		case L'\\':
			if(!TokushuMoji(wc, src)) return false;
			break;
		}
		if(src.GetChar() != L'\''){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
			return false;
		}
		return true;
	}
	bool Object(PWCArray<intptr_t> &tk, Src &src, intptr_t &t, TokenKind kin, SourceTree &hkishiki, Hensuu<SourceTree> *const phs, intptr_t &jj)
	{
		int32_t fi;
		if(phs->type.GetConst()[jj] == REF_TOKEN || phs->type.GetConst()[jj] == LIST_TOKEN){
			if(phs->type.GetConst()[jj+1] == NULL_TOKEN){
				if(kin == POINT_TOKEN){
					jj++;
					if(!hkishiki.RefDotNanika(tk, src, phs->type.GetConst()+jj)) return false;
				}else{
					src.idx = t;
				}
			}else switch(kin){
			case DAIKAKKOOPEN_TOKEN:
			case DIA_TOKEN:
				while(true){
					tk += kin;
					if(kin == DAIKAKKOOPEN_TOKEN){
						if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), DAIKAKKOCLOSE_TOKEN)) return false;
						tk += DAIKAKKOCLOSE_TOKEN;
						if(tk.GetConst()[tk.GetLength()-2] == TENTEN_TOKEN) jj--;
					}
					switch(kin = src.GetToken(t)){
					case POINT_TOKEN:
						jj++;
						if(IsPublicToken((TokenKind)phs->type.GetConst()[jj])) jj++;
						if(phs->type.GetConst()[jj] == CLASS_TOKEN || phs->type.GetConst()[jj] == ENUM_TOKEN){
							if(!ClassNanika(tk, src, hkishiki, phs->type, jj, false)) return false;
						}else if(phs->type.GetConst()[jj] == REF_TOKEN || phs->type.GetConst()[jj] == LIST_TOKEN){
							jj++;
							if(!hkishiki.RefDotNanika(tk, src, phs->type.GetConst()+jj)) return false;
							if(src.source.GetConst()[src.idx-1] == L']' && !Object(tk, src, t, src.GetToken(t), hkishiki, phs, jj)) return false;
						}else{
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"炩̃G[łB");
							return false;
						}
						break;
					case TENTEN_TOKEN:
						jj++;
						if(phs->type.GetConst()[jj] == THREAD_TOKEN){
							if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
								return false;
							}
							fi = phs->type.GetConst()[jj+1];
							if(!stat->funclist.Get(fi)->GblhaniSet(src, hkishiki)) return false;
							tk += SHOUKAKKOOPEN_TOKEN;
							tk += TENTEN_TOKEN;
							if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
							tk += SHOUKAKKOCLOSE_TOKEN;
						}else{
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'..' słB");
							return false;
						}
						break;
					case DAIKAKKOOPEN_TOKEN:
					case DIA_TOKEN:
						jj++;
						if(IsPublicToken((TokenKind)phs->type.GetConst()[jj])) jj++;
						if(phs->type.GetConst()[jj] != REF_TOKEN && phs->type.GetConst()[jj] != LIST_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"炩̃G[łB");
							return false;
						}
						continue;
						break;
					default:
						src.idx = t;
						break;
					}
					break;
				}
				break;
			case POINT_TOKEN:
				jj++;
				if(!hkishiki.RefDotNanika(tk, src, phs->type.GetConst()+jj)) return false;
				if(src.source.GetConst()[src.idx-1] == L']' && !Object(tk, src, t, src.GetToken(t), hkishiki, phs, jj)) return false;
				break;
			default:
				src.idx = t;
				break;
			}
		}else if(phs->type.GetConst()[jj] == THREAD_TOKEN) switch(kin){
		case TENTEN_TOKEN:
			if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
				return false;
			}
			fi = phs->type.GetConst()[jj+1];
			if(!stat->funclist.Get(fi)->GblhaniSet(src, hkishiki)) return false;
			tk += SHOUKAKKOOPEN_TOKEN;
			tk += TENTEN_TOKEN;
			if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
			tk += SHOUKAKKOCLOSE_TOKEN;
			break;
		default:
			src.idx = t;
			break;
		}else switch(kin){
		case POINT_TOKEN:
			if(phs->type.GetConst()[0] == LIB_TOKEN){
				if(src.GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				tk.Pop();
				tk += phs->type;
				if(!((SourceTree*)phs->type.GetConst()[1])->Nanika(tk, src, t, OTHERKIND_TOKEN, true, hkishiki)) return false;
			}else{
				if(phs->type.GetConst()[jj] != CLASS_TOKEN && phs->type.GetConst()[jj] != ENUM_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'.' słB");
					return false;
				}
				if(!ClassNanika(tk, src, hkishiki, phs->type, jj, jj==1 && phs->type.GetConst()[0]==CORE_TOKEN)) return false;
			}
			break;
		default:
			src.idx = t;
			break;
		}
		return true;
	}
	bool Cast(PWCArray<intptr_t> &tk, Src &src, intptr_t &t, const TokenKind kin, const bool pub, SourceTree &hkishiki)
	{
		PWCArray<intptr_t> type;
		if(src.GetToken(t) != SHOUKAKKOCLOSE_TOKEN){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"')' ܂B");
			return false;
		}
		if(!Nanika(tk, src, t, src.GetToken(t), pub, hkishiki)) return false;
		tk += SHOUKAKKOOPEN_TOKEN;
		tk += kin;
		tk += SHOUKAKKOCLOSE_TOKEN;
		return true;
	}
	bool Nanika(PWCArray<intptr_t> &tk, Src &src, intptr_t &t, const TokenKind kin, const bool pub, SourceTree &hkishiki)
	{
		Hensuu<SourceTree> *phs;
		PWCArray<wchar_t> name, tmp;
		PWCArray<intptr_t> type;
		TokenKind ki;
		int32_t fi;
		intptr_t oi, ot, ogy, ii, jj;
		switch(kin){
		case MBR_TOKEN:
			if(hkishiki.frp == hkishiki.root || hkishiki.frp->parent == hkishiki.root || hkishiki.frp->selftype == CLASS_BLOCK){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"mbr͎g܂B");
				return false;
			}
			switch(src.GetToken(t)){
			case POINT_TOKEN:
				if((ki = src.GetToken(t)) != OTHERKIND_TOKEN && ki != SELF_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(!hkishiki.frp->parent->Nanika(tk, src, t, OTHERKIND_TOKEN, pub, hkishiki)) return false;
				break;
			default:
				if(hkishiki.frp->parent->selftype != CLASS_BLOCK){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXȊOmbr͎QƂł܂B");
					return false;
				}
				tk += NULL_TOKEN;
				tk += MBR_TOKEN;
				src.idx = t;
				break;
			}
			break;
		case TILDE_TOKEN:
			switch(src.GetToken(t)){
			case POINT_TOKEN:
				if(src.GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(!hkishiki.root->Nanika(tk, src, t, OTHERKIND_TOKEN, pub, hkishiki)) return false;
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			break;
		case WAIT_TOKEN:
		case CONSTEVAL_TOKEN:
			if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
				return false;
			}
			tk += kin;
			tk += SHOUKAKKOOPEN_TOKEN;
			if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
			tk += SHOUKAKKOCLOSE_TOKEN;
			break;
		case LEN_TOKEN:
			if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
				return false;
			}
			if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
			tk += LEN_TOKEN;
			break;
		case IDNAME_TOKEN:
			type = REF_TOKEN;
			type += CHAR_TOKEN;
			ii = hkishiki.AddHiddenHensuu(&tk, type, sizeof(Reference));
			if(src.GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(' ܂B");
				return false;
			}
			if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
			tk += IDNAME_TOKEN;
			tk += ii;
			break;
		case SHOUKAKKOOPEN_TOKEN:
			ki = src.GetToken(t);
			switch(ki){
			case BOOL_TOKEN:
			case BYTE_TOKEN:
			case CHAR_TOKEN:
			case DOUBLE_TOKEN:
			case FLOAT_TOKEN:
			case INT_TOKEN:
			case LONG_TOKEN:
			case SHORT_TOKEN:
			case UBYTE_TOKEN:
			case UINT_TOKEN:
			case ULONG_TOKEN:
			case USHORT_TOKEN:
			case INDEX_TOKEN:
			case VOID_TOKEN:
				if(!Cast(tk, src, t, ki, pub, hkishiki)) return false;
				break;
			case SHOUKAKKOCLOSE_TOKEN:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"̏ʂłB");
				return false;
			case OTHERKIND_TOKEN:
			case MBR_TOKEN:
			case TILDE_TOKEN:
				oi = src.idx;
				ot = t;
				ogy = src.gyousuu;
				src.idx = t;
				type.Clear();
				if(TypeNanika(type, src, false, false, false, *this, false) && type.GetConst()[0] != NULL_TOKEN){
					switch(type.GetConst()[0]){
					case BOOL_TOKEN:
					case BYTE_TOKEN:
					case CHAR_TOKEN:
					case DOUBLE_TOKEN:
					case FLOAT_TOKEN:
					case INT_TOKEN:
					case LONG_TOKEN:
					case SHORT_TOKEN:
					case UBYTE_TOKEN:
					case UINT_TOKEN:
					case ULONG_TOKEN:
					case USHORT_TOKEN:
					case INDEX_TOKEN:
					case VOID_TOKEN:
						if(!Cast(tk, src, t, (TokenKind)type.GetConst()[0], pub, hkishiki)) return false;
						break;
					default:
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"̌^ɂ̓LXgł܂B");
						return false;
					}
					break;
				}
				if(stat->tokubetuerr) return false;
				stat->ss->ErrorMessage.Clear();
				src.idx = oi;
				t = ot;
				src.gyousuu = ogy;
			default:
				if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, ki, SHOUKAKKOCLOSE_TOKEN)) return false;
				break;
			}
			break;
		case DAIKAKKOOPEN_TOKEN:
			if(!hkishiki.Delegate(src, tk)) return false;
			break;
		case FALSE_TOKEN:
			tk += CONST_TOKEN;
			tk += BOOL_TOKEN;
			tk += (intptr_t)false;
			break;
		case TRUE_TOKEN:
			tk += CONST_TOKEN;
			tk += BOOL_TOKEN;
			tk += (intptr_t)true;
			break;
		case PULUS_TOKEN:
		case MINUS_TOKEN:
			if(!Nanika(tk, src, t, src.GetToken(t), pub, hkishiki)) return false;
			tk += NULL_TOKEN;
			tk += kin;
			break;
		case EXCLAMATION_TOKEN:
		case PLSPLS_TOKEN:
		case MNSMNS_TOKEN:
			if(!Nanika(tk, src, t, src.GetToken(t), pub, hkishiki)) return false;
			tk += kin;
			break;
		case SELF_TOKEN:
			ot = t;
			ogy = src.gyousuu;
			src.idx = ot;
			src.gyousuu = ogy;
			if(!NanikaPath(tk, src, pub, hkishiki)) return false;
			break;
		case OTHERKIND_TOKEN:
			oi = src.idx;
			ot = t;
			ogy = src.gyousuu;
			switch(ki = src.GetToken(t)){
			case SHOUKAKKOOPEN_TOKEN:
			case BRACOLOOPEN_TOKEN:
			case COLOYAJIRUSHI_TOKEN:
			case COLOCOLO_TOKEN:
				src.idx = ot;
				src.gyousuu = ogy;
				if(!NanikaPath(tk, src, pub, hkishiki)) return false;
				break;
			case DAIKAKKOOPEN_TOKEN:
			case DIA_TOKEN:
			case POINT_TOKEN:
			default:
				name.Clear();
				name.AddTail(src.source.GetConst()+ot, oi-ot);
				if((phs = GetHensuu(name, ii, false)) == NULL){
					if((fi = GetFuncId(name, false)) < 0){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ͒`Ă܂B";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
						return false;
					}else if(stat->funclist.Get(fi)->selftype != FUNC_BLOCK){
						tmp = L'\'';
						tmp += name;
						tmp += L"' class܂enumłB";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
						return false;
					}else if(!stat->funclist.Get(fi)->kakutei){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ͊m肵ĂȂߊ֐|C^ł܂B";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
						return false;
					}
					if(stat->funclist.Get(fi)->parent == stat->funclist.Get(fi)->root){
						src.idx = t;
						tk += CONST_TOKEN;
						tk += FUNC_TOKEN;
						if(!stat->funclist.Get(fi)->FuncTypeGet(src, tk, true, hkishiki)) return false;
					}else{
						src.idx = t;
						tk += CONST_TOKEN;
						tk += METHOD_TOKEN;
						tk += stat->funclist.Get(fi)->parent->funcid;
						if(!stat->funclist.Get(fi)->FuncTypeGet(src, tk, true, hkishiki)) return false;
					}
					break;
				}
				if(phs->type.GetConst()[0] == TYPE_TOKEN){
					src.idx = ot;
					src.gyousuu = ogy;
					if(!NanikaPath(tk, src, pub, hkishiki)) return false;
					break;
				}
				if(frp == root){
					tk += TILDE_TOKEN;
				}else if(frp == hkishiki.frp->parent){
					tk += MBR_TOKEN;
				}
				tk += ~ii;
				if(phs->type.GetConst()[0] == PLUGIN_TOKEN){
					if(ki != KAKKOCOLOOPEN_TOKEN){
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(:' ܂B");
						return false;
					}
					tk += KAKKOCOLOOPEN_TOKEN;
					if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), KAKKOCOLOCLOSE_TOKEN)) return false;
					tk += KAKKOCOLOCLOSE_TOKEN;
					break;
				}
				jj = 0;
				if(phs->type.GetConst()[jj] == CONST_TOKEN){
					jj++;
				}else{
					if(hkishiki.root == frp){
						if(ii >= hkishiki.frp->gblhani){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ÕO[oϐg\܂B");
							return false;
						}
						if(hkishiki.frp->gblsaikouhani < ii) hkishiki.frp->gblsaikouhani = ii;
					}
					if(hkishiki.frp->parent == frp){
						if(ii >= hkishiki.frp->mbrhani){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"Õo[ϐg\܂B");
							return false;
						}
						if(hkishiki.frp->mbrsaikouhani < ii) hkishiki.frp->mbrsaikouhani = ii;
					}
					if(IsPublicToken((TokenKind)phs->type.GetConst()[jj])){
						jj++;
					}else if(pub && (TokenKind)phs->type.GetConst()[jj] != LIB_TOKEN){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ̓vCx[głB";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
						return false;
					}else if(phs->type.GetConst()[jj] == CORE_TOKEN){
						jj++;
					}
				}
				if(!Object(tk, src, t, ki, hkishiki, phs, jj)) return false;
			}
			break;
		default:
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
			return false;
		}
		while(true){
			ki = src.GetToken(t);
			switch(ki){
			case KAKKOCOLOOPEN_TOKEN:
				tk += KAKKOCOLOOPEN_TOKEN;
				if(!hkishiki.Shiki(tk, src, t, NULL_TOKEN, src.GetToken(t), KAKKOCOLOCLOSE_TOKEN)) return false;
				tk += KAKKOCOLOCLOSE_TOKEN;
				continue;
			case PLSPLS_TOKEN:
			case MNSMNS_TOKEN:
				tk += NULL_TOKEN;
				tk += ki;
				continue;
			}
			src.idx = t;
			break;
		}
		return true;
	}
	bool CSorNanika(PWCArray<intptr_t> &tk, Src &src, intptr_t &t, const TokenKind enzansi, const TokenKind kin)
	{
		PWCArray<intptr_t> cons;
		intptr_t oi, ot, ogy;
		ot = t;
		oi = src.idx;
		ogy = src.gyousuu;
		if(ConstShiki(src, t, enzansi, kin, cons, NULL_TOKEN, true) && cons.GetConst()[1] != FUNC_TOKEN && cons.GetConst()[1] != METHOD_TOKEN){
			tk += cons;
		}else{
			if(stat->tokubetuerr) return false;
			stat->ss->ErrorMessage.Clear();
			t = ot;
			src.idx = oi;
			src.gyousuu = ogy;
			if(!Nanika(tk, src, t, kin, false, *this)) return false;
		}
		return true;
	}
	bool Shiki(PWCArray<intptr_t> &tk, Src &src, intptr_t &t, const TokenKind enzansi, const TokenKind oskin, const TokenKind end)
	{
		TokenKind kin, ki;
		kin = oskin;
		if(kin != end){
			if(!CSorNanika(tk, src, t, enzansi, kin)) return false;
			while(SHARP_TOKEN <= (kin = src.GetToken(t)) && kin <= (end == SEMICOLON_TOKEN ? TENDAINYUU_TOKEN :  COMMA_TOKEN)){
				switch(enzansi){
				case NULL_TOKEN:
					break;
				case SHARP_TOKEN:
					src.idx = t;
					return true;
				case KAKERU_TOKEN:case WARU_TOKEN:case AMARI_TOKEN:
					if(kin >= KAKERU_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case PULUS_TOKEN:case MINUS_TOKEN:
					if(kin >= PULUS_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case LSHIFT_TOKEN:case RSHIFT_TOKEN:
					if(kin >= LSHIFT_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case SHOU_TOKEN:case SHOUOREQUAL_TOKEN:case DAI_TOKEN:case DAIOREQUAL_TOKEN:
					if(kin >= SHOU_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case EQUAL_TOKEN:case NOTEQUAL_TOKEN:
					if(kin >= EQUAL_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case AND_TOKEN:
					if(kin >= AND_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case XOR_TOKEN:
					if(kin >= XOR_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case OR_TOKEN:
					if(kin >= OR_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case ANDAND_TOKEN://E(ɍŌɒԂ)
					if(kin >= OROR_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case OROR_TOKEN://E(ɍŌɒԂ)
					if(kin >= QUESTION_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case QUESTION_TOKEN://E
					if(kin >= DAINYUU_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case DAINYUU_TOKEN:case PULUSDAINYUU_TOKEN:case MINUSDAINYUU_TOKEN:case KAKERUDAINYUU_TOKEN:case WARUDAINYUU_TOKEN:case AMARIDAINYUU_TOKEN://E
				case ANDDAINYUU_TOKEN:case XORDAINYUU_TOKEN:case ORDAINYUU_TOKEN:case LSHIFTDAINYUU_TOKEN:case RSHIFTDAINYUU_TOKEN://E
				case TENDAINYUU_TOKEN:
					if(kin >= TENDAINYUU_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				case TENTEN_TOKEN:
					if(end != DAIKAKKOCLOSE_TOKEN){
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'..' ̉Zq '[ ]' łB");
						return false;
					}
				case COMMA_TOKEN:
					if(kin >= TENTEN_TOKEN){
						src.idx = t;
						return true;
					}
					break;
				default:
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				ki = src.GetToken(t);
				if(
					(enzansi == NULL_TOKEN || enzansi == COMMA_TOKEN) && (end == SHOUKAKKOCLOSE_TOKEN || end == KAKKOCOLOCLOSE_TOKEN)
					&& kin == DAINYUU_TOKEN && (ki == end || ki == COMMA_TOKEN))
				{
					tk += NULL_TOKEN;
					tk += DAINYUU_TOKEN;
					tk += COMMA_TOKEN;
					if(ki == end){
						if(enzansi != NULL_TOKEN) src.idx = t;
						return true;
					}
					kin = ki;
					ki = src.GetToken(t);
				}
				if(ki == end){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				if(kin == SHARP_TOKEN || kin == COMMA_TOKEN){
					tk += kin;
				}else if((ANDAND_TOKEN <= kin && kin <= QUESTION_TOKEN)
					|| (DAINYUU_TOKEN <= kin && kin <= TENDAINYUU_TOKEN) || kin == TENTEN_TOKEN)
				{
					tk += NULL_TOKEN;
					tk += kin;
					if(kin == QUESTION_TOKEN){
						if(!Shiki(tk, src, t, QUESTION_TOKEN, ki, COLON_TOKEN)) return false;
						if(src.source.GetConst()[t] != L':'){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"':' ܂B");
							return false;
						}
						tk += COLON_TOKEN;
						src.GetToken(t);
						ki = src.GetToken(t);
					}
				}
				if(!Shiki(tk, src, t, kin, ki, end)) return false;
				if(kin != SHARP_TOKEN && kin != COMMA_TOKEN) tk += kin;
			}
		}
		if(enzansi != NULL_TOKEN) src.idx = t;
		if(kin != end){
			if(end != SEMICOLON_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			if(kin != COMMA_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"';' ܂B");
				return false;
			}
		}
		return true;
	}
	bool TypeNanikaPath(PWCArray<intptr_t> &type, Src &src, const bool conf, SourceTree &hkishiki, const bool create)
	{
		intptr_t t, ii;
		int32_t tyki;
		PWCArray<wchar_t> name;
		Hensuu<SourceTree> *phs;
		if((tyki = PathtoClassID(src, FUNC_BLOCK, true, false, hkishiki, create)) < 0) return false;
		switch(stat->funclist.Get(tyki)->selftype){
		case FUNC_BLOCK:
			if(stat->funclist.Get(tyki) != stat->funclist.Get(tyki)->root){
				if(src.GetToken(t) == SHOUKAKKOOPEN_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^ł͂Ȃ֐ĂяołB");
					return false;
				}
				src.idx = t;
				type += NULL_TOKEN;
				type += tyki;
				break;
			}
		case CLASS_BLOCK:
			src.GetToken(t);
			src.GetToken(t);
			name.Clear();
			name.AddTail(src.source.GetConst()+t, src.idx-t);
			phs = stat->funclist.Get(tyki)->GetHensuu(name, ii, false);
			if(phs->type.GetConst()[0] != TYPE_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"type^ł܂B");
				return false;
			}
			type.AddTail(phs->type.GetConst()+1, phs->type.GetLength()-1);
			break;
		default:
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
			return false;
		}
		return true;
	}
	bool TypeNanika(PWCArray<intptr_t> &type, Src &src, const bool conf, const bool corf, bool tenf, SourceTree &hkishiki, const bool create)
	{
		Hensuu<SourceTree> *phs;
		PWCArray<wchar_t> name;
		intptr_t t, kmfs, oi, ot, ogy;
		int32_t tyki, ci;
		TokenKind kin;
		kin = src.GetToken(t);
		switch(kin){
		case MBR_TOKEN:
			if(frp == root || frp->parent == root || frp->selftype == CLASS_BLOCK){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"mbr͎g܂B");
				return false;
			}
			switch(src.GetToken(t)){
			case POINT_TOKEN:
				if((kin = src.GetToken(t)) != OTHERKIND_TOKEN && kin != SELF_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				src.idx = t;
				if(!frp->parent->TypeNanika(type, src, conf, corf, tenf, hkishiki, create)) return false;
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			return true;
		case TILDE_TOKEN:
			switch(src.GetToken(t)){
			case POINT_TOKEN:
				if((kin = src.GetToken(t)) != OTHERKIND_TOKEN && kin != SELF_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				src.idx = t;
				if(!root->TypeNanika(type, src, conf, corf, tenf, hkishiki, create)) return false;
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			return true;
		case SELF_TOKEN:
			if(frp->selftype == CLASS_BLOCK){
				type += CLASS_TOKEN;
			}else{
				if(corf){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
					return false;
				}
				if(frp->selftype == ENUM_BLOCK){
					type += ENUM_TOKEN;
				}else{
					type += NULL_TOKEN;
				}
			}
			type += frp->funcid;
			return true;
		case BOOL_TOKEN:
		case BYTE_TOKEN:
		case CHAR_TOKEN:
		case DOUBLE_TOKEN:
		case FLOAT_TOKEN:
		case INT_TOKEN:
		case LONG_TOKEN:
		case SHORT_TOKEN:
		case UBYTE_TOKEN:
		case UINT_TOKEN:
		case ULONG_TOKEN:
		case USHORT_TOKEN:
		case INDEX_TOKEN:
		case VOID_TOKEN:
			if(corf){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
				return false;
			}
			type += kin;
			return true;
		case LIST_TOKEN:
			if(conf){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"萔list^͂܂B");
				return false;
			}
		case REF_TOKEN:
			if(corf){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
				return false;
			}
			type += kin;
			if(src.GetToken(t) != COLON_TOKEN){
				if(conf){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
					return false;
				}
				src.idx = t;
				type += NULL_TOKEN;
			}else{
				kin = src.GetToken(t);
				if(kin == CONST_TOKEN){
					type += CONST_TOKEN;
					if(!TypeNanika(type, src, false, false, tenf, hkishiki, create)) return false;
				}else if(kin == FUNC_TOKEN || kin == METHOD_TOKEN || kin == DELEGATE_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"֐|C^͒newł܂B");
					return false;
				}else{
					src.idx = t;
					if(!TypeNanika(type, src, false, false, tenf, hkishiki, create)) return false;
				}
			}
			return true;
		case ENUM_TOKEN:
			if(corf){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
				return false;
			}
		case CLASS_TOKEN:
			type += kin;
			if((ci = PathtoClassID(src, (kin==CLASS_TOKEN?CLASS_BLOCK:ENUM_BLOCK), false, false, hkishiki, create)) < 0) return false;
			if(src.source.GetConst()[src.idx-1] == L'>' && src.source.GetConst()[src.idx-2] != L':'){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"̓G[łB");
				return false;
			}
			type += (intptr_t)ci;
			return true;
		case FUNC_TOKEN:
		case DELEGATE_TOKEN:
			if(corf){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
				return false;
			}
			type += kin;
			hkishiki.FuncType(type, src, create);
			return true;
		case METHOD_TOKEN:
			type += METHOD_TOKEN;
			type += (hkishiki.frp->touka ? hkishiki.frp->parent->funcid : hkishiki.frp->funcid);
			hkishiki.FuncType(type, src, create);
			return true;
		case THREAD_TOKEN:
			if(corf){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
				return false;
			}
			if(conf){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"萔thread^͂܂B");
				return false;
			}
			type += THREAD_TOKEN;
			if(src.GetToken(t) != BRACOLOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"<:' ܂B");
				return false;
			}
			if((ci = PathtoClassID(src, FUNC_BLOCK, false, false, hkishiki, create)) < 0) return false;
			if(stat->funclist.Get(ci)->parent != stat->funclist.Get(ci)->root
				|| (src.source.GetConst()[src.idx-1] == L'>' && src.source.GetConst()[src.idx-2] != L':'))
			{
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"̓G[łB");
				return false;
			}
			type += (intptr_t)ci;
			if(src.GetToken(t) != BRACOLOCLOSE_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"':>' ܂B");
				return false;
			}
			return true;
		case TENTEN_TOKEN:
			if(!tenf || srce->GetToken(t) != OTHERKIND_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			type += TENTEN_TOKEN;
			name.Clear();
			name.AddTail(srce->source.GetConst()+t, srce->idx-t);
			if(name.GetLength() < 2 || name.GetConst()[name.GetLength()-2] != L'_' || name.GetConst()[name.GetLength()-1] != L't'){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O̍Ōオ '_t' ł܂B");
				return false;
			}
			kmfs = type.GetLength();
			type.SetLength(kmfs+((name.GetLength()+1)*sizeof(wchar_t)+sizeof(intptr_t)-1) / sizeof(intptr_t));
			type.Tail() = 0;
			memcpy(type.GetData()+kmfs, name.GetConst(), name.GetLength()*sizeof(wchar_t));
			return true;
		case OTHERKIND_TOKEN:
			oi = src.idx;
			ot = t;
			ogy = src.gyousuu;
			switch(src.GetToken(t)){
			case SHOUKAKKOOPEN_TOKEN:
			case BRACOLOOPEN_TOKEN:
			case COLOYAJIRUSHI_TOKEN:
			case COLOCOLO_TOKEN:
				src.idx = ot;
				src.gyousuu = ogy;
				if(!TypeNanikaPath(type, src, conf, hkishiki, create)) return false;
				break;
			default:
				src.idx = t;
				name.Clear();
				name.AddTail(src.source.GetConst()+ot, oi-ot);
				if((phs = GetHensuu(name, kmfs, false)) == NULL){
					if((tyki = GetFuncId(name, false)) < 0 || stat->funclist.Get(tyki)->selftype != FUNC_BLOCK){
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
						return false;
					}
					src.idx = t;
					type += NULL_TOKEN;
					type += tyki;
					break;
				}
				kmfs = 0;
				while(true){
					if(phs->type.GetConst()[kmfs] == TYPE_TOKEN){
						kmfs++;
						if(conf){
							switch(phs->type.GetConst()[kmfs]){
							case LIST_TOKEN:
							case METHOD_TOKEN:
							case THREAD_TOKEN:
							case DELEGATE_TOKEN:
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"萔Œ`ł܂B");
								return false;
							}
						}
						if(corf && phs->type.GetConst()[kmfs] != CLASS_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
							return false;
						}
						type.AddTail(phs->type.GetConst()+kmfs, phs->type.GetLength()-kmfs);
					}else if((phs->type.GetConst()[kmfs] == CLASS_TOKEN && (kmfs++, true))
						|| ((IsSoushokuToken((TokenKind)phs->type.GetConst()[kmfs]) && phs->type.GetConst()[kmfs+1] == CLASS_TOKEN) && (kmfs += 2, true))
					){
						if(src.GetToken(t) != POINT_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						kin = src.GetToken(t);
						if(kin != SELF_TOKEN && kin != OTHERKIND_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						tyki = (int32_t)phs->type.GetConst()[kmfs];
						src.idx = t;
						if(!stat->funclist.Get(tyki)->TypeNanika(type, src, conf, corf, tenf, hkishiki, create)) return false;
					}else if((phs->type.GetConst()[kmfs] == REF_TOKEN || phs->type.GetConst()[kmfs] == LIST_TOKEN)
						|| (
						(IsSoushokuToken((TokenKind)phs->type.GetConst()[kmfs])
						&& (phs->type.GetConst()[kmfs+1] == REF_TOKEN
						|| phs->type.GetConst()[kmfs+1] == LIST_TOKEN)
						) && (kmfs++, true)
						))
					{
						if(corf){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
							return false;
						}
						switch(src.GetToken(t)){
						case POINT_TOKEN:
							if(conf){
								if(phs->type.GetConst()[kmfs] == LIST_TOKEN){
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"萔list^͂܂B");
									return false;
								}
								if(phs->type.GetConst()[kmfs+1] == NULL_TOKEN){
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
									return false;
								}
							}
							if(src.GetToken(t) != SELF_TOKEN){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
								return false;
							}
							if(kmfs >= 1 && phs->type.GetConst()[kmfs-1] == CONST_TOKEN){
								type.AddTail(phs->type.GetConst()+kmfs, phs->type.GetLength()-kmfs-1);
							}else{
								type.AddTail(phs->type.GetConst()+kmfs, phs->type.GetLength()-kmfs);
							}
							break;
						case DIA_TOKEN:
							if(phs->type.GetConst()[kmfs+1] == NULL_TOKEN){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
								return false;
							}
							kmfs++;
							continue;
						default:
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
					}else if(phs->type.GetConst()[kmfs] == LIB_TOKEN){
						if(corf){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
							return false;
						}
						if(src.GetToken(t) != POINT_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						kin = src.GetToken(t);
						switch(kin){
						case SELF_TOKEN:
							type += NULL_TOKEN;
							type += ((SourceTree *)phs->type.GetConst()[kmfs+1])->funcid;
							break;
						case OTHERKIND_TOKEN:
							if(L'0' <= src.source.GetConst()[t] && src.source.GetConst()[t] <= L'9'){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
								return false;
							}
							src.idx = t;
							if(!((SourceTree *)phs->type.GetConst()[1])->TypeNanika(type, src, conf, corf, tenf, hkishiki, create)) return false;
							break;
						default:
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
					}else if((phs->type.GetConst()[kmfs] == FUNC_TOKEN || phs->type.GetConst()[kmfs] == METHOD_TOKEN || phs->type.GetConst()[kmfs] == DELEGATE_TOKEN)
						|| (
						(IsSoushokuToken((TokenKind)phs->type.GetConst()[kmfs])
						&& (phs->type.GetConst()[kmfs+1] == FUNC_TOKEN
						|| phs->type.GetConst()[kmfs+1] == METHOD_TOKEN)
						) && (kmfs++, true)
						))
					{
						if(corf){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
							return false;
						}
						if(conf && (phs->type.GetConst()[kmfs] == METHOD_TOKEN || phs->type.GetConst()[kmfs] == DELEGATE_TOKEN)){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"萔ł͂܂B");
							return false;
						}
						if(src.GetToken(t) != POINT_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						if(src.GetToken(t) != SELF_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						if(kmfs >= 1 && phs->type.GetConst()[kmfs-1] == CONST_TOKEN){
							type.AddTail(phs->type.GetConst()+kmfs, phs->type.GetLength()-kmfs-1);
						}else{
							type.AddTail(phs->type.GetConst()+kmfs, phs->type.GetLength()-kmfs);
						}
					}else{
						if(IsSoushokuToken((TokenKind)phs->type.GetConst()[kmfs])) kmfs++;
						if(corf){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł܂B");
							return false;
						}
						if(conf && phs->type.GetConst()[kmfs] == THREAD_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"萔thread^͂܂B");
							return false;
						}
						if(src.GetToken(t) != POINT_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						if(src.GetToken(t) != SELF_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return false;
						}
						if(phs->type.GetConst()[kmfs] == ENUM_TOKEN || phs->type.GetConst()[kmfs] == THREAD_TOKEN){
							type.AddTail(phs->type.GetConst()+kmfs, 2);
						}else{
							type += phs->type.GetConst()[kmfs];
						}
					}
					break;
				}
				break;
			}
			if(type.Tail() == ~DAINYUU_TOKEN) type.Pop();
			return true;
		}
		stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
		return false;
	}
	bool LibNanikaPath(PWCArray<intptr_t> &type, Src &src, SourceTree &hkishiki)
	{
		intptr_t t, ii;
		int32_t tyki;
		PWCArray<wchar_t> name;
		Hensuu<SourceTree> *phs;
		if((tyki = PathtoClassID(src, FUNC_BLOCK, true, false, hkishiki, false)) < 0) return false;
		switch(stat->funclist.Get(tyki)->selftype){
		case FUNC_BLOCK:
			if(stat->funclist.Get(tyki) != stat->funclist.Get(tyki)->root){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
				return false;
			}
		case CLASS_BLOCK:
			src.GetToken(t);
			src.GetToken(t);
			name.Clear();
			name.AddTail(src.source.GetConst()+t, src.idx-t);
			phs = stat->funclist.Get(tyki)->GetHensuu(name, ii, false);
			if(phs->type.GetConst()[0] != LIB_TOKEN){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"lib^ł܂B");
				return false;
			}
			type.AddTail(phs->type.GetConst()+1, phs->type.GetLength()-1);
			break;
		default:
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
			return false;
		}
		return true;
	}
	bool LibPath(Src &src, PWCArray<wchar_t> &name)
	{
		intptr_t t;
		TokenKind kin;
		PWCArray<wchar_t> path;
		kin = src.GetToken(t);
		name.Clear();
		switch(kin){
		case DQUOT_TOKEN:
		case BACKSLA_TOKEN:
		case SHOU_TOKEN:
			if(kin == SHOU_TOKEN){
				src.Find(L'>', t);
				name.AddTail(src.source.GetConst()+t, src.idx-t);
				src.idx++;
			}else{
				src.idx = t;
				if(!MjiretuLiteral(name, src)) return false;
			}
			if(name.GetLength() <= 0){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"t@CłB");
				return false;
			}
			if(!KanzenPath(name, (kin==SHOU_TOKEN ? (path=stat->ss->exedir, path+=L"lib/") : src.filename))){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"΃pXւ̕ϊɎs܂B");
				return false;
			}
			name += L'\0';
			break;
		default:
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
			return false;
		}
		return true;
	}
	bool LibNanika(PWCArray<intptr_t> &type, Src &src, SourceTree &hkishiki)
	{
		SourceTree *pst;
		Hensuu<SourceTree> *phs;
		intptr_t t, kmfs, oi, ot, ogy;
		PWCArray<wchar_t> name, tmp;
		TokenKind kin;
		kin = src.GetToken(t);
		switch(kin){
		case DQUOT_TOKEN:
		case BACKSLA_TOKEN:
		case SHOU_TOKEN:
			src.idx = t;
			if(!LibPath(src, name)) return false;
			if(stat->srclist.Exist((_TCHAR*)name.GetConst())){
				pst = stat->srclist[(_TCHAR*)name.GetConst()].get();
				do{
					if(pst == root){
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"CȕzQƂ͂ł܂B");
						return false;
					}
					pst = pst->subsrc;
				}while(pst != NULL);
				type += (intptr_t)stat->srclist[(_TCHAR*)name.GetConst()].get();
			}else{
				pst = root->subsrc;
				root->subsrc = new SourceTree(stat);
				root->subsrc->parent = root;
				if(pst != NULL){
					pst->parent = root->subsrc;
					root->subsrc->subsrc = pst;
				}
				if(!root->subsrc->FileRead(name.GetConst())){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"");
					delete root->subsrc;
					root->subsrc = NULL;
					return false;
				}
				type += (intptr_t)root->subsrc;
			}
			return true;
		case MBR_TOKEN:
			if(frp == root || frp->parent == root || frp->selftype == CLASS_BLOCK){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"mbr͎g܂B");
				return false;
			}
			switch(src.GetToken(t)){
			case POINT_TOKEN:
				if(src.GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				src.idx = t;
				if(!frp->parent->LibNanika(type, src, hkishiki)) return false;
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			return true;
		case TILDE_TOKEN:
			switch(src.GetToken(t)){
			case POINT_TOKEN:
				if(src.GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				src.idx = t;
				if(!root->LibNanika(type, src, hkishiki)) return false;
				break;
			default:
				src.idx = t;
				type += (intptr_t)root;
				break;
			}
			return true;
		case OTHERKIND_TOKEN:
			oi = src.idx;
			ot = t;
			ogy = src.gyousuu;
			switch(src.GetToken(t)){
			case SHOUKAKKOOPEN_TOKEN:
			case BRACOLOOPEN_TOKEN:
			case COLOYAJIRUSHI_TOKEN:
			case COLOCOLO_TOKEN:
				src.idx = ot;
				src.gyousuu = ogy;
				if(!LibNanikaPath(type, src, hkishiki)) return false;
				break;
			default:
				src.idx = t;
				name.Clear();
				name.AddTail(src.source.GetConst()+ot, oi-ot);
				if((phs = GetHensuu(name, kmfs, false)) == NULL){
					tmp = L'\'';
					tmp += name;
					tmp += L"' ܂B";
					tmp += L'\0';
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
					return false;
				}
				if(phs->type.GetConst()[0] != LIB_TOKEN){
					tmp = L'\'';
					tmp += name;
					tmp += L"' ̓Cuł܂B";
					tmp += L'\0';
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
					return false;
				}
				if(src.GetToken(t) == POINT_TOKEN){
					if(src.GetToken(t) != OTHERKIND_TOKEN){
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
						return false;
					}
					src.idx = t;
					if(!((SourceTree *)phs->type.GetConst()[1])->LibNanika(type, src, hkishiki)) return false;
				}else{
					src.idx = t;
					type += phs->type.GetConst()[1];
				}
				break;
			}
			return true;
		}
		stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
		return false;
	}
	bool KanzenPath(PWCArray<wchar_t> &name, const PWCArray<wchar_t> &karento)
	{
		intptr_t i;
		wchar_t *pwc;
		PWCArray<wchar_t> tmp;
		if(name.GetLength() > 0 && name.GetConst()[0] != L'/' && (name.GetLength() < 4 || name.GetConst()[1] != L':')){
			tmp = name;
			name = karento;
			for(i = name.GetLength()-1; i >= 0; i--){
				if(name.GetConst()[i] == L'/' || name.GetConst()[i] == L'\\') break;
			}
			name.SetLength(i+1);
			name += tmp;
		}
		tmp = name;
		tmp += L'\0';
		pwc = _wfullpath(NULL, tmp.GetConst(), 0);
		if(pwc == NULL) return false;
		name = pwc;
		free(pwc);
		for(i = 0; i < name.GetLength(); i++) name.GetData()[i] = tolower(name.GetData()[i]);
		return true;
	}
	bool MjiretuLiteral(PWCArray<wchar_t> &str, Src &src)
	{
		wchar_t wc;
		intptr_t t;
		while(true){
			switch(src.GetToken(t)){
			case DQUOT_TOKEN:
				src.Find(L'"', t);
				str.AddTail(src.source.GetConst()+t, src.idx-t);
				src.idx++;
				continue;
			case BACKSLA_TOKEN:
				if(!TokushuMoji(wc, src)) return false;
				str += wc;
				continue;
			default:
				src.idx = t;
				break;
			}
			break;
		}
		return true;
	}
	bool TokushuMoji(wchar_t &wc, Src &src)
	{
		uint64_t ui64;
		intptr_t t;
		switch(src.GetChar()){
		case L'0':
			wc = L'\0';
			return true;
		case L't':
			wc =  L'\t';
			return true;
		case L'r':
			wc = L'\r';
			return true;
		case L'n':
			wc = L'\n';
			return true;
		case L'\\':
			wc = L'\\';
			return true;
		case L'\'':
			wc = L'\'';
			return true;
		case L'"':
			wc = L'"';
			return true;
		case L'b':
		case L'd':
		case L'o':
		case L'x':
			src.idx--;
			src.GetToken(t);
			if(!ZeroHajimari(ui64, src.source.GetConst()+t, src.idx-t)){
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return false;
			}
			wc = (wchar_t)ui64;
			return true;
		}
		stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
		return false;
	}
	int32_t PathtoClassID(Src &src, const BlockType blty, const bool toka, const bool publ, SourceTree &hkishiki, const bool create)
	{
		TokenKind kin;
		SourceTree *pst, *opst;
		Hensuu<SourceTree> *phs;
		bool pub, kalono, moou;
		BlockType bt;
		intptr_t t, oi, ogy, kmfs, size;
		int32_t fi;
		PWCArray<wchar_t> name, tmp;
		PWCArray<intptr_t> type;
		fi = -1;
		oi = src.idx;
		ogy = src.gyousuu;
		moou = false;
		pst = this;
		pub = publ;
		while(true){
TOPLOOP:
			kin = src.GetToken(t);
			switch(kin){
			case TILDE_TOKEN:
				if(toka || moou){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'~' \G[łB");
					return -1;
				}
				pst = root;
				switch(src.GetToken(t)){
				case POINT_TOKEN:
					pst = root;
					moou = true;
					break;
				default:
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return -1;
				}
				break;
			case MBR_TOKEN:
				if(toka || moou || frp == root || frp->parent == root){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'mbr' \G[łB");
					return -1;
				}
				switch(src.GetToken(t)){
				case POINT_TOKEN:
					pst = frp->parent;
					moou = true;
					break;
				default:
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return -1;
				}
				break;
			case SELF_TOKEN:
				if(moou){
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'self' \G[łB");
					return -1;
				}
				fi = frp->funcid;
				goto FOOGOTOLABEL;
			case OTHERKIND_TOKEN:
				name.Clear();
				name.AddTail(src.source.GetConst()+t, src.idx-t);
				if((fi = pst->GetFuncId(name, create)) < 0){
					if((phs = pst->GetHensuu(name, kmfs, false)) == NULL){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ͒`Ă܂B";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
						return -1;
					}
					switch(phs->type.GetConst()[0]){
					case TYPE_TOKEN:
						if(toka){
							kin = src.GetToken(t);
							if(kin == COLOCOLO_TOKEN){
								src.idx = t;
							}else{
								if(phs->type.GetConst()[1] != NULL_TOKEN){
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"֐ł܂B");
									return -1;
								}
								src.idx = t;
								return (int32_t)phs->type.GetConst()[2];
							}
						}
						if(phs->type.GetConst()[1] != CLASS_TOKEN
							&& phs->type.GetConst()[1] != ENUM_TOKEN && phs->type.GetConst()[1] != NULL_TOKEN)
						{
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
							return -1;
						}
						fi = (int32_t)phs->type.GetConst()[2];
						break;
					case LIB_TOKEN:
						switch(src.GetToken(t)){
						case POINT_TOKEN:
							pst = (SourceTree *)phs->type.GetConst()[1];
							pub = true;
							continue;
						default:
							if(toka){
								src.idx = oi;
								src.gyousuu = ogy;
								return pst->funcid;
							}
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return -1;
						}
						break;
					default:
						stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
						return -1;
					}
				}else if(pst->touka){
					stat->funclist.Get(fi)->parent = pst->parent;
				}
FOOGOTOLABEL:
				moou = true;
				kalono = false;
				while(true){
					kin = src.GetToken(t);
					switch(kin){
					case BRACOLOOPEN_TOKEN:
						if(kalono){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return -1;
						}
						oi = src.idx;
						ogy = src.gyousuu;
						for(kmfs = 1; kmfs != 0 && kin != NULL_TOKEN; ){
							kin = src.GetToken(t);
							switch(kin){
							case BRACOLOOPEN_TOKEN:
								kmfs++;
								break;
							case BRACOLOCLOSE_TOKEN:
								kmfs--;
								break;
							case CHUUKAKKOOPEN_TOKEN:
							case CHUUKAKKOCLOSE_TOKEN:
							case SEMICOLON_TOKEN:
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"':>' ܂B");
								return -1;
							}
						}
						if(kin == NULL_TOKEN){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"':>' ܂B");
							return -1;
						}
						switch(src.GetToken(t)){
						case COLOYAJIRUSHI_TOKEN:
							bt = FUNC_BLOCK;
							break;
						case POINT_TOKEN:
						case COLOCOLO_TOKEN:
							bt = CLASS_BLOCK;
							break;
						default:
							if(blty == ENUM_BLOCK){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"enumłB");
								return -1;
							}
							bt = blty;
							break;
						}
						src.idx = oi;
						src.gyousuu = ogy;
						if(!stat->funclist.Get(fi)->kakutei){
							if(stat->funclist.Get(fi)->selftype == UNKNOWN_BLOCK){
								stat->funclist.Get(fi)->selftype = bt;
								stat->funclist.Get(fi)->srce = &src;
								stat->funclist.Get(fi)->topsrcgyo = src.gyousuu;
							}
						}
						if(stat->funclist.Get(fi)->selftype != bt){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^G[łB");
							return -1;
						}
						oi = 0;
						opst = stat->funclist.Get(fi);
						while(true){
							if(opst->tpcount > 0 && oi >= opst->tpcount){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ev[g܂B");
								return -1;
							}
							type.Clear();
							if(!hkishiki.TypeNanika(type, src, false, false, false, hkishiki, create)) return -1;
							name.SetLength((sizeof(SourceTree*)+sizeof(intptr_t)*type.GetLength())/sizeof(wchar_t) + 1);
							name.GetData()[0] = L'\0';
							memcpy(name.GetData() + 1, &opst, sizeof(SourceTree*));
							memcpy(name.GetData() + TNT_OFFSET, type.GetConst(), sizeof(intptr_t)*type.GetLength());
							if((pst = opst->BlockOpen(&name, bt, true, create)) == NULL){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"G[łB");
								return -1;
							}
							if(pst->selftype != opst->selftype){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^G[łB");
								return -1;
							}
							if(!pst->kakutei && pst->token.GetLength() == 0){
								pst->parent = opst->parent;
								if(stat->funclist.Get(fi)->kakutei){
									pst->rettype = opst->rettype;
									pst->tpcount = opst->tpcount;
									pst->funchensuu = opst->funchensuu;
									pst->hensuname = opst->hensuname;
									pst->hensuidx = opst->hensuidx;
									pst->srce = opst->srce;
									pst->topsrcidx = opst->topsrcidx;
									pst->topsrcgyo = opst->topsrcgyo;
									pst->funchensuu.GetData()[oi].type += type;
								}
							}
							opst = pst;
							kin = src.GetToken(t);
							switch(kin){
							case BRACOLOCLOSE_TOKEN:
								oi++;
								break;
							case COMMA_TOKEN:
								oi++;
								pst->kakutei = stat->funclist.Get(fi)->kakutei;
								continue;
							default:
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
								return -1;
							}
							break;
						}
						if(pst->tpcount > 0 && oi != pst->tpcount){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ev[g܂B");
							return -1;
						}
						if(!pst->kakutei && pst->token.GetLength() == 0 && stat->funclist.Get(fi)->kakutei){
							oi = pst->srce->idx;
							ogy = pst->srce->gyousuu;
							pst->srce->idx = pst->topsrcidx;
							pst->srce->gyousuu = pst->topsrcgyo;
							if((pst->selftype == FUNC_BLOCK && !pst->KariHikisuu()) || !pst->MakeTree()){
								stat->tokubetuerr = true;
								return -1;
							}
							pst->srce->idx = oi;
							pst->srce->gyousuu = ogy;
							if(pst->selftype == CLASS_BLOCK && !pst->ClassShokika(src)) return -1;
							size = pst->GetMemberSize();
							for(kmfs = 0; kmfs < pst->backpatch.GetLength(); kmfs++){
								((SourceTree *)pst->backpatch.GetConst()[kmfs])->funchensuu.GetData()[pst->breakpatch.GetConst()[kmfs]].size = size;
							}
							pst->backpatch.Clear();
							pst->breakpatch.Clear();
						}else if(stat->funclist.Get(fi)->tpcount <= 0){
							stat->funclist.Get(fi)->tpcount = oi;
						}
						fi = pst->funcid;
						kalono = true;
						continue;
					case SHOU_TOKEN:
						if(kalono){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return -1;
						}
						if(blty == ENUM_BLOCK){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"enumłB");
							return -1;
						}
						if(toka){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return -1;
						}
						oi = -1;
						if(stat->funclist.Get(fi)->selftype == UNKNOWN_BLOCK){
							stat->funclist.Get(fi)->selftype = blty;
							stat->funclist.Get(fi)->srce = &src;
							stat->funclist.Get(fi)->topsrcgyo = src.gyousuu;
						}else{
							oi = stat->funclist.Get(fi)->tpcount;
							stat->funclist.Get(fi)->tpcount = 0;
						}
						if(stat->funclist.Get(fi)->selftype != blty){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^G[łB");
							return -1;
						}
						if((toka || blty == CLASS_BLOCK) && pub){
							if(stat->funclist.Get(fi)->rettype.GetLength() > 0){
								if(!IsPublicToken((TokenKind)stat->funclist.Get(fi)->rettype.GetConst()[0])){
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"vCx[głB");
									return -1;
								}
							}else{
								stat->funclist.Get(fi)->rettype = PUBLIC_TOKEN;
							}
						}
						kin = src.GetToken(t);
						switch(kin){
						case OTHERKIND_TOKEN:
							while(true){
								if(src.idx-t < 2 || src.source.GetConst()[src.idx-2] != L'_' || src.source.GetConst()[src.idx-1] != L't'){
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"type^̎ʎq '_t' ŏIȂ΂܂B");
									return -1;
								}
								name.Clear();
								name.AddTail(src.source.GetConst()+t, src.idx-t);
								type = TYPE_TOKEN;
								if(stat->funclist.Get(fi)->GetHensuu(name, kmfs, false) == NULL && !stat->funclist.Get(fi)->AddHensuu(type, name, 0)){
									tmp = L'\'';
									tmp += name;
									tmp += L"' ͂łɒ`Ă܂B";
									tmp += L'\0';
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, tmp.GetConst());
									return -1;
								}
								stat->funclist.Get(fi)->tpcount++;
								kin = src.GetToken(t);
								switch(kin){
								case DAI_TOKEN:
									break;
								case COMMA_TOKEN:
									kin = src.GetToken(t);
									switch(kin){
									case OTHERKIND_TOKEN:
										break;
									default:
										stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
										return -1;
									}
									continue;
								default:
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
									return -1;
								}
								break;
							}
							if(oi >= 0 && stat->funclist.Get(fi)->tpcount != oi){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ev[g̐܂B");
								return -1;
							}
							return fi;
						default:
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
							return -1;
						}
						break;
					case COLOYAJIRUSHI_TOKEN:
						if(stat->funclist.Get(fi)->tpcount > 0
							&& stat->funclist.Get(fi)->funchensuu.GetConst()[stat->funclist.Get(fi)->tpcount-1].type.GetLength() <= 1)
						{
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ev[g܂B");
							return -1;
						}
						if(!stat->funclist.Get(fi)->kakutei){
							if(stat->funclist.Get(fi)->selftype == UNKNOWN_BLOCK){
								stat->funclist.Get(fi)->selftype = FUNC_BLOCK;
								stat->funclist.Get(fi)->touka = true;
								stat->funclist.Get(fi)->srce = &src;
								stat->funclist.Get(fi)->topsrcgyo = src.gyousuu;
							}
						}
						if(stat->funclist.Get(fi)->selftype != FUNC_BLOCK || !stat->funclist.Get(fi)->touka){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ߊ֐ł͂܂B");
							return -1;
						}
						if((toka || blty == CLASS_BLOCK) && pub){
							if(stat->funclist.Get(fi)->rettype.GetLength() > 0){
								if(!IsPublicToken((TokenKind)stat->funclist.Get(fi)->rettype.GetConst()[0])){
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"vCx[głB");
									return -1;
								}
							}else{
								stat->funclist.Get(fi)->rettype = PUBLIC_TOKEN;
							}
						}
						pst = stat->funclist.Get(fi);
						pub = true;
						break;
					case COLOCOLO_TOKEN:
						if(stat->funclist.Get(fi)->tpcount > 0
							&& stat->funclist.Get(fi)->funchensuu.GetConst()[stat->funclist.Get(fi)->tpcount-1].type.GetLength() <= 1)
						{
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ev[g܂B");
							return -1;
						}
						while(true){
							if(stat->funclist.Get(fi)->selftype == ENUM_BLOCK){
								if(!toka){
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"enumłB");
									return -1;
								}
								src.idx = t;
								return fi;
							}
							if(!stat->funclist.Get(fi)->kakutei){
								if(stat->funclist.Get(fi)->selftype == UNKNOWN_BLOCK){
									stat->funclist.Get(fi)->selftype = CLASS_BLOCK;
									stat->funclist.Get(fi)->srce = &src;
									stat->funclist.Get(fi)->topsrcgyo = src.gyousuu;
								}
							}
							if(stat->funclist.Get(fi)->selftype != CLASS_BLOCK){
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"NXł͂܂B");
								return -1;
							}
							if((toka || blty == CLASS_BLOCK) && pub){
								if(stat->funclist.Get(fi)->rettype.GetLength() > 0){
									if(!IsPublicToken((TokenKind)stat->funclist.Get(fi)->rettype.GetConst()[0])){
										stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"vCx[głB");
										return -1;
									}
								}else{
									stat->funclist.Get(fi)->rettype = PUBLIC_TOKEN;
								}
							}
							if(toka){
								oi = t;
								ogy = src.gyousuu;
								if(src.GetToken(t) == OTHERKIND_TOKEN){
									name.Clear();
									name.AddTail(src.source.GetConst()+t, src.idx-t);
									if((phs = stat->funclist.Get(fi)->GetHensuu(name, kmfs, false)) != NULL){
										switch(phs->type.GetConst()[0]){
										case TYPE_TOKEN:
											if(
												(phs->type.GetConst()[1] == CLASS_TOKEN || phs->type.GetConst()[1] == ENUM_TOKEN)
												&& src.GetToken(t) == COLOCOLO_TOKEN)
											{
												fi = (int32_t)phs->type.GetConst()[2];
												pst = stat->funclist.Get(fi);
												pub = true;
												continue;
											}
											break;
										case LIB_TOKEN:
											if(src.GetToken(t) == POINT_TOKEN){
												pst = (SourceTree *)phs->type.GetConst()[1];
												pub = true;
												oi = t;
												ogy = src.gyousuu;
												goto TOPLOOP;
											}
											break;
										}
										src.idx = oi;
										src.gyousuu = ogy;
										return fi;
									}else{
										src.idx = t;
										pst = stat->funclist.Get(fi);
										pub = true;
										goto TOPLOOP;
									}
								}
								stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
								return -1;
							}
							pst = stat->funclist.Get(fi);
							pub = true;
							break;
						}
						break;
					default:
						if(stat->funclist.Get(fi)->tpcount > 0
							&& stat->funclist.Get(fi)->funchensuu.GetConst()[stat->funclist.Get(fi)->tpcount-1].type.GetLength() <= 1)
						{
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"ev[g܂B");
							return -1;
						}
						if(!stat->funclist.Get(fi)->kakutei){
							if(stat->funclist.Get(fi)->selftype == UNKNOWN_BLOCK){
								stat->funclist.Get(fi)->selftype = blty;
								stat->funclist.Get(fi)->srce = &src;
								stat->funclist.Get(fi)->topsrcgyo = src.gyousuu;
							}
						}
						if(stat->funclist.Get(fi)->selftype != blty){
							stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"^G[łB");
							return -1;
						}
						if((toka || blty == CLASS_BLOCK) && pub){
							if(stat->funclist.Get(fi)->rettype.GetLength() > 0){
								if(
									!IsPublicToken((TokenKind)stat->funclist.Get(fi)->rettype.GetConst()[0])
									&& stat->funclist.Get(fi)->selftype != ENUM_BLOCK)
								{
									stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"pubNł܂B");
									return -1;
								}
							}else{
								stat->funclist.Get(fi)->rettype = PUBLIC_TOKEN;
							}
						}
						src.idx = t;
						return fi;
					}
					break;
				}
				break;
			default:
				stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
				return -1;
			}
		}
		return fi;
	}
	bool ClassShokika(Src &src)
	{
		PWCArray<wchar_t> name;
		PWCArray<intptr_t> meniya;
		int32_t fi;
		if(!MemberNew(src, meniya)) return false;
		name = L"default";
		if(meniya.GetLength() > 0){
			fi = GetFuncId(name, true);
			if(!stat->funclist.Get(fi)->GblhaniSet(src, *this)) return false;
			stat->funclist.Get(fi)->selftype = FUNC_BLOCK;
			stat->funclist.Get(fi)->rettype = VOID_TOKEN;
			stat->funclist.Get(fi)->token[0] = meniya;
			stat->funclist.Get(fi)->kakutei = true;
		}else if((fi=GetFuncId(name, false)) >= 0){
			stat->funclist.Get(fi)->kakutei = true;
			stat->funclist.Get(fi)->gblhani = INTPTR_MAX;
		}
		if(!MemberDel(src, meniya)) return false;
		name = L"break";
		if(meniya.GetLength() > 0){
			fi = GetFuncId(name, true);
			if(!stat->funclist.Get(fi)->GblhaniSet(src, *this)) return false;
			stat->funclist.Get(fi)->selftype = FUNC_BLOCK;
			stat->funclist.Get(fi)->rettype = VOID_TOKEN;
			stat->funclist.Get(fi)->token[0] = meniya;
			stat->funclist.Get(fi)->kakutei = true;
		}else if((fi=GetFuncId(name, false)) >= 0){
			stat->funclist.Get(fi)->kakutei = true;
			stat->funclist.Get(fi)->gblhani = INTPTR_MAX;
		}
		return true;
	}
	bool ReturnTemplate()
	{
		intptr_t i, j;
		PWCArray<intptr_t> tmp;
		const PWCArray<intptr_t> *pt;
		Hensuu<SourceTree> *ph;
		PWCArray<intptr_t> type;
		tmp.Clear();
		i = 0;
		while(i < rettype.GetLength()){
			if(IsSoushokuToken((TokenKind)rettype.GetConst()[i])) tmp += rettype.GetConst()[i++];
			switch(rettype.GetConst()[i]){
			case TENTEN_TOKEN:
				i++;
				{
					PWCArray<wchar_t> name;
					name += (wchar_t*)(rettype.GetConst()+i);
					i += ((name.GetLength()+1)*sizeof(wchar_t)+sizeof(intptr_t)-1) / sizeof(intptr_t);
					for(j = 0; j < tpcount; j++){
						if(name == hensuname.GetConst()[j]){
							pt = &funchensuu.GetConst()[hensuidx.GetConst()[j]].type;
							tmp.AddTail(pt->GetConst()+1, pt->GetLength()-1);
							break;
						}
					}
					if(j == tpcount){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"ev[g܂B");
						return false;
					}
				}
				break;
			case CLASS_TOKEN:
			case ENUM_TOKEN:
			case THREAD_TOKEN:
				tmp += rettype.GetConst()[i++];
				tmp += rettype.GetConst()[i++];
				break;
			default:
				tmp += rettype.GetConst()[i++];
				break;
			}
		}
		rettype = tmp;
		hikisuucount = 0;
		j = 0;
		if(IsSoushokuToken((TokenKind)rettype.GetConst()[j])) j++;
		if(rettype.GetConst()[j] == REF_TOKEN || rettype.GetConst()[j] == LIST_TOKEN){
			intptr_t size = (rettype.GetConst()[j+1] == NULL_TOKEN ? sizeof(DynamicRef) : sizeof(Reference));
			type.AddTail(rettype.GetConst()+j, rettype.GetLength()-j);
			hikisuucount = 1;
			AddHiddenHensuu(NULL, type, sizeof(intptr_t));
			funchensuu.Tail().type += ~DAINYUU_TOKEN;
			for(i = 0; i < backpatch.GetLength(); i++){
				ph = &((SourceTree*)backpatch.GetConst()[i])->funchensuu.GetData()[breakpatch.GetConst()[i]];
				ph->type = type;
				ph->size = size;
			}
			rettype.GetData()[j] = VOID_TOKEN;
			rettype.SetLength(j+1);
		}else{
			type = REF_TOKEN;
			type += OTHERKIND_TOKEN;
			for(i = 0; i < backpatch.GetLength(); i++){
				ph = &((SourceTree*)backpatch.GetConst()[i])->funchensuu.GetData()[breakpatch.GetConst()[i]];
				ph->type = type;
				ph->size = 0;
			}
		}
		backpatch.Clear();
		breakpatch.Clear();
		return true;
	}
	bool BackTemplate(const int ato)
	{
		intptr_t i, oi, ogy;
		intptr_t size;
		if(ato <= 0){
			oi = srce->idx;
			ogy = srce->gyousuu;
			srce->idx = topsrcidx;
			srce->gyousuu = topsrcgyo;
			if(selftype == FUNC_BLOCK && !KariHikisuu()) return false;
			if(!MakeTree()) return false;
			srce->idx = oi;
			srce->gyousuu = ogy;
			if(selftype == CLASS_BLOCK && !ClassShokika(*srce)) return false;
			kakutei = true;
			size = GetMemberSize();
			for(i = 0; i < backpatch.GetLength(); i++){
				((SourceTree *)backpatch.GetConst()[i])->funchensuu.GetData()[breakpatch.GetConst()[i]].size = size;
			}
			backpatch.Clear();
			breakpatch.Clear();
			return true;
		}
		if(children.GetLength() <= 0){
			stat->ss->ErrorMessage.Add(srce->filename, topsrcgyo, L"G[łB");
			return false;
		}
		for(i=0; i < children.GetLength(); i++){
			children.GetData()[i]->parent = parent;
			children.GetData()[i]->rettype = rettype;
			children.GetData()[i]->tpcount = tpcount;
			children.GetData()[i]->funchensuu = funchensuu;
			children.GetData()[i]->hensuname = hensuname;
			children.GetData()[i]->hensuidx = hensuidx;
			children.GetData()[i]->srce = srce;
			children.GetData()[i]->topsrcidx = topsrcidx;
			children.GetData()[i]->topsrcgyo = topsrcgyo;
			children.GetData()[i]->funchensuu.GetData()[tpcount-ato].type.AddTail(
				(intptr_t *)(subfunc.GetConst()[i]->myname.GetConst() + TNT_OFFSET),
				(
				(subfunc.GetConst()[i]->myname.GetLength() - TNT_OFFSET)*sizeof(wchar_t) + sizeof(intptr_t) - 1
				)/sizeof(intptr_t)
			);
			if(!children.GetData()[i]->BackTemplate(ato-1)) return false;
		}
		kakutei = true;
		return true;
	}
	bool RefTouroku(const PWCArray<intptr_t> &type, intptr_t &t, PWCArray<intptr_t> *const ptok)
	{
		PWCArray<wchar_t> name, tmp;
		if(srce->GetToken(t) != OTHERKIND_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
			return false;
		}
		name.Clear();
		name.AddTail(srce->source.GetConst()+t, srce->idx-t);
		if(type.GetConst()[0] == CONST_TOKEN){
			if(!AddHensuu(type, name, 0)){
				tmp = L'\'';
				tmp += name;
				tmp += L"' ͖OdĂ邩AsłB";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
				return false;
			}
		}else{
			if(!AddHensuu(type, name, GetTypeSize(type))){
				tmp = L'\'';
				tmp += name;
				tmp += L"' ͖OdĂ邩AsłB";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
				return false;
			}
			if(ptok != NULL){
				if(frp == root) *ptok += TILDE_TOKEN;
				*ptok += ~(frp->funchensuu.GetLength()-1);
				*ptok += VOID_TOKEN;
				*ptok += SEMICOLON_TOKEN;
			}
		}
		return true;
	}
	bool ListTouroku(const PWCArray<intptr_t> &type, intptr_t &t, PWCArray<intptr_t> *const ptok)
	{
		PWCArray<wchar_t> name, tmp;
		if(srce->GetToken(t) != OTHERKIND_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
			return false;
		}
		name.Clear();
		name.AddTail(srce->source.GetConst()+t, srce->idx-t);
		if(!AddHensuu(type, name, GetTypeSize(type))){
			tmp = L'\'';
			tmp += name;
			tmp += L"' ͖OdĂ邩AsłB";
			tmp += L'\0';
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
			return false;
		}
		if(ptok != NULL){
			if(frp == root) *ptok += TILDE_TOKEN;
			*ptok += ~(frp->funchensuu.GetLength()-1);
			*ptok += VOID_TOKEN;
			*ptok += SEMICOLON_TOKEN;
		}
		return true;
	}
	bool ThreadTouroku(const PWCArray<intptr_t> &type, PWCArray<intptr_t> *const ptok)
	{
		intptr_t t;
		PWCArray<wchar_t> name, tmp;
		if(srce->GetToken(t) != OTHERKIND_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
			return false;
		}
		name.Clear();
		name.AddTail(srce->source.GetConst()+t, srce->idx-t);
		if(!AddHensuu(type, name, sizeof(Thread))){
			tmp = L'\'';
			tmp += name;
			tmp += L"' ͖OdĂ邩AsłB";
			tmp += L'\0';
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
			return false;
		}
		if(ptok != NULL){
			if(frp == root) *ptok += TILDE_TOKEN;
			*ptok += ~(frp->funchensuu.GetLength()-1);
			*ptok += VOID_TOKEN;
			*ptok += SEMICOLON_TOKEN;
		}
		return true;
	}
	void CallNew(PWCArray<intptr_t> &tk, PWCArray<intptr_t> &meniya)
	{
		PWCArray<wchar_t> name;
		int32_t fi;
		name = L"default";
		if((fi = GetFuncId(name, false)) >= 0){
			tk += meniya;
			tk += SHOUKAKKOOPEN_TOKEN;
			tk += ~fi;
			tk += SHOUKAKKOCLOSE_TOKEN;
			tk += SEMICOLON_TOKEN;
		}
	}
	bool MemberNew(Src &src, PWCArray<intptr_t> &meniya)
	{
		PWCArray<wchar_t> name;
		int32_t fi;
		intptr_t ii, il;
		meniya.Clear();
		for(ii = tpcount+hikisuucount; ii < funchensuu.GetLength(); ii++){
			if(funchensuu.GetConst()[ii].type.GetConst()[0] == CONST_TOKEN) continue;
			fi = TokenTtoTypeId(funchensuu.GetConst()[ii].type.GetConst(), il=0);
			if(fi > ENDCLASS_TYPEID) continue;
			if(fi < 0){
				if(!(DYNLIST_TYPEID <= fi && fi <= REF_TYPEID) && fi < ENDTHREAD_TYPEID) continue;
				meniya += MBR_TOKEN;
				meniya += ~ii;
				meniya += VOID_TOKEN;
				meniya += SEMICOLON_TOKEN;
			}else if(stat->funclist.Get(fi)->selftype == CLASS_BLOCK){
				name = L"default";
				if((fi = stat->funclist.Get(fi)->GetFuncId(name, false)) >= 0){
					meniya += MBR_TOKEN;
					meniya += ~ii;
					meniya += POINT_TOKEN;
					meniya += SHOUKAKKOOPEN_TOKEN;
					meniya += ~fi;
					meniya += SHOUKAKKOCLOSE_TOKEN;
					meniya += SEMICOLON_TOKEN;
				}
			}
		}
		name = L"new";
		if((fi = GetFuncId(name, false)) >= 0){
			if(stat->funclist.Get(fi)->token.GetLength() > 0 && stat->funclist.Get(fi)->kakutei){
				if(!stat->funclist.Get(fi)->GblhaniSet(src, *this)) return false;
				meniya += MBR_TOKEN;
				meniya += SHOUKAKKOOPEN_TOKEN;
				meniya += ~fi;
				meniya += SHOUKAKKOCLOSE_TOKEN;
				meniya += SEMICOLON_TOKEN;
			}else{
				stat->funclist.Get(fi)->selftype = FUNC_BLOCK;
				stat->funclist.Get(fi)->rettype = VOID_TOKEN;
				stat->funclist.Get(fi)->kakutei = true;
				stat->funclist.Get(fi)->gblhani = INTPTR_MAX;
			}
		}
		return true;
	}
	void CallDel(PWCArray<intptr_t> &tk, PWCArray<intptr_t> &meniya)
	{
		PWCArray<wchar_t> name;
		int32_t fi;
		name = L"break";
		if((fi = GetFuncId(name, false)) >= 0){
			tk += meniya;
			tk += SHOUKAKKOOPEN_TOKEN;
			tk += ~fi;
			tk += SHOUKAKKOCLOSE_TOKEN;
			tk += SEMICOLON_TOKEN;
		}
	}
	bool MemberDel(Src &src, PWCArray<intptr_t> &meniya)
	{
		PWCArray<wchar_t> name;
		int32_t fi;
		intptr_t ii, il;
		meniya.Clear();
		name = L"delete";
		if((fi = GetFuncId(name, false)) >= 0){
			if(stat->funclist.Get(fi)->token.GetLength() > 0 && stat->funclist.Get(fi)->kakutei){
				if(!stat->funclist.Get(fi)->GblhaniSet(src, *this)) return false;
				meniya += MBR_TOKEN;
				meniya += SHOUKAKKOOPEN_TOKEN;
				meniya += ~fi;
				meniya += SHOUKAKKOCLOSE_TOKEN;
				meniya += SEMICOLON_TOKEN;
			}else{
				stat->funclist.Get(fi)->selftype = FUNC_BLOCK;
				stat->funclist.Get(fi)->rettype = VOID_TOKEN;
				stat->funclist.Get(fi)->kakutei = true;
				stat->funclist.Get(fi)->gblhani = INTPTR_MAX;
			}
		}
		for(ii = funchensuu.GetLength()-1; ii >= tpcount+hikisuucount; ii--){
			if(funchensuu.GetConst()[ii].type.GetConst()[0] == CONST_TOKEN) continue;
			fi = TokenTtoTypeId(funchensuu.GetConst()[ii].type.GetConst(), il=0);
			if(fi > ENDCLASS_TYPEID) continue;
			if(fi < 0){
				if(!(DYNLIST_TYPEID <= fi && fi <= REF_TYPEID) && fi < ENDTHREAD_TYPEID) continue;
				meniya += MBR_TOKEN;
				meniya += ~ii;
				meniya += DELETE_TOKEN;
				meniya += SEMICOLON_TOKEN;
			}else if(stat->funclist.Get(fi)->selftype == CLASS_BLOCK){
				name = L"break";
				if((fi = stat->funclist.Get(fi)->GetFuncId(name, false)) >= 0){
					meniya += MBR_TOKEN;
					meniya += ~ii;
					meniya += POINT_TOKEN;
					meniya += SHOUKAKKOOPEN_TOKEN;
					meniya += ~fi;
					meniya += SHOUKAKKOCLOSE_TOKEN;
					meniya += SEMICOLON_TOKEN;
				}
			}
		}
		return true;
	}
	bool ClassObjToroku(const PWCArray<intptr_t> &type, const int32_t ci, PWCArray<intptr_t> *const ptok)
	{
		PWCArray<wchar_t> name, tmp;
		PWCArray<intptr_t> meniya;
		intptr_t t;
		if(srce->GetToken(t) != OTHERKIND_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
			return false;
		}
		name.Clear();
		name.AddTail(srce->source.GetConst()+t, srce->idx-t);
		if(!AddHensuu(type, name, stat->funclist.Get(ci)->GetMemberSize())){
			tmp = L'\'';
			tmp += name;
			tmp += L"' ͖OdĂ邩AsłB";
			tmp += L'\0';
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
			return false;
		}
		stat->funclist.Get(ci)->mbsize = -1;//C
		if(ptok != NULL){
			if(frp == root){
				meniya = TILDE_TOKEN;
			}else{
				meniya.Clear();
			}
			meniya += ~(frp->funchensuu.GetLength()-1);
			meniya += POINT_TOKEN;
			stat->funclist.Get(ci)->CallNew(*ptok, meniya);
		}
		return true;
	}
	bool KariHikisuu()
	{
		TokenKind kin;
		PWCArray<wchar_t> name, tmp;
		intptr_t t;
		intptr_t size;
		PWCArray<intptr_t> type;
		if(!ReturnTemplate()) return false;
		if(srce->GetToken(t) != SHOUKAKKOOPEN_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'(' ܂B");
			return false;
		}
		kin = srce->GetToken(t);
		while(kin != SHOUKAKKOCLOSE_TOKEN){
			srce->idx = t;
			type.Clear();
			if(!TypeNanika(type, *srce, false, false, false, *this, true)) return false;
			switch(type.GetConst()[0]){
			case REF_TOKEN:
				if(!RefTouroku(type, t, NULL)) return false;
				break;
			case LIST_TOKEN:
				if(!ListTouroku(type, t, NULL)) return false;
				break;
			case THREAD_TOKEN:
				if(!ThreadTouroku(type, NULL)) return false;
				break;
			case CLASS_TOKEN:
				if(!ClassObjToroku(type, (int32_t)type.GetConst()[1], NULL)) return false;
				break;
			default:
				if(srce->GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				name.Clear();
				name.AddTail(srce->source.GetConst()+t, srce->idx-t);
				if((size = GetTypeSize(type)) < 0){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"G[łB");
					return false;
				}
				if(!AddHensuu(type, name, size)){
					tmp = L'\'';
					tmp += name;
					tmp += L"' ͖OdĂ邩AsłB";
					tmp += L'\0';
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
					return false;
				}
				break;
			}
			hikisuucount++;
			kin = srce->GetToken(t);
			if(kin == DAINYUU_TOKEN){
				if(type.GetConst()[0] == DELEGATE_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"delegate^͎QƓnł܂B");
					return false;
				}
				funchensuu.Tail().type += ~DAINYUU_TOKEN;
				funchensuu.Tail().size = sizeof(intptr_t);
				kin = srce->GetToken(t);
			}else{
				if(type.GetConst()[0] == THREAD_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"thread^͒lnł܂B");
					return false;
				}
				if(type.GetConst()[0] == CLASS_TOKEN && !stat->funclist.Get((int32_t)type.GetConst()[1])->kakutei){
					stat->funclist.Get((int32_t)type.GetConst()[1])->backpatch += (intptr_t)this;
					stat->funclist.Get((int32_t)type.GetConst()[1])->breakpatch += funchensuu.GetLength()-1;
				}
			}
			funchensuu.Tail().size = Aligner(funchensuu.GetTail().size);
			switch(kin){
			case COMMA_TOKEN:
				srce->GetToken(t);
				break;
			case SHOUKAKKOCLOSE_TOKEN:
				break;
			default:
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
		}
		if(srce->GetToken(t) != CHUUKAKKOOPEN_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'{' ܂B");
			return false;
		}
		if(srce->GetToken(t) == COLOYAJIRUSHI_TOKEN){
			touka = true;
		}else{
			if(touka){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"łɓߊ֐ƂĎgĂ܂B");
				return false;
			}
			srce->idx = t;
		}
		return true;
	}
	bool ReturnType(PWCArray<intptr_t> &type, Src &src, const bool create)
	{
		TokenKind kin;
		intptr_t t;
		int32_t ci;
		kin = src.GetToken(t);
		if(IsReadOnlyToken(kin)){
			type += kin;
			kin = src.GetToken(t);
		}
		switch(kin){
		case BOOL_TOKEN:
		case CHAR_TOKEN:
		case DOUBLE_TOKEN:
		case FLOAT_TOKEN:
		case BYTE_TOKEN:
		case INT_TOKEN:
		case LONG_TOKEN:
		case SHORT_TOKEN:
		case UBYTE_TOKEN:
		case UINT_TOKEN:
		case ULONG_TOKEN:
		case USHORT_TOKEN:
		case INDEX_TOKEN:
		case VOID_TOKEN:
			type += kin;
			break;
		case ENUM_TOKEN:
			type += kin;
			if((ci = PathtoClassID(src, ENUM_BLOCK, false, false, *this, true)) < 0) return false;
			type += ci;
			break;
		case FUNC_TOKEN:
			type += kin;
			if(!FuncType(type, src, create)) return false;
			break;
		default:
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"߂ľ^słB");
			return false;
		}
		return true;
	}
	bool FuncType(PWCArray<intptr_t> &type, Src &src, const bool create)
	{
		TokenKind kin;
		intptr_t t;
		if(!ReturnType(type, src, create)) return false;
		if(src.GetToken(t) != KAKKOCOLOOPEN_TOKEN){
			stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"'(:' ܂B");
			return false;
		}
		type += KAKKOCOLOOPEN_TOKEN;
		if(src.GetToken(t) != KAKKOCOLOCLOSE_TOKEN){
			src.idx = t;
			while(true){
				if(!TypeNanika(type, src, false, false, false, *this, create)) return false;
				kin = src.GetToken(t);
				if(kin == DAINYUU_TOKEN){
					type += ~DAINYUU_TOKEN;
					kin = src.GetToken(t);
				}
				switch(kin){
				case COMMA_TOKEN:
					type += COMMA_TOKEN;
					continue;
				case KAKKOCOLOCLOSE_TOKEN:
					break;
				default:
					stat->ss->ErrorMessage.Add(src.filename, src.gyousuu, L"\G[łB");
					return false;
				}
				break;
			}
		}
		type += KAKKOCOLOCLOSE_TOKEN;
		return true;
	}
	bool PluginType(PWCArray<intptr_t> &type, PWCArray<wchar_t> &name)
	{
		TokenKind kin;
		intptr_t t;
		if(!ReturnType(type, *srce, true)) return false;
		if(srce->GetToken(t) != OTHERKIND_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
			return false;
		}
		name.Clear();
		name.AddTail(srce->source.GetConst()+t, srce->idx-t);
		if(srce->GetToken(t) != KAKKOCOLOOPEN_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'(:' ܂B");
			return false;
		}
		type += KAKKOCOLOOPEN_TOKEN;
		if(srce->GetToken(t) != KAKKOCOLOCLOSE_TOKEN){
			srce->idx = t;
			while(true){
				if(!TypeNanika(type, *srce, false, false, false, *this, true)) return false;
				kin = srce->GetToken(t);
				if(kin == DAINYUU_TOKEN){
					type += ~DAINYUU_TOKEN;
					kin = srce->GetToken(t);
				}
				switch(kin){
				case COMMA_TOKEN:
					type += COMMA_TOKEN;
					continue;
				case KAKKOCOLOCLOSE_TOKEN:
					break;
				default:
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				break;
			}
		}
		type += KAKKOCOLOCLOSE_TOKEN;
		return true;
	}
	bool ClassnoTeisuuDainyuu(const PWCArray<intptr_t> &type, intptr_t &t)
	{
		PWCArray<intptr_t> last, tkty, tmpgy;
		PWCArray<wchar_t> name;
		int32_t ci;
		SourceTree *pst;
		PWCArrayQuad< PWCArray<intptr_t> > tmptk;
		if(!ConstShiki(*srce, t, NULL_TOKEN, srce->GetToken(t), last, SEMICOLON_TOKEN, false)) return false;
		tkty = CONST_TOKEN;
		if(IsSoushokuToken((TokenKind)type.GetConst()[0])){
			tkty.AddTail(type.GetConst()+1, type.GetLength()-1);
		}else{
			tkty += type;
		}
		if(!ConstCaster(last, tkty)){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"^G[łB");
			return false;
		}
		name = L"new";
		if((ci = GetFuncId(name, true)) < 0){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"svcȃG[łB");
			return false;
		}
		pst = stat->funclist.Get(ci);
		pst->selftype = FUNC_BLOCK;
		pst->rettype = VOID_TOKEN;
		tmptk = pst->token;
		tmpgy = pst->gyo;
		pst->token.SetLength(1);
		pst->token.GetData()[0] = MBR_TOKEN;
		pst->token.GetData()[0] += ~(funchensuu.GetLength()-1);
		pst->token.GetData()[0] += NULL_TOKEN;
		pst->token.GetData()[0] += DAINYUU_TOKEN;
		pst->token.GetData()[0] += last;
		pst->token.GetData()[0] += DAINYUU_TOKEN;
		pst->token.GetData()[0] += SEMICOLON_TOKEN;
		pst->gyo.SetLength(1);
		pst->gyo.GetData()[0] = -1;
		pst->token += tmptk;
		pst->gyo += tmpgy;
		pst->kakutei = true;
		return true;
	}
	bool KansuuTeigika(intptr_t &t)
	{
		TokenKind kin;
		intptr_t oi, ot, ogy;
		bool ret = false;
		oi = srce->idx;
		ot = t;
		ogy = srce->gyousuu;
		kin = srce->GetToken(t);
		switch(kin){
		case OTHERKIND_TOKEN:
			kin = srce->GetToken(t);
			switch(kin){
			case SHOUKAKKOOPEN_TOKEN:
			case BRACOLOOPEN_TOKEN:
			case SHOU_TOKEN:
			case COLOCOLO_TOKEN:
			case COLOYAJIRUSHI_TOKEN:
			case POINT_TOKEN:
				ret = true;
				break;
			}
			break;
		case TILDE_TOKEN:
		case MBR_TOKEN:
			ret = true;
			break;
		}
		srce->idx = oi;
		t = ot;
		srce->gyousuu = ogy;
		return ret;
	}
	bool RetTenCheck(const PWCArray<intptr_t> &type)
	{
		intptr_t i = 0;
		while(i < rettype.GetLength()){
			if(IsSoushokuToken((TokenKind)rettype.GetConst()[i])) i++;
			switch(rettype.GetConst()[i]){
			case TENTEN_TOKEN:
				return true;
			case CLASS_TOKEN:
			case ENUM_TOKEN:
			case THREAD_TOKEN:
				i += 2;
				break;
			default:
				i++;
				break;
			}
		}
		return false;
	}
	bool HenFuTouroku(PWCArray<intptr_t> &tk, PWCArray<intptr_t> &type)
	{
		TokenKind kin;
		intptr_t t, oi, ot, ogy, kmfs, size;
		int32_t ci, t32;
		bool cst, flu, funcfg;
		SourceTree *pst;
		PWCArray<wchar_t> name, tmp, path;
		PWCArray<intptr_t> last, inty;
		if(type.GetConst()[0] == CORE_TOKEN && selftype != CLASS_BLOCK){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"core`ł̂̓NXłB");
			return false;
		}
		if(teigikinshi >= 0){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"ł͒`ł܂B");
			return false;
		}
		cst = type.GetConst()[0] == CONST_TOKEN;
		funcfg = false;
		kmfs = 0;
		if(IsSoushokuToken((TokenKind)type.GetConst()[kmfs])) kmfs++;
		switch(type.GetConst()[kmfs]){
		case REF_TOKEN:
			if(root == this){
				if(returnshita && IsKoukaiHensuuToken((TokenKind)type.GetConst()[0])){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O[oreturnoAJO[oϐ`邱Ƃ͂ł܂B");
					return false;
				}
			}
			if(kmfs == type.GetLength()-1){
				if(srce->GetToken(t) != COLON_TOKEN){
					if(cst){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"G[łB");
						return false;
					}
					srce->idx = t;
					type += NULL_TOKEN;
				}else{
					kin = srce->GetToken(t);
					if(kin == CONST_TOKEN){
						type += CONST_TOKEN;
						if(!TypeNanika(type, *srce, false, false, true, *this, true)) return false;
					}else if(kin == FUNC_TOKEN || kin == METHOD_TOKEN || kin == DELEGATE_TOKEN){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"֐|C^͒newł܂B");
						return false;
					}else{
						srce->idx = t;
						if(!TypeNanika(type, *srce, false, false, true, *this, true)) return false;
					}
				}
				if(RetTenCheck(type)){
					flu = true;
					goto FOOLABEL;
				}
			}
			if(flu = KansuuTeigika(t)) goto FOOLABEL;
			while(true){
				if(!RefTouroku(type, t, &tk)) return false;
				oi = srce->idx;
				ot = t;
				ogy = srce->gyousuu;
				switch(srce->GetToken(t)){
				case DAINYUU_TOKEN:
					if(cst){
						if(!ConstShiki(*srce, t, NULL_TOKEN, srce->GetToken(t), last, SEMICOLON_TOKEN, false)) return false;
						if(!ConstCaster(last, type)){
							stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"^G[łB");
							return false;
						}
						frp->funchensuu.Tail().type = last;
						frp->funchensuu.Tail().size = 0;
						if(srce->source.GetConst()[t] == L',') continue;
					}else if(selftype == CLASS_BLOCK){
						if(!ClassnoTeisuuDainyuu(type, t)) return false;
						if(srce->source.GetConst()[t] == L',') continue;
					}else{
						srce->idx = oi;
						t = ot;
						srce->gyousuu = ogy;
						if(!Shiki(tk, *srce, t, NULL_TOKEN, OTHERKIND_TOKEN, SEMICOLON_TOKEN)) return false;
						tk += SEMICOLON_TOKEN;
						if(srce->source.GetConst()[t] == L',') continue;
					}
					break;
				case COMMA_TOKEN:
					if(cst){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"萔̒lw肳Ă܂B");
						return false;
					}
					continue;
				case SEMICOLON_TOKEN:
					if(cst){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"萔̒lw肳Ă܂B");
						return false;
					}
					break;
				default:
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
					return false;
				}
				break;
			}
			return true;
		case LIST_TOKEN:
			if(root == this){
				if(returnshita && IsKoukaiHensuuToken((TokenKind)type.GetConst()[0])){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O[oreturnoAJO[oϐ`邱Ƃ͂ł܂B");
					return false;
				}
			}
			if(kmfs == type.GetLength()-1){
				if(srce->GetToken(t) != COLON_TOKEN){
					srce->idx = t;
					type += NULL_TOKEN;
				}else{
					kin = srce->GetToken(t);
					if(kin == CONST_TOKEN){
						type += CONST_TOKEN;
						if(!TypeNanika(type, *srce, false, false, true, *this, true)) return false;
					}else if(kin == FUNC_TOKEN || kin == METHOD_TOKEN || kin == DELEGATE_TOKEN){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"֐|C^͒newł܂B");
						return false;
					}else{
						srce->idx = t;
						if(!TypeNanika(type, *srce, false, false, true, *this, true)) return false;
					}
				}
				if(RetTenCheck(type)){
					flu = true;
					goto FOOLABEL;
				}
			}
			if(flu = KansuuTeigika(t)) goto FOOLABEL;
			while(true){
				if(!ListTouroku(type, t, &tk)) return false;
				oi = srce->idx;
				ot = t;
				ogy = srce->gyousuu;
				switch(srce->GetToken(t)){
				case DAINYUU_TOKEN:
					if(!ClassnaiKinsoku()) return false;
					srce->idx = oi;
					t = ot;
					srce->gyousuu = ogy;
					if(!Shiki(tk, *srce, t, NULL_TOKEN, OTHERKIND_TOKEN, SEMICOLON_TOKEN)) return false;
					tk += SEMICOLON_TOKEN;
					if(srce->source.GetConst()[t] == L',') continue;
					break;
				case COMMA_TOKEN:
					continue;
				case SEMICOLON_TOKEN:
					break;
				default:
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
					return false;
				}
				break;
			}
			return true;
		case THREAD_TOKEN:
			if(root == this){
				if(returnshita && IsKoukaiHensuuToken((TokenKind)type.GetConst()[0])){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O[oreturnoAJO[oϐ`邱Ƃ͂ł܂B");
					return false;
				}
			}
			if(kmfs == type.GetLength()-1){
				if(srce->GetToken(t) != BRACOLOOPEN_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'<:' ܂B");
					return false;
				}
				if((ci = PathtoClassID(*srce, FUNC_BLOCK, false, false, *this, true)) < 0) return false;
				if(stat->funclist.Get(ci)->parent != stat->funclist.Get(ci)->root
					|| (srce->source.GetConst()[srce->idx-1] == L'>' && srce->source.GetConst()[srce->idx-2] != L':'))
				{
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"̓G[łB");
					return false;
				}
				type += ci;
				if(srce->GetToken(t) != BRACOLOCLOSE_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"':>' ܂B");
					return false;
				}
			}
			while(true){
				if(!ThreadTouroku(type, &tk)) return false;
				switch(srce->GetToken(t)){
				case COMMA_TOKEN:
					continue;
				case SEMICOLON_TOKEN:
					break;
				default:
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
					return false;
				}
				break;
			}
			return true;
		case FUNC_TOKEN:
			if(kmfs == type.GetLength()-1 && !FuncType(type, *srce, true)) return false;
			funcfg = true;
			break;
		case METHOD_TOKEN:
			if(kmfs < type.GetLength()-1){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\bhselfŒ`ł܂B");
				return false;
			}
			if(frp == root){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\bh̓O[oɒ`ł܂B");
				return false;
			}
			type += (frp->touka ? frp->parent->funcid : frp->funcid);
			if(!FuncType(type, *srce, true)) return false;
			funcfg = true;
			break;
		case DELEGATE_TOKEN:
			if(selftype == CLASS_BLOCK){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"NXodelegate^͒`ł܂B");
				return false;
			}
			if(frp == root){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O[odelegate^͒`ł܂B");
				return false;
			}
			if(kmfs == type.GetLength()-1 && !FuncType(type, *srce, true)) return false;
			funcfg = true;
			break;
		case PLUGIN_TOKEN:
			type = PLUGIN_TOKEN;
			if(!PluginType(type, name)) return false;
			if(srce->GetToken(t) != DAINYUU_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"vOC̃t@Cw肳Ă܂B");
				return false;
			}
			if(!LibPath(*srce, path)) return false;
			if(srce->GetToken(t) != SEMICOLON_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
				return false;
			}
			name += L'\0';
			type += (intptr_t)stat->dlllist[(_TCHAR*)path.GetConst()].getfunc((_TCHAR*)name.GetConst());
			name.Pop();
			if(type.GetTail() == NULL){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"vOCǂݍ߂܂łB");
				return false;
			}
			if(!AddHensuu(type, name, 0)){
				tmp = L'\'';
				tmp += name;
				tmp += L"' ͖OdĂ邩AsłB";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
				return false;
			}
			return true;
		case ENUM_TOKEN:
			if(kmfs < type.GetLength()-1){
				ci = (int32_t)type.GetConst()[++kmfs];
			}else{
				if((ci = PathtoClassID(*srce, ENUM_BLOCK, false, false, *this, true)) < 0) return false;
				type += ci;
			}
			oi = srce->idx;
			ogy = srce->gyousuu;
			if(!stat->funclist.Get(ci)->kakutei){
				if(frp != this){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"enum̓TuubNŒ`ł܂B");
					return false;
				}
				if(IsSoushokuToken((TokenKind)type.GetConst()[0])){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"enumɑq͂܂B");
					return false;
				}
				pst = stat->funclist.Get(ci);
				pst->rettype = type;
				pst->glbconstant = true;
				if(srce->GetToken(t) != CHUUKAKKOOPEN_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'{' ܂B");
					return false;
				}
				t32 = 0;
				type = CONST_TOKEN;
				type += ENUM_TOKEN;
				type += ci;
				type.SetLength(type.GetLength()+(sizeof(int32_t)+sizeof(intptr_t)-1)/sizeof(intptr_t));
				inty = CONST_TOKEN;
				inty += INT_TOKEN;
				kin = srce->GetToken(t);
				while(kin != CHUUKAKKOCLOSE_TOKEN){
					if(kin != OTHERKIND_TOKEN){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
						return false;
					}
					name.Clear();
					name.AddTail(srce->source.GetConst()+t, srce->idx-t);
					kin = srce->GetToken(t);
					if(kin == DAINYUU_TOKEN){
						if(!ConstShiki(*srce, t, NULL_TOKEN, srce->GetToken(t), last, CHUUKAKKOCLOSE_TOKEN, false)) return false;
						if(!ConstCaster(last, inty)){
							stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"^G[łB");
							return false;
						}
						t32 = *(int32_t *)(last.GetConst()+2);
						srce->idx = t;
						kin = srce->GetToken(t);
					}
					switch(kin){
					case COMMA_TOKEN:
						kin = srce->GetToken(t);
						break;
					case OTHERKIND_TOKEN:
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
						return false;
					}
					*(int32_t *)(type.GetConst()+3) = t32++;
					if(!pst->AddHensuu(type, name, 0)){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ͖OdĂ邩AsłB";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
						return false;
					}
				}
				pst->selftype = ENUM_BLOCK;
				pst->kakutei = true;
				return true;
			}
			break;
		case CLASS_TOKEN:
			if(kmfs < type.GetLength()-1){
				ci = (int32_t)type.GetConst()[++kmfs];
				type.SetLength(kmfs);
			}else{
				if((ci = PathtoClassID(*srce, CLASS_BLOCK, false, false, *this, true)) < 0) return false;
			}
			oi = srce->idx;
			ogy = srce->gyousuu;
			if(!stat->funclist.Get(ci)->kakutei){
				if(frp != this){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"TuubN̒ł̓NX`ł܂B");
					return false;
				}
				if(type.GetConst()[0] == PROPERTY_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"propertyŃNX`ł܂B");
					return false;
				}
				if(type.GetConst()[0] == CORE_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"coreŃNX`ł܂B");
					return false;
				}
				pst = stat->funclist.Get(ci);
				if(srce->source.GetConst()[srce->idx-1] == L'>' && srce->source.GetConst()[srce->idx-2] == L':'){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"̓G[łB");
					return false;
				}
				if(pst->rettype.GetLength() > 0){
					if(pst->rettype.GetLength() >= 2 || !IsPublicToken((TokenKind)pst->rettype.GetConst()[0])){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"NX̒`ŎȂĒ`悤Ƃ܂B");
						return false;
					}else{
						if(!IsPublicToken((TokenKind)type.GetConst()[0])){
							stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"łɃpubNƂĎgĂ܂B");
							return false;
						}
					}
				}
				pst->rettype = type;
				if(cst) pst->glbconstant = true;
				if(srce->source.GetConst()[srce->idx-1] == L'>'){
					if((kin = srce->GetToken(t)) != CHUUKAKKOOPEN_TOKEN){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'{' ܂B");
						return false;
					}
					pst->topsrcidx = srce->idx;
					pst->topsrcgyo = srce->gyousuu;
					for(kmfs = 1; kmfs != 0 && kin != NULL_TOKEN; ){
						kin = srce->GetToken(t);
						switch(kin){
						case CHUUKAKKOOPEN_TOKEN:
							kmfs++;
							break;
						case CHUUKAKKOCLOSE_TOKEN:
							kmfs--;
							break;
						case DQUOT_TOKEN:
							srce->Find(L'"', t);
							srce->idx++;
							break;
						case SQUOT_TOKEN:
							srce->Find(L'\'', t);
							srce->idx++;
							break;
						}
					}
					if(kin == NULL_TOKEN){
						stat->ss->ErrorMessage.Add(srce->filename, pst->topsrcgyo, L"'}' ܂B");
						return false;
					}
					if(pst->children.GetLength() > 0){
						if(!pst->BackTemplate(pst->tpcount)) return false;
					}else{
						pst->kakutei = true;
					}
				}else{
					if(srce->GetToken(t) != CHUUKAKKOOPEN_TOKEN){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"`ĂȂNXgƂA '{' ܂B");
						return false;
					}
					if(!pst->MakeTree()) return false;
					if(!pst->ClassShokika(*srce)) return false;
				}
				size = pst->GetMemberSize();
				for(kmfs = 0; kmfs < pst->backpatch.GetLength(); kmfs++){
					((SourceTree *)pst->backpatch.GetConst()[kmfs])->funchensuu.GetData()[pst->breakpatch.GetConst()[kmfs]].size = size;
				}
				pst->backpatch.Clear();
				pst->breakpatch.Clear();
			}else{
				if(root == this){
					if(returnshita && IsKoukaiHensuuToken((TokenKind)type.GetConst()[0])){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O[oreturnoAJO[oϐ`邱Ƃ͂ł܂B");
						return false;
					}
				}
				if(cst){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"萔̃NXIuWFNg܂B");
					return false;
				}
				if(srce->source.GetConst()[srce->idx-1] == L'>' && srce->source.GetConst()[srce->idx-2] != L':'){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"̓G[łB");
					return false;
				}
				type += ci;
				while(true){
					if(!ClassObjToroku(type, ci, &tk)) return false;
					switch(srce->GetToken(t)){
					case COMMA_TOKEN:
						continue;
					case SEMICOLON_TOKEN:
						break;
					default:
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
						return false;
					}
					break;
				}
			}
			return true;
		case TYPE_TOKEN:
			while(true){
				if(srce->GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				name.Clear();
				name.AddTail(srce->source.GetConst()+t, srce->idx-t);
				if(srce->GetToken(t) != DAINYUU_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"^w肵Ă܂B");
					return false;
				}
				if(!TypeNanika(type, *srce, false, false, false, *this, true)) return false;
				switch(srce->GetToken(t)){
				case COMMA_TOKEN:
					if(!AddHensuu(type, name, 0)){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ͖OdĂ邩AsłB";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
						return false;
					}
					type.SetLength(1);
					continue;
				case SEMICOLON_TOKEN:
					if(!AddHensuu(type, name, 0)){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ͖OdĂ邩AsłB";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
						return false;
					}
					break;
				default:
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
					return false;
				}
				break;
			}
			return true;
		case LIB_TOKEN:
			while(true){
				if(srce->GetToken(t) != OTHERKIND_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				name.Clear();
				name.AddTail(srce->source.GetConst()+t, srce->idx-t);
				if(srce->GetToken(t) != DAINYUU_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"Cuw肵Ă܂B");
					return false;
				}
				if(!LibNanika(type, *srce, *this)) return false;
				switch(srce->GetToken(t)){
				case COMMA_TOKEN:
					if(!AddHensuu(type, name, 0)){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ͖OdĂ邩AsłB";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
						return false;
					}
					type.SetLength(1);
					continue;
				case SEMICOLON_TOKEN:
					if(!AddHensuu(type, name, 0)){
						tmp = L'\'';
						tmp += name;
						tmp += L"' ͖OdĂ邩AsłB";
						tmp += L'\0';
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
						return false;
					}
					break;
				default:
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
					return false;
				}
				break;
			}
			return true;
		case TENTEN_TOKEN:
			if(srce->GetToken(t) != OTHERKIND_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			name.Clear();
			name.AddTail(srce->source.GetConst()+t, srce->idx-t);
			if(name.GetLength() < 2 || name.GetConst()[name.GetLength()-2] != L'_' || name.GetConst()[name.GetLength()-1] != L't'){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O̍Ōオ '_t' ł܂B");
				return false;
			}
			kmfs++;
			type.SetLength(kmfs+((name.GetLength()+1)*sizeof(wchar_t)+sizeof(intptr_t)-1) / sizeof(intptr_t));
			type.Tail() = 0;
			memcpy(type.GetData()+kmfs, name.GetConst(), name.GetLength()*sizeof(wchar_t));
			flu = true;
			goto FOOLABEL;
		}
		if((size = GetTypeSize(type)) < 0){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"G[łB");
			return false;
		}
		flu = KansuuTeigika(t);
FOOLABEL:
		if(flu){
			if(frp != this){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"TuubN̒ł͊֐`ł܂B");
				return false;
			}
			if((ci = PathtoClassID(*srce, FUNC_BLOCK, false, false, *this, true)) < 0) return false;
			pst = stat->funclist.Get(ci);
			if(pst->kakutei){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"łɒ`Ă܂B");
				return false;
			}
			if(srce->source.GetConst()[srce->idx-1] == L'>' && srce->source.GetConst()[srce->idx-2] == L':'){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"̓G[łB");
				return false;
			}
			if(pst->rettype.GetLength() > 0){
				if(pst->rettype.GetLength() >= 2 || !IsPublicToken((TokenKind)pst->rettype.GetConst()[0])){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"֐̒`ŎȂĒ`悤Ƃ܂B");
					return false;
				}else{
					if(!IsPublicToken((TokenKind)type.GetConst()[0])){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"łɃpubNƂĎgĂ܂B");
						return false;
					}
				}
			}
			if(pst->parent == pst->root){
				if(type.GetConst()[0] == PROPERTY_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O[oproperty֐͒`ł܂B");
					return false;
				}else if(pst->root->returnshita && IsPublicToken((TokenKind)type.GetConst()[0])){
					for(kmfs = 0; kmfs < pst->root->children.GetLength(); kmfs++){
						if(pst->root->children.GetConst()[kmfs] == pst) break;
					}
					if(kmfs < pst->root->children.GetLength()){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O[oreturnoAconstłȂJO[o֐`邱Ƃ͂ł܂B");
						return false;
					}
				}
			}
			pst->srce = srce;
			pst->rettype = type;
			if(cst) pst->glbconstant = true;
			if(srce->source.GetConst()[srce->idx-1] == L'>' && srce->source.GetConst()[srce->idx-2] != L':'){
				pst->topsrcidx = srce->idx;
				pst->topsrcgyo = srce->gyousuu;
				if(srce->GetToken(t) != SHOUKAKKOOPEN_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'(' ܂B");
					return false;
				}
				kmfs = 1;
				do{
					kin = srce->GetToken(t);
					switch(kin){
					case SHOUKAKKOOPEN_TOKEN:
						kmfs++;
						break;
					case SHOUKAKKOCLOSE_TOKEN:
						kmfs--;
						break;
					case CHUUKAKKOOPEN_TOKEN:
					case CHUUKAKKOCLOSE_TOKEN:
					case SEMICOLON_TOKEN:
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"')' ܂B");
						return false;
					}
				}while(kmfs != 0 && kin != NULL_TOKEN);
				if(kin == NULL_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"')' ܂B");
					return false;
				}
				if(srce->GetToken(t) != CHUUKAKKOOPEN_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'{' ܂B");
					return false;
				}
				for(kmfs = 1; kmfs != 0 && kin != NULL_TOKEN; ){
					kin = srce->GetToken(t);
					switch(kin){
					case CHUUKAKKOOPEN_TOKEN:
						kmfs++;
						break;
					case CHUUKAKKOCLOSE_TOKEN:
						kmfs--;
						break;
					case DQUOT_TOKEN:
						srce->Find(L'"', t);
						srce->idx++;
						break;
					case SQUOT_TOKEN:
						srce->Find(L'\'', t);
						srce->idx++;
						break;
					}
				}
				if(kin == NULL_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, pst->topsrcgyo, L"'}' ܂B");
					return false;
				}
				if(pst->children.GetLength() > 0){
					if(!pst->BackTemplate(pst->tpcount)) return false;
				}else{
					pst->kakutei = true;
				}
			}else{
				if(!pst->KariHikisuu()) return false;
				if(!pst->MakeTree()) return false;
			}
			return true;
		}
		kin = srce->GetToken(t);
		oi = srce->idx;
		ot = t;
		ogy = srce->gyousuu;
		if(kin != OTHERKIND_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
			return false;
		}
		if(root == this){
			if(returnshita && IsKoukaiHensuuToken((TokenKind)type.GetConst()[0])){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"O[oreturnoAJO[oϐ`邱Ƃ͂ł܂B");
				return false;
			}
		}
		name.Clear();
		name.AddTail(srce->source.GetConst()+t, srce->idx-t);
		kin = srce->GetToken(t);
		switch(kin){
		case DAINYUU_TOKEN:
			if(cst){
				if(!ConstShiki(*srce, t, NULL_TOKEN, srce->GetToken(t), last, SEMICOLON_TOKEN, false)) return false;
				if(!ConstCaster(last, type)){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"^G[łB");
					return false;
				}
				if(!AddHensuu(last, name, 0)){
					tmp = L'\'';
					tmp += name;
					tmp += L"' ͖OdĂ邩AsłB";
					tmp += L'\0';
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
					return false;
				}
				if(srce->source.GetConst()[t] == L','){
					if(!HenFuTouroku(tk, type)) return false;
				}
			}else if(selftype == CLASS_BLOCK){
				if(!AddHensuu(type, name, size)){
					tmp = L'\'';
					tmp += name;
					tmp += L"' ͖OdĂ邩AsłB";
					tmp += L'\0';
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
					return false;
				}
				if(!ClassnoTeisuuDainyuu(type, t)) return false;
				if(srce->source.GetConst()[t] == L','){
					if(!HenFuTouroku(tk, type)) return false;
				}
			}else{
				if(!AddHensuu(type, name, size)){
					tmp = L'\'';
					tmp += name;
					tmp += L"' ͖OdĂ邩AsłB";
					tmp += L'\0';
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
					return false;
				}
				srce->idx = oi;
				t = ot;
				srce->gyousuu = ogy;
				if(!Shiki(tk, *srce, t, NULL_TOKEN, OTHERKIND_TOKEN, SEMICOLON_TOKEN)) return false;
				tk += SEMICOLON_TOKEN;
				if(srce->source.GetConst()[t] == L','){
					if(!HenFuTouroku(tk, type)) return false;
				}
			}
			break;
		case COMMA_TOKEN:
			if(cst || funcfg){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"lw肳Ă܂B");
				return false;
			}
			if(!AddHensuu(type, name, size)){
				tmp = L'\'';
				tmp += name;
				tmp += L"' ͖OdĂ邩AsłB";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
				return false;
			}
			if(!HenFuTouroku(tk, type)) return false;
			break;
		case SEMICOLON_TOKEN:
			if(cst || funcfg){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"lw肳Ă܂B");
				return false;
			}
			if(!AddHensuu(type, name, size)){
				tmp = L'\'';
				tmp += name;
				tmp += L"' ͖OdĂ邩AsłB";
				tmp += L'\0';
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, tmp.GetConst());
				return false;
			}
			break;
		default:
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
			return false;
		}
		return true;
	}
	bool SetType(PWCArray<intptr_t> &type, const TokenKind oskin)
	{
		TokenKind kin;
		bool pubdsk, constdsk, propdsk, coredsk;
		intptr_t t;
		kin = oskin;
		pubdsk = constdsk = propdsk = coredsk = false;
		type.Clear();
		while(kin != NULL_TOKEN){
			type += kin;
			switch(kin){
			case PUBLIC_TOKEN:
			case CONST_TOKEN:
			case PROPERTY_TOKEN:
			case CORE_TOKEN:
				if(pubdsk || constdsk || propdsk || coredsk){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"qȏd˂邱Ƃ͂ł܂B");
					return false;
				}
				pubdsk = kin == PUBLIC_TOKEN;
				constdsk = kin == CONST_TOKEN;
				propdsk = kin == PROPERTY_TOKEN;
				coredsk = kin == CORE_TOKEN;
				kin = srce->GetToken(t);
				continue;
			case BOOL_TOKEN:
			case CHAR_TOKEN:
			case DOUBLE_TOKEN:
			case FLOAT_TOKEN:
			case BYTE_TOKEN:
			case INT_TOKEN:
			case LONG_TOKEN:
			case SHORT_TOKEN:
			case UBYTE_TOKEN:
			case UINT_TOKEN:
			case ULONG_TOKEN:
			case USHORT_TOKEN:
			case INDEX_TOKEN:
			case VOID_TOKEN:
			case ENUM_TOKEN:
			case REF_TOKEN:
			case FUNC_TOKEN:
			case TENTEN_TOKEN:
				if(coredsk){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"core̓NXłB");
					return false;
				}
				break;
			case CLASS_TOKEN:
				break;
			case LIST_TOKEN:
			case METHOD_TOKEN:
			case THREAD_TOKEN:
				if(constdsk){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"萔ɂȂ܂B");
					return false;
				}
				if(coredsk){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"core̓NXłB");
					return false;
				}
				break;
			case TYPE_TOKEN:
			case LIB_TOKEN:
			case PLUGIN_TOKEN:
			case DELEGATE_TOKEN:
				if(pubdsk || constdsk || propdsk || coredsk){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"q͂܂B");
					return false;
				}
				break;
			default:
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			break;
		}
		if(kin == NULL_TOKEN){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"G[łB");
			return false;
		}
		return true;
	}
	bool Teigi(PWCArray<intptr_t> &tk, const TokenKind oskin)
	{
		PWCArray<intptr_t> type;
		if(!SetType(type, oskin)) return false;
		if(!HenFuTouroku(tk, type)) return false;
		return true;
	}
	Hensuu<SourceTree>* GetHensuu(const PWCArray<wchar_t> &name, intptr_t &iii, bool cst)
	{
		SourceTree *pst;
		pst = this;
		intptr_t ii;
		while(true){
			for(ii = 0; ii < (!pst->breakin||pst->breaknaihani<0 ? pst->hensuname.GetLength() : breaknaihani); ii++){
				if(name == pst->hensuname.GetConst()[ii]){
					iii = pst->hensuidx.GetConst()[ii];
					if(cst && frp->funchensuu.GetConst()[iii].type.GetConst()[0] != CONST_TOKEN) return NULL;
					return frp->funchensuu.GetData() + iii;
				}
			}
			if(pst == frp) break;
			ii = 0;
			pst = pst->parent;
		}
		return NULL;
	}
	bool TeigitoIfKinshi()
	{
		TokenKind ki;
		intptr_t t, oi, ot, ogy;
		PWCArray<intptr_t> type;
		switch(srce->GetToken(t)){
		case CONST_TOKEN:
		case PUBLIC_TOKEN:
		case PROPERTY_TOKEN:
		case CORE_TOKEN:
		case BOOL_TOKEN:
		case BYTE_TOKEN:
		case CHAR_TOKEN:
		case CLASS_TOKEN:
		case DOUBLE_TOKEN:
		case ENUM_TOKEN:
		case FLOAT_TOKEN:
		case INT_TOKEN:
		case LONG_TOKEN:
		case LIST_TOKEN:
		case REF_TOKEN:
		case SHORT_TOKEN:
		case TYPE_TOKEN:
		case UBYTE_TOKEN:
		case UINT_TOKEN:
		case ULONG_TOKEN:
		case USHORT_TOKEN:
		case INDEX_TOKEN:
		case VOID_TOKEN:
		case LIB_TOKEN:
		case FUNC_TOKEN:
		case PLUGIN_TOKEN:
		case METHOD_TOKEN:
		case THREAD_TOKEN:
		case DELEGATE_TOKEN:
		case NEW_TOKEN:
		case DELETE_TOKEN:
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"Œ`͂ł܂B");
			return false;
		case CHUUKAKKOCLOSE_TOKEN:
		case NULL_TOKEN:
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"G[łB");
			return false;
		case IF_TOKEN:
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"if͏܂B");
			return false;
		case CASE_TOKEN:
		case COND_TOKEN:
		case DIFF_TOKEN:
		case COMM_TOKEN:
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"Ƀx͏܂B");
			return false;
		case BREAK_TOKEN:
		case WHILE_TOKEN:
		case DEFAULT_TOKEN:
		case DO_TOKEN:
		case ELSE_TOKEN:
		case CONTINUE_TOKEN:
			oi = srce->idx;
			ot = t;
			ogy = srce->gyousuu;
			ki = srce->GetToken(t);
			if(ki != SEMICOLON_TOKEN && ki != COMMA_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"Ƀx͏܂B");
				return false;
			}
			srce->idx = oi;
			t = ot;
			srce->gyousuu = ogy;
			break;
		case MBR_TOKEN:
		case TILDE_TOKEN:
		case COLOCOLO_TOKEN:
		case TENTEN_TOKEN:
		case OTHERKIND_TOKEN:
			oi = srce->idx;
			ot = t;
			ogy = srce->gyousuu;
			srce->idx = t;
			type.Clear();
			if(TypeNanika(type, *srce, false, false, false, *this, true)){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"Œ`͂ł܂B");
				return false;
			}
			if(stat->tokubetuerr) return false;
			stat->ss->ErrorMessage.Clear();
			srce->idx = oi;
			t = ot;
			srce->gyousuu = ogy;
			break;
		}
		srce->idx = t;
		return true;
	}
	bool BlockOpenKinshi()
	{
		intptr_t t;
		switch(srce->GetToken(t)){
		case CHUUKAKKOOPEN_TOKEN:
		case SWITCH_TOKEN:
		case BRANCH_TOKEN:
		case LOOP_TOKEN:
		case LOCK_TOKEN:
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"ɃubN͏܂B");
			return false;
		}
		srce->idx = t;
		return true;
	}
	bool ClassnaiKinsoku()
	{
		if(selftype == CLASS_BLOCK){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"NXɏ̂͒`łB");
			return false;
		}
		return true;
	}
	bool KoubunKanouJoutai(const BlockType bt)
	{
		if((selftype != bt && bt != NORMAL_BLOCK) || crp == this || breakin){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"ł͂ł܂B");
			return false;
		}
		return true;
	}
	bool Bun(PWCArray<intptr_t> &tk, intptr_t &t, const TokenKind kin)
	{
		if(!Shiki(tk, *srce, t, NULL_TOKEN, kin, SEMICOLON_TOKEN)) return false;
		if(srce->source.GetConst()[t] == L','){
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"',' słB");
			return false;
		}
		return true;
	}
	PWCArray<intptr_t>* Indo()
	{
		if(teigikinshi < 0){
			ptk = &token[token.GetLength()];
			gyo.SetLength(token.GetLength());
			*ptk = token.GetConst()[token.GetLength()-2];
			gyo.Tail() = gyo.GetConst()[gyo.GetLength()-2];
			if(token.GetLength() >= 3 && token.GetConst()[token.GetLength()-3].GetConst()[0] == IF_TOKEN){
				token.GetData()[token.GetLength()-2] = token.GetConst()[token.GetLength()-3];
				gyo.GetData()[gyo.GetLength()-2] = gyo.GetConst()[gyo.GetLength()-3];
				token.GetData()[token.GetLength()-3].SetLength(0);
				teigikinshi = token.GetLength()-3;
			}else{
				token.GetData()[token.GetLength()-2].SetLength(0);
				teigikinshi = token.GetLength()-2;
			}
		}
		return ptk;
	}
	bool MakeTreeSwitch(PWCArray<intptr_t> *ptok, intptr_t &t, const TokenKind kin)
	{
		PWCArray<intptr_t> type;
		intptr_t oi, ot, ogy, ii;
		TokenKind ki;
		SourceTree *pst;
		int32_t fi, ipr;
		bool temp;
		PWCArray<wchar_t> name;
		switch(kin){
		case CONST_TOKEN:
		case PUBLIC_TOKEN:
		case PROPERTY_TOKEN:
		case CORE_TOKEN:
			oi = srce->idx;
			ot = t;
			ogy = srce->gyousuu;
			type = kin;
			if(TypeNanika(type, *srce, kin == CONST_TOKEN, kin == CORE_TOKEN, false, *this, true) && type.GetConst()[1] != NULL_TOKEN){
				if(!HenFuTouroku(*ptok, type)) return false;
				break;
			}
			if(stat->tokubetuerr) return false;
			stat->ss->ErrorMessage.Clear();
			srce->idx = oi;
			t = ot;
			srce->gyousuu = ogy;
		case BOOL_TOKEN:
		case BYTE_TOKEN:
		case CHAR_TOKEN:
		case CLASS_TOKEN:
		case DOUBLE_TOKEN:
		case ENUM_TOKEN:
		case FLOAT_TOKEN:
		case INT_TOKEN:
		case LONG_TOKEN:
		case LIST_TOKEN:
		case REF_TOKEN:
		case SHORT_TOKEN:
		case TYPE_TOKEN:
		case UBYTE_TOKEN:
		case UINT_TOKEN:
		case ULONG_TOKEN:
		case USHORT_TOKEN:
		case INDEX_TOKEN:
		case VOID_TOKEN:
		case LIB_TOKEN:
		case FUNC_TOKEN:
		case PLUGIN_TOKEN:
		case METHOD_TOKEN:
		case THREAD_TOKEN:
		case DELEGATE_TOKEN:
		case TENTEN_TOKEN:
			if(!Teigi(*ptok, kin)) return false;
			break;
		case NEW_TOKEN:
		case DELETE_TOKEN:
			if(selftype != CLASS_BLOCK){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"NXłȂƂ܂B");
				return false;
			}
			name.Clear();
			name.AddTail(srce->source.GetConst()+t, srce->idx-t);
			if((fi = GetFuncId(name, true)) < 0){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"svcȃG[łB");
				return false;
			}
			if(stat->funclist.Get(fi)->mbrhani != INTPTR_MAX){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"łɒ`Ă܂B");
				return false;
			}
			stat->funclist.Get(fi)->mbrhani = stat->funclist.Get(fi)->parent->hensuidx.GetLength();
			stat->funclist.Get(fi)->selftype = FUNC_BLOCK;
			stat->funclist.Get(fi)->rettype = VOID_TOKEN;
			if(srce->GetToken(t) != SHOUKAKKOOPEN_TOKEN
				|| srce->GetToken(t) != SHOUKAKKOCLOSE_TOKEN || srce->GetToken(t) != CHUUKAKKOOPEN_TOKEN)
			{
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			if(srce->GetToken(t) == COLOYAJIRUSHI_TOKEN){
				stat->funclist.Get(fi)->touka = true;
			}else{
				srce->idx = t;
			}
			stat->funclist.Get(fi)->kakutei = false;
			if(!stat->funclist.Get(fi)->MakeTree()) return false;
			break;
		case OTHERKIND_TOKEN:
		case MBR_TOKEN:
		case TILDE_TOKEN:
			oi = srce->idx;
			ot = t;
			ogy = srce->gyousuu;
			srce->idx = t;
			type.Clear();
			if(TypeNanika(type, *srce, false, false, false, *this, true) && type.GetConst()[0] != NULL_TOKEN){
				if(!HenFuTouroku(*ptok, type)) return false;
				break;
			}
			if(stat->tokubetuerr) return false;
			stat->ss->ErrorMessage.Clear();
			srce->idx = oi;
			t = ot;
			srce->gyousuu = ogy;
		case PULUS_TOKEN:
		case MINUS_TOKEN:
		case EXCLAMATION_TOKEN:
		case PLSPLS_TOKEN:
		case MNSMNS_TOKEN:
		case SHOUKAKKOOPEN_TOKEN:
		case DAIKAKKOOPEN_TOKEN:
		case SQUOT_TOKEN:
		case DQUOT_TOKEN:
		case FALSE_TOKEN:
		case TRUE_TOKEN:
		case SELF_TOKEN:
		case WAIT_TOKEN:
		case LEN_TOKEN:
		case IDNAME_TOKEN:
		case CONSTEVAL_TOKEN:
			if(!ClassnaiKinsoku()) return false;
			if(!Bun(*ptok, t, kin)) return false;
			*ptok += SEMICOLON_TOKEN;
			break;
		case RETURN_TOKEN:
			if(!ClassnaiKinsoku()) return false;
			if(breakinnest){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"breakłreturnł܂B");
				return false;
			}
			temp = frp->hikisuucount > 0 && frp->hensuname.GetConst()[frp->tpcount].GetConst()[0] == L'0';
			if(temp){
				*ptok += ~frp->tpcount;
				*ptok += NULL_TOKEN;
				*ptok += DAINYUU_TOKEN;
				ii = ptok->GetLength();
			}
			if(!Bun(*ptok, t, srce->GetToken(t))) return false;
			if(temp){
				if(ptok->GetLength() == ii){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"Ԃl܂B");
					return false;
				}
				*ptok += DAINYUU_TOKEN;
				*ptok += SEMICOLON_TOKEN;
			}
			*ptok += NULL_TOKEN;
			*ptok += RETURN_TOKEN;
			pst = this;
			while(pst != pst->crp){
				*ptok += BREAK_TOKEN;
				breaknaihani = pst->hensuname.GetLength();
				for(ii = pst->hensuidx.GetLength()-1; ii >= 0; ii--){
					if(frp->funchensuu.GetConst()[pst->hensuidx.GetConst()[ii]].type.GetConst()[0] == CONST_TOKEN) continue;
					fi = TokenTtoTypeId(frp->funchensuu.GetConst()[pst->hensuidx.GetConst()[ii]].type.GetConst(), oi=0);
					if(fi > ENDCLASS_TYPEID) continue;
					if(fi < 0){
						if(!(DYNLIST_TYPEID <= fi && fi <= REF_TYPEID) && fi < ENDTHREAD_TYPEID) continue;
						if(frp == root) *ptok += TILDE_TOKEN;
						*ptok += ~pst->hensuidx.GetConst()[ii];
						*ptok += DELETE_TOKEN;
						*ptok += SEMICOLON_TOKEN;
					}else if(stat->funclist.Get(fi)->selftype == CLASS_BLOCK){
						if(frp == root){
							type = TILDE_TOKEN;
						}else{
							type.Clear();
						}
						type += ~pst->hensuidx.GetConst()[ii];
						type += POINT_TOKEN;
						stat->funclist.Get(fi)->CallDel(*ptok, type);
					}
				}
				*ptok += COMMA_TOKEN;
				pst->returnshita = true;
				pst = pst->parent;
			}
			for(ii = pst->hensuidx.GetLength()-1; ii >= 0; ii--){
				if(pst->hensuidx.GetConst()[ii] < pst->tpcount+pst->hikisuucount) break;
				if(frp->funchensuu.GetConst()[pst->hensuidx.GetConst()[ii]].type.GetConst()[0] == CONST_TOKEN) continue;
				fi = TokenTtoTypeId(frp->funchensuu.GetConst()[pst->hensuidx.GetConst()[ii]].type.GetConst(), oi=0);
				if(fi > ENDCLASS_TYPEID) continue;
				if(fi < 0){
					if(!(DYNLIST_TYPEID <= fi && fi <= REF_TYPEID) && fi < ENDTHREAD_TYPEID) continue;
					if(frp == root) *ptok += TILDE_TOKEN;
					*ptok += ~pst->hensuidx.GetConst()[ii];
					*ptok += DELETE_TOKEN;
					*ptok += SEMICOLON_TOKEN;
				}else if(stat->funclist.Get(fi)->selftype == CLASS_BLOCK){
					if(frp == root){
						type = TILDE_TOKEN;
					}else{
						type.Clear();
					}
					type += ~pst->hensuidx.GetConst()[ii];
					type += POINT_TOKEN;
					stat->funclist.Get(fi)->CallDel(*ptok, type);
				}
			}
			*ptok += RETURN_TOKEN;
			pst->returnshita = true;
			break;
		case BRANCH_TOKEN:
		case LOOP_TOKEN:
			if(!ClassnaiKinsoku()) return false;
			if(srce->GetToken(t) != CHUUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'{' ܂B");
				return false;
			}
			*ptok += CHUUKAKKOOPEN_TOKEN;
			*ptok += children.GetLength();
			if((pst = BlockOpen(NULL, (kin==BRANCH_TOKEN ? BRANCH_BLOCK : LOOP_BLOCK), false, true)) == NULL){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"svcȃG[łB");
				return false;
			}
			if(!pst->MakeTree()) return false;
			*ptok += CHUUKAKKOCLOSE_TOKEN;
			break;
		case SWITCH_TOKEN:
		case LOCK_TOKEN:
			if(!ClassnaiKinsoku()) return false;
			*ptok += CHUUKAKKOOPEN_TOKEN;
			*ptok += children.GetLength();
			if((pst = BlockOpen(NULL, (kin==SWITCH_TOKEN ? SWITCH_BLOCK : LOCK_BLOCK), false, true)) == NULL){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"svcȃG[łB");
				return false;
			}
			if(srce->GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'(' ܂B");
				return false;
			}
			*ptok += SHOUKAKKOOPEN_TOKEN;
			if(!Shiki(*ptok, *srce, t, NULL_TOKEN, srce->GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
			*ptok += SHOUKAKKOCLOSE_TOKEN;
			if(srce->GetToken(t) != CHUUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'{' ܂B");
				return false;
			}
			if(!pst->MakeTree()) return false;
			*ptok += CHUUKAKKOCLOSE_TOKEN;
			break;
		case DO_TOKEN:
			if(!KoubunKanouJoutai(LOOP_BLOCK)) return false;
			if(ptok == ptk){
				ptok = Indo();
			}else{
				Indo();
			}
			*ptok += DO_TOKEN;
			ki = srce->GetToken(t);
			switch(ki){
			case COLON_TOKEN:
				if(extloop->mode != 0){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				extloop->mode = 1;
				break;
			case SEMICOLON_TOKEN:
				break;
			default:
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += ki;
			break;
		case CONTINUE_TOKEN:
			if(!KoubunKanouJoutai(LOOP_BLOCK)) return false;
			if(ptok == ptk){
				ptok = Indo();
			}else{
				Indo();
			}
			*ptok += CONTINUE_TOKEN;
			ki = srce->GetToken(t);
			switch(ki){
			case COLON_TOKEN:
				if(extloop->mode != 1){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				extloop->mode = 2;
				break;
			case SEMICOLON_TOKEN:
				break;
			default:
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += ki;
			break;
		case WHILE_TOKEN:
			if(!KoubunKanouJoutai(LOOP_BLOCK)) return false;
			if(ptok == ptk){
				ptok = Indo();
			}else{
				Indo();
			}
			*ptok += WHILE_TOKEN;
			ki = srce->GetToken(t);
			switch(ki){
			case SEMICOLON_TOKEN:
				*ptok += SEMICOLON_TOKEN;
				break;
			default:
				if(extloop->mode < 1 || 2 < extloop->mode){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				extloop->mode = 3;
				if(!Shiki(*ptok, *srce, t, NULL_TOKEN, ki, COLON_TOKEN)) return false;
				if(srce->source.GetConst()[t] != L':'){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				*ptok += COLON_TOKEN;
				break;
			}
			break;
		case COND_TOKEN:
			if(!KoubunKanouJoutai(BRANCH_BLOCK)) return false;
			if(ptok == ptk){
				ptok = Indo();
			}else{
				Indo();
			}
			if((extbranch->mode < 0 ? -extbranch->mode : extbranch->mode) >= 2){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += COND_TOKEN;
			if(srce->GetToken(t) == SHOU_TOKEN){
				if(extbranch->mode > 0){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				if(srce->GetToken(t) != OTHERKIND_TOKEN || !extbranch->addlabel(srce->source.GetConst()+t, srce->idx-t)){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				if(srce->GetToken(t) != DAI_TOKEN){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'>' ܂B");
					return false;
				}
				*ptok += extbranch->label.GetLength()-1;
				extbranch->mode = -1;
			}else{
				if(extbranch->mode < 0){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				srce->idx = t;
				extbranch->mode = 1;
			}
			if(!Shiki(*ptok, *srce, t, NULL_TOKEN, srce->GetToken(t), COLON_TOKEN)) return false;
			if(srce->source.GetConst()[t] != L':'){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += COLON_TOKEN;
			break;
		case COMM_TOKEN:
			if(!KoubunKanouJoutai(BRANCH_BLOCK)) return false;
			if(extbranch->mode >= 3 || extbranch->mode == 0 || extbranch->mode == -4){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += COMM_TOKEN;
			extbranch->dino = 0;
			extbranch->mode = (extbranch->mode < 0 ? -4 : 4);
			if(srce->GetToken(t) != COLON_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"':' ܂B");
				return false;
			}
			*ptok += COLON_TOKEN;
			break;
		case DIFF_TOKEN:
			if(!KoubunKanouJoutai(BRANCH_BLOCK)) return false;
			if(extbranch->mode >= 0 || extbranch->mode == -2){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += DIFF_TOKEN;
			if(srce->GetToken(t) != SHOU_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'<' ܂B");
				return false;
			}
			if(srce->GetToken(t) != OTHERKIND_TOKEN || ((ipr=extbranch->findlabel(srce->source.GetConst()+t, srce->idx-t)) < 0)){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			if(srce->GetToken(t) != DAI_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'>' ܂B");
				return false;
			}
			if(ipr < extbranch->dino){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += ipr;
			extbranch->dino = ipr+1;
			extbranch->mode = -3;
			if(srce->GetToken(t) != COLON_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"':' ܂B");
				return false;
			}
			*ptok += COLON_TOKEN;
			break;
		case ELSE_TOKEN:
			if(!KoubunKanouJoutai(BRANCH_BLOCK)) return false;
			if(ptok == ptk){
				ptok = Indo();
			}else{
				Indo();
			}
			*ptok += ELSE_TOKEN;
			ki = srce->GetToken(t);
			switch(ki){
			case COLON_TOKEN:
				if(extbranch->mode % 2 == 0 && extbranch->mode != -4){
					stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
					return false;
				}
				if(extbranch->mode < 0){
					extbranch->mode = -2;
				}else{
					extbranch->mode = 2;
				}
				break;
			case SEMICOLON_TOKEN:
				break;
			default:
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += ki;
			break;
		case CASE_TOKEN:
			if(!KoubunKanouJoutai(SWITCH_BLOCK)) return false;
			if(ptok == ptk){
				ptok = Indo();
			}else{
				Indo();
			}
			*ptok += CASE_TOKEN;
			do{
				if(!ConstShiki(*srce, t, NULL_TOKEN, srce->GetToken(t), type, COLON_TOKEN, false)) return false;
				*ptok += type;
				*ptok += COMMA_TOKEN;
			}while(srce->source.GetConst()[t] == L',');
			ptok->Tail() = (srce->idx-t == 1 ? COLON_TOKEN : COLOYAJIRUSHI_TOKEN);
			break;
		case DEFAULT_TOKEN:
			if(!KoubunKanouJoutai(SWITCH_BLOCK)) return false;
			if(ptok == ptk){
				ptok = Indo();
			}else{
				Indo();
			}
			*ptok += DEFAULT_TOKEN;
			ki = srce->GetToken(t);
			if(ki != COLON_TOKEN && ki != COLOYAJIRUSHI_TOKEN && ki != SEMICOLON_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
			*ptok += ki;
			break;
		case IF_TOKEN:
			if(!ClassnaiKinsoku()) return false;
			*ptok += IF_TOKEN;
			if(srce->GetToken(t) != SHOUKAKKOOPEN_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'(' ܂B");
				return false;
			}
			*ptok += SHOUKAKKOOPEN_TOKEN;
			if(!Shiki(*ptok, *srce, t, NULL_TOKEN, srce->GetToken(t), SHOUKAKKOCLOSE_TOKEN)) return false;
			*ptok += SHOUKAKKOCLOSE_TOKEN;
			if(!TeigitoIfKinshi()) return false;
			break;
		case BREAK_TOKEN:
			if(!KoubunKanouJoutai(NORMAL_BLOCK)) return false;
			*ptok += BREAK_TOKEN;
			ki = srce->GetToken(t);
			if(ki == COLON_TOKEN){
				*ptok += COLON_TOKEN;
				breakin = breakinnest = true;
				if(ptok == ptk){
					ptok = Indo();
				}else{
					Indo();
				}
			}else{
				breaknaihani = hensuname.GetLength();
				for(ii = hensuidx.GetLength()-1; ii >= 0; ii--){
					if(frp->funchensuu.GetConst()[hensuidx.GetConst()[ii]].type.GetConst()[0] == CONST_TOKEN) continue;
					fi = TokenTtoTypeId(frp->funchensuu.GetConst()[hensuidx.GetConst()[ii]].type.GetConst(), oi=0);
					if(fi > ENDCLASS_TYPEID) continue;
					if(fi < 0){
						if(!(DYNLIST_TYPEID <= fi && fi <= REF_TYPEID) && fi < ENDTHREAD_TYPEID) continue;
						if(frp == root) *ptok += TILDE_TOKEN;
						*ptok += ~hensuidx.GetConst()[ii];
						*ptok += DELETE_TOKEN;
						*ptok += SEMICOLON_TOKEN;
					}else if(stat->funclist.Get(fi)->selftype == CLASS_BLOCK){
						if(frp == root){
							type = TILDE_TOKEN;
						}else{
							type.Clear();
						}
						type += ~hensuidx.GetConst()[ii];
						type += POINT_TOKEN;
						stat->funclist.Get(fi)->CallDel(*ptok, type);
					}
				}
				if(ki == COMMA_TOKEN){
					*ptok += COMMA_TOKEN;
					if(!BlockOpenKinshi()) return false;
					if(!TeigitoIfKinshi()) return false;
					ki = srce->GetToken(t);
					if(!parent->MakeTreeSwitch(ptok, t, ki)) return false;
					ptok = ptk;
				}else{
					if(ki != SEMICOLON_TOKEN){
						stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"';' ܂B");
						return false;
					}
					*ptok += SEMICOLON_TOKEN;
				}
			}
			break;
		case CHUUKAKKOOPEN_TOKEN:
			if(!ClassnaiKinsoku()) return false;
			*ptok += CHUUKAKKOOPEN_TOKEN;
			*ptok += children.GetLength();
			if((pst = BlockOpen(NULL, NORMAL_BLOCK, false, true)) == NULL){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"svcȃG[łB");
				return false;
			}
			if(!pst->MakeTree()) return false;
			*ptok += CHUUKAKKOCLOSE_TOKEN;
			break;
		default:
			name = L'\'';
			name.AddTail(srce->source.GetConst()+t, srce->idx-t);
			name += L"' słB";
			name += L'\0';
			stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, name.GetConst());
			return false;
		}
		return true;
	}
	bool MakeTree()
	{
		PWCArray<intptr_t> meniya;
		TokenKind kin;
		intptr_t t, il, ii;
		int32_t fi;
		topsrcidx = srce->idx;
		topsrcgyo = srce->gyousuu;
		switch(selftype){
		case SWITCH_BLOCK:
			extswitch = new SwitchExtStruct;
			break;
		case BRANCH_BLOCK:
			extbranch = new BranchExtStruct;
			break;
		case LOOP_BLOCK:
			extloop = new LoopExtStruct;
			break;
		}
		while((kin = srce->GetToken(t)) != NULL_TOKEN && kin != CHUUKAKKOCLOSE_TOKEN){
			ptk = &token[token.GetLength()];
			ptk->Clear();
			gyo += srce->gyousuu;
			if(!MakeTreeSwitch(ptk, t, kin)) return false;
			if(ptk->GetLength() == 0){
				token.Pop();
				gyo.Pop();
			}
		}
		if(root == this){
			if(kin != NULL_TOKEN || srce->idx != srce->source.GetLength()){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"\G[łB");
				return false;
			}
		}else{
			if(kin != CHUUKAKKOCLOSE_TOKEN){
				stat->ss->ErrorMessage.Add(srce->filename, srce->gyousuu, L"'}' ܂B");
				return false;
			}
		}
		if(crp != this || (selftype == FUNC_BLOCK || selftype == DELEGATE_BLOCK) && rettype.GetTail() == VOID_TOKEN){
			returnshita = true;
			for(ii = hensuidx.GetLength()-1; ii >= 0; ii--){
				if(hensuidx.GetConst()[ii] < tpcount+hikisuucount) break;
				if(frp->funchensuu.GetConst()[hensuidx.GetConst()[ii]].type.GetConst()[0] == CONST_TOKEN) continue;
				fi = TokenTtoTypeId(frp->funchensuu.GetConst()[hensuidx.GetConst()[ii]].type.GetConst(), il=0);
				if(fi > ENDCLASS_TYPEID) continue;
				if(fi < 0){
					if(!(DYNLIST_TYPEID <= fi && fi <= REF_TYPEID) && fi < ENDTHREAD_TYPEID) continue;
					if(frp == root) defdel += TILDE_TOKEN;
					defdel += ~hensuidx.GetConst()[ii];
					defdel += DELETE_TOKEN;
					defdel += SEMICOLON_TOKEN;
				}else if(stat->funclist.Get(fi)->selftype == CLASS_BLOCK){
					if(frp == root){
						meniya = TILDE_TOKEN;
					}else{
						meniya.Clear();
					}
					meniya += ~hensuidx.GetConst()[ii];
					meniya += POINT_TOKEN;
					stat->funclist.Get(fi)->CallDel(defdel, meniya);
				}
			}
		}
		kakutei = true;
		return true;
	}
	bool FileRead(const wchar_t *const fn)
	{
		if(!srce->Load(fn)) return false;
		srce->filename += L'\0';
		stat->srclist[(_TCHAR*)srce->filename.GetConst()].set(this);
		srce->filename.Pop();
		if(!MakeTree()) return false;
		return true;
	}
	void FlgReset()
	{
		breakin = false;
		teigikinshi = -1;
		returnshita = false;
		breaknaihani = -1;
		tpcount = 0;
		hikisuucount = 0;
		sonota.golbalgeta = 0;
		mbsize = -1;
		byteadress = 0;
		deladress = 0;
		topsrcidx = 0;
		topsrcgyo = 0;
		touka = false;
		gblhani = INTPTR_MAX;
		gblsaikouhani = 0;
		mbrhani = INTPTR_MAX;
		mbrsaikouhani = 0;
	}
	void ArrayReset()
	{
		myname.Clear();
		token.Clear();
		defdel.Clear();
		gyo.Clear();
		funchensuu.Clear();
		hensuname.Clear();
		hensuidx.Clear();
		subfunc.Clear();
		backpatch.Clear();
		breakpatch.Clear();
	}
	void Clear()
	{
		stat->tokubetuerr = false;
		Del();
		stat->funclist.Clear();
		stat->funclist.Add(this);
		stat->srclist.Clear();
		stat->dlllist.Clear();
		stat->literallist.Clear();
		stat->literaltotal = 0;
		ArrayReset();
		FlgReset();
	}
};
